#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#from copy import copy
import psp2d, functions

class card(object):
  def __init__(self, pCardSet):
    self.cardSet = pCardSet
    self.drawValues = pCardSet.drawValues
    self.font = pCardSet.font
    self.path = pCardSet.path
    self.imageBack = pCardSet.imageBack
    self.ID = 0
    self.name = ""
    self.cardSetName = ""
    self.count = 0
    self.owner = None
    self.level = 0
    self.imageP1 = None
    self.imageP2 = None
    self.buffImageP1 = None
    self.buffImageP2 = None
    self.top = 0
    self.bottom = 0
    self.right = 0
    self.left = 0
    self.delta = 0
    self.elemental = None
    self.elementalString = None
    self.status = None
    self.deckCell = None
    self.buyPrice = 0
    self.sellPrice = 0
    self.data = None

  def copy(self):
    """Returns a copy of the card"""
    #return copy(self)
    tCard = card(self.cardSet)
    tCard.drawValues = self.drawValues
    tCard.font = self.font
    tCard.path = self.path
    tCard.imageBack = psp2d.Image(self.imageBack)
    tCard.ID = self.ID
    tCard.name = self.name
    tCard.cardSetName = self.cardSetName
    tCard.count = self.count
    tCard.owner = self.owner
    tCard.level = self.level
    tCard.imageP1 = self.imageP1
    tCard.imageP2 = self.imageP2
    tCard.top = self.top
    tCard.bottom = self.bottom
    tCard.right = self.right
    tCard.left = self.left
    tCard.delta = self.delta
    tCard.elemental = self.elemental
    tCard.elementalString = self.elementalString
    tCard.status = self.status
    tCard.deckCell = self.deckCell
    tCard.buyPrice = self.buyPrice
    tCard.sellPrice = self.sellPrice
    tCard.data = self.data
    return tCard


  def value(self):
    """Calculates card value"""
    val = self.top + self.bottom + self.left + self.right
    if self.elemental:
      val += 2
    return val


  def stringValue(self):
    """Return a string with card's value"""
    def toStr(val):
      if val < 10:
        return str(val)
      elif val == 10:
        return "A"
      elif val > 10:
        return "A+"
    ret = self.name + " LV:" + str(self.level) + " (" + toStr(self.top) + "-" + toStr(self.right) + "-" + toStr(self.bottom) + "-" + toStr(self.left)
    if self.elemental != None:
      ret = ret + " " + self.elementalString
    ret = ret  + ")"
    return ret


  def writeValues(self, pImage):
    """Writes values on an image"""
    def toStr(val):
      if val < 10:
        return str(val)
      elif val == 10:
        return "A"
      elif val > 10:
        return "A+"

    if self.drawValues:
      self.font.drawText(pImage, 12, 2, toStr(self.top + self.delta))
      self.font.drawText(pImage, 22, 12, toStr(self.right + self.delta))
      self.font.drawText(pImage, 2, 12, toStr(self.left + self.delta))
      self.font.drawText(pImage, 12, 22, toStr(self.bottom + self.delta))

    if self.delta > 0:
      tWidth = functions.textWidth(self.font, str(self.delta))
      tHeight = functions.textHeight(self.font, "I")
      self.font.drawText(pImage, 37 - tWidth / 2, 37 - tHeight / 2, "+" + str(self.delta))
    elif self.delta < 0:
      tWidth = functions.textWidth(self.font, str(self.delta))
      tHeight = functions.textHeight(self.font, "I")
      self.font.drawText(pImage, 37 - tWidth / 2, 37 - tHeight / 2, str(self.delta))

    return pImage