#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
class deck(object):
  """Class deck"""
  def __init__(self):
    self.cells = [None] * 9
    self.elements = [None] * 9

  def getRightCell(self, pcell):
    """Return the cell to the right"""
    if pcell not in [2,5,8]:
      return self.cells[pcell + 1]
    else:
      return None


  def getLeftCell(self, pcell):
    """Return the cell to the left"""
    if pcell not in [0,3,6]:
      return self.cells[pcell - 1]
    else:
      return None


  def getTopCell(self, pcell):
    """Return the cell to the top"""
    if pcell not in [0,1,2]:
      return self.cells[pcell - 3]
    else:
      return None


  def getBottomCell(self, pcell):
    """Return the cell to the bottom"""
    if pcell not in [6,7,8]:
      return self.cells[pcell + 3]
    else:
      return None


  def getNearCells(self, pCell):
    """Return the near cells in order: top, right, bottom, left"""
    return (self.getTopCell(pCell), self.getRightCell(pCell), self.getBottomCell(pCell), self.getLeftCell(pCell))


  def previousFreeCell(self, pCell):
    """Return the previous free Cell"""
    tDeck = self.cells[:]
    tDeck.reverse()
    for i in xrange(9 - pCell, 9):
      if tDeck[i] == None:
        return 8 - i
    return None


  def nextFreeCell(self, pCell):
    """Returns the next free cell"""
    for i in xrange(pCell + 1, 9):
      if self.cells[i] == None:
        return i
    return None


  def freeCells(self):
    """Return indexes of freecells"""
    freeCell = []
    i = 0
    for cell in self.cells:
      if cell == None:
        freeCell.append(i)
      i += 1
    return freeCell


  def copy(self):
    """Returns deck's copy"""
    tDeck = deck()
    tDeck.cells = []
    for tCard in self.cells:
      if tCard != None:
        tDeck.cells.append(tCard.copy())
      else:
        tDeck.cells.append(None)
    tDeck.elements = self.elements[:]
    return tDeck


  def exposedCells(self, pPlayer="P1"):
    """Return [exposedNumber, exposedValue, minExposed]"""
    number = 0
    value = 0
    min = 99
    for cell, tCard in enumerate(self.cells):
      if tCard != None and tCard.owner == pPlayer:
        #Controllo i quattro lati:
        tNear = self.getTopCell(cell)
        if tNear == None and cell not in [0,1,2]:
          number += 1
          value += tCard.top
          min = tCard.top

        tNear = self.getRightCell(cell)
        if tNear == None and cell not in [2,5,8]:
          number += 1
          value += tCard.right
          if tCard.right < min:
            min = tCard.right

        tNear = self.getBottomCell(cell)
        if tNear == None and cell not in [6,7,8]:
          number += 1
          value += tCard.bottom
          if tCard.bottom < min:
            min = tCard.bottom

        tNear = self.getLeftCell(cell)
        if tNear == None and cell not in [0,3,6]:
          number += 1
          value += tCard.left
          if tCard.left < min:
            min = tCard.left

    return [number, value, min]


  def playerCards(self, pPlayer="P1"):
    """Return cards of one player"""
    ret = []
    for tCard in self.cells:
      if tCard and tCard.owner == pPlayer:
        ret.append(tCard)
    return ret


  def stringForServer(self, pPlayer1_ID, pPlayer2_ID):
    """Return the deck string for the server"""
    retL = []
    for tCard in self.cells:
      retL.append(str(tCard.ID) + "||" + tCard.owner + "//%//")
    ret = ''.join(retL)
    ret = ret.replace("P1", str(pPlayer1_ID))
    ret = ret.replace("P2", str(pPlayer2_ID))
    return ret[:-5]
