#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d
from core import graphicElement, animation, functions, buttons

class helpItem(object):
  def __init__(self, pGraphics, pGlobalVars, pFormVars):
    self.globalVars = pGlobalVars
    self.formVars = pFormVars
    self.formGraphics = pGraphics
    self.active = False
    self.images = []
    self.rows = []
    self.data = None
    self.postHideFunc = None
    self.preShowFunc = None


  def show(self):
    """Show help"""
    self.data = [self.formVars.get("status"), self.globalVars.get("message")]
    self.globalVars.set(None, "message")
    self.formVars.set("HELP", "status")
    self.formGraphics.set(self.createGraphics())
    self.active = True


  def hide(self):
    """Hide help"""
    graph = self.formGraphics.get("HELP")
    anim = animation()
    anim.name = "HELP"
    anim.front = True
    anim.exclusive = True
    anim.data = [graph.posX, 275, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim

    graph = graphicElement()
    graph.name = "REMOVE_HELP"
    anim = animation()
    anim.name = graph.name
    anim.priority = -200
    def code(globalVars, formVars, target):
      graph = formVars.get("formGraphics").get("HELP")
      if graph:
        graph.name = "DEL_" + graph.name
      formVars.set(formVars.get("helpItem").data[0], "status")
      globalVars.set(formVars.get("helpItem").data[1], "message")
      formVars.get("helpItem").data = None
      formVars.get("helpItem").active = False
      if self.postHideFunc:
        self.postHideFunc()
      target.name = "DEL_" + target.name
    anim.code = code
    graph.animation = anim
    self.formGraphics.set(graph)


  def createGraphics(self):
    """Create graphic element for help"""
    graph = graphicElement()
    graph.name = "HELP"
    graph.priority = 999
    graph.image = psp2d.Image('themes/' + self.globalVars.get("userData").THEME + '/formsbg/gm_matchhelp.png')
    tX = (480 - graph.image.width) / 2
    tY = (272 - graph.image.height) / 2
    graph.posX = tX
    graph.posY = 275
    anim = animation()
    anim.name = graph.name
    anim.priority = 100
    anim.front = True
    anim.data = [graph.posX, tY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim

    #Controllo quanto sono larghe le immagini:
    maxW = 0
    for pulsante in self.images:
      if pulsante.width > maxW:
        maxW = pulsante.width

    #Aiuto pulsanti:
    posX = 5
    posY = 5

    for i, riga in enumerate(self.rows):
      pulsante = self.images[i]
      if pulsante:
        graph.image.blit(pulsante, 0, 0, pulsante.width, pulsante.height, posX + (maxW - pulsante.width) / 2, posY + 20 * i, True)
        self.globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + maxW + 5, posY + 20 * i, riga)
      else:
        self.globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + 25, posY + 20 * i, riga)

    return graph


  def button_down(self, pbuttons):
    if buttons.square in pbuttons\
    or buttons.circle in pbuttons and self.active:
      #Visualizzo l'aiuto:
      if not self.active:
        try:
          if self.preShowFunc:
            if self.preShowFunc():
              self.show()
          else:
            self.show()
        except AttributeError:
          return
      else:
        self.hide()
