#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d, pspsnd, functions
from clsGraphicElement import graphicElement
from clsAnimation import animation
from clsButtons import buttons
from os import name

class menu(object):
  """Class menu"""
  def __init__(self, pGraphics, pGlobalVars, pFormVars):
    self.name = None
    self.formName = pGlobalVars.get("formName")
    self.formGraphics = pGraphics
    self.globalVars = pGlobalVars
    self.formVars = pFormVars
    self.oldSelected = None
    self.selected = None
    self.elements = []
    self.graphicIndex = []
    self.graphicPriority = 5
    self.animationPriority = 100
    self.posX = 0
    self.posY = 0
    self.textSubY = 0
    self.menuWidth = 0
    self.menuHeight = 0
    self.menuAlignment = "CENTER"
    self.menuVAlignment = "CENTER"
    self.selector = None
    self.background_image = None
    self.firstVisible = None
    self.lastVisible = None
    self.data = None
    self.creating = False

    userData = __import__("userdata")
    self.themeOptions = __import__("themes." + userData.THEME, None, None, userData.THEME)
    self.nextImage = psp2d.Image('themes/' + userData.THEME + '/menu_next.png')
    self.prevImage = psp2d.Image('themes/' + userData.THEME + '/menu_previous.png')

    self.selectorLeftImage = psp2d.Image('themes/' + userData.THEME + '/selector_left.png')
    self.selectorRightImage = psp2d.Image('themes/' + userData.THEME + '/selector_right.png')


  def clear(self):
    """Clear menu data"""
    self.oldSelected = None
    self.selected = None
    for elem in self.elements:
      elem.__dict__.clear()
    self.elements = []
    #self.delGraphics()
    self.graphicIndex = []
    self.posX = 0
    self.posY = 0
    self.textSubY = 0
    self.menuWidth = 0
    self.menuHeight = 0
    self.menuAlignment = "CENTER"
    self.menuVAlignment = "CENTER"
    self.selector = None
    self.background_image = None
    self.firstVisible = None
    self.lastVisible = None
    self.data = None


  def getSelected(self):
    """Returns the selected element"""
    if self.selected and len(self.selected):
      for tElement in self.elements:
        if tElement.value == self.selected:
          return tElement
    return None


  def getElement(self, pvalue):
    """Returns the an element"""
    for tElement in self.elements:
      if tElement.value.split("|")[0] == pvalue or tElement.value == pvalue:
        return tElement
    return None


  def selectPrevious(self):
    """Selects previous menu"""
    next = False
    tElements = self.elements[:]
    tElements.reverse()
    for i, tElement in enumerate(tElements):
      #Controllo se e' abilitato:
      if next and tElement.enabled:
        self.oldSelected = self.selected
        self.selected = tElement.value

        #Controllo se e' visibile:
        if not tElement.visible:
          self.scrollDown()
        else:
          self.createGraphicElements()

        tElement.trigger_enter(self.globalVars, self.formVars, self, tElement)
        return True

      if tElement.value == self.selected:
        if i < len(tElements) - 1:
          next = True
        else:
          return False
    return False


  def selectNext(self):
    """Selects next menu"""
    next = False
    for i, tElement in enumerate(self.elements):
      #Controllo se e' abilitato:
      if next and tElement.enabled:
        self.oldSelected = self.selected
        self.selected = tElement.value

        #Controllo se e' visibile:
        if not tElement.visible:
          self.scrollUp()
        else:
          self.createGraphicElements()

        tElement.trigger_enter(self.globalVars, self.formVars, self, tElement)
        return True

      if tElement.value == self.selected:
        if i < len(self.elements) - 1:
          next = True
        else:
          return False
    return False


  def selectPreviousSubMenu(self):
    """Selects previous sub menu"""
    next = False
    if self.selected.find("|") >= 0:
      find = self.selected.split("|")[1]
      subMenu = self.getSelected().subMenu[:]
      subMenu.reverse()
      for i, tElement in enumerate(subMenu):
        #Controllo se e' abilitato:
        if next and tElement.enabled:
          selected = self.getSelected()
          selected.value = selected.value.split("|")[0] + "|" + tElement.value
          self.selected = self.selected.split("|")[0] + "|" + tElement.value
          tElement.trigger_enter(self.globalVars, self.formVars, self, tElement)
          return True

        if tElement.value == find:
          if i < len(subMenu) - 1:
            next = True
          else:
            return False


  def selectNextSubMenu(self):
    """Selects next sub menu"""
    next = False
    if self.selected.find("|") >= 0:
      find = self.selected.split("|")[1]
      subMenu = self.getSelected().subMenu
      for i, tElement in enumerate(subMenu):
        if next and tElement.enabled:
          selected = self.getSelected()
          selected.value = selected.value.split("|")[0] + "|" + tElement.value
          self.selected = self.selected.split("|")[0] + "|" + tElement.value
          tElement.trigger_enter(self.globalVars, self.formVars, self, tElement)
          return True

        if tElement.value == find:
          if i < len(subMenu) - 1:
            next = True
          else:
            return False


  def scrollUp(self):
    """Scrolls up the menu"""
    #Nascondo i selector:
    self.hideSelectors()

    graph = self.getGraphicElement("MENU_TEXT")
    if graph:
      anim = animation()
      graph.animation = anim
      anim.name = "SCROLL_MENU_TEXT"
      anim.lockKeyboard = True
      anim.priority = 999
      anim.exclusive = True
      sel = self.getSelected()
      prev = self.elements[self.elements.index(sel) - 1]
      anim.data = [0, sel.posY - prev.posY]
      self.firstVisible = self.elements[self.elements.index(self.getElement(self.firstVisible.value)) + 1]
      self.lastVisible = self.elements[self.elements.index(self.getElement(self.lastVisible.value)) + 1]
      def code(globalVars, formVars, target):
        #Scrollo le scritte:
        if target.animation.data[0] < target.animation.data[1]:
          target.subY += 2
          self.textSubY += 2
          target.animation.data[0] += 2
        else:
          if self.elements.index(self.getElement(self.firstVisible.value)) > 0:
            target.subY = target.animation.data[1]
          else:
            target.subY = 0
          target.animation.name = "DEL_" + target.animation.name
          self.createGraphicElements()
          self.showSelectors()
      anim.code = code


  def scrollDown(self):
    """Scrolls down the menu"""
    #Nascondo i selector:
    self.hideSelectors()

    graph = self.getGraphicElement("MENU_TEXT")
    if graph:
      anim = animation()
      graph.animation = anim
      anim.name = "SCROLL_MENU_TEXT"
      anim.lockKeyboard = True
      anim.priority = 999
      anim.exclusive = True
      sel = self.getSelected()
      next = self.elements[self.elements.index(sel) + 1]
      anim.data = [0, next.posY - sel.posY]
      self.firstVisible = self.elements[self.elements.index(self.getElement(self.firstVisible.value)) - 1]
      self.lastVisible = self.elements[self.elements.index(self.getElement(self.lastVisible.value)) - 1]
      def code(globalVars, formVars, target):
        #Scrollo le scritte:
        if target.animation.data[0] < target.animation.data[1] and target.subY >= 2:
          target.subY -= 2
          self.textSubY -= 2
          target.animation.data[0] += 2
        else:
          if self.elements.index(self.getElement(self.firstVisible.value)) > 0:
            target.subY = target.animation.data[1]
          else:
            target.subY = 0
          target.animation.name = "DEL_" + target.animation.name
          self.createGraphicElements()
          self.showSelectors()
      anim.code = code


  def getGraphicElement(self, pName):
    return self.formGraphics.get(pName.replace("MENU_", "MENU_" + self.name + "_"))


  def delGraphicElement(self, pName):
    """Delete a graphic element"""
    tGraph = self.formGraphics.get(pName.replace("MENU_", "MENU_" + self.name + "_"))
    if tGraph:
      if tGraph.name[:4] != "DEL_":
        tGraph.name = "DEL_" + tGraph.name
        #tGraph.animation = None
        tGraph.image = None


  def createGraphicElements(self, pAnimate=None):
    """Create menu's graphic elements"""
    self.creating = True
    if self.selector == None:
      self.selector = self.themeOptions.MENU_SELECTOR
    scrollPrev = None
    scrollNext = None
    leftSelector = None
    rightSelector = None

    #Calcolo altezza/larghezza a disposizione per il testo:
    tempX = self.posX + self.themeOptions.MENU_BORDERWIDTH
    tempW = self.menuWidth - 2 * self.themeOptions.MENU_BORDERWIDTH
    tempH = self.menuHeight - 2 * self.themeOptions.MENU_BORDERWIDTH

    #Sfondo menu:
    bgImage = self.getGraphicElement("MENU_BG")
    if not bgImage:
      bgImage = graphicElement()
      bgImage.name = "MENU_BG"
    bgImage.posX = self.posX
    bgImage.posY = self.posY
    if not self.background_image:
      bgImage.data = [self.menuWidth, self.menuHeight, self.themeOptions.MENU_OUTERCOLOR, self.themeOptions.MENU_INNERCOLOR, self.themeOptions.MENU_CORNERCOLOR, self.themeOptions.MENU_BORDERWIDTH, self.themeOptions.MENU_ALPHA]
      bgImage.image = None
      bgImage.image = functions.drawRect(bgImage)
    else:
      bgImage.image = self.background_image

    bgImage.posX = self.posX
    bgImage.posY = self.posY

    #Calcolo font e posizioni:
    posY = 0
    textH = 0
    visibleElements = 0
    for tElementIndex, tElement in enumerate(self.elements):
      #Controllo che font devo usare:
      #Selezionato:
      if self.selected != None \
      and self.selected == tElement.value:
        if tElement.font:
          tFont = tElement.font
        else:
          tFont = self.globalVars.get("fonts").get("fontMenuSel")
      #Disabilitato:?
      elif not tElement.enabled:
        tFont = self.globalVars.get("fonts").get("fontMenuDisabled")
      #Normale:
      else:
        if tElement.font:
          tFont = tElement.font
        else:
          tFont = self.globalVars.get("fonts").get("fontMenu")

      tElement.posY = posY
      tElement.fontUsed = tFont
      #Calcolo l'altezza:
      if not tElement.icon \
      or tElement.icon.height <= functions.textHeight(tFont, "I"):
        textH += functions.textHeight(tFont, "I") + self.themeOptions.MENU_INTERLINE
        posY += functions.textHeight(tFont, "I") + self.themeOptions.MENU_INTERLINE
      else:
        textH += tElement.icon.height + self.themeOptions.MENU_INTERLINE
        posY += tElement.icon.height + self.themeOptions.MENU_INTERLINE
      if textH < tempH:
        visibleElements += 1

    #Oggetto grafico per testi:
    text = self.getGraphicElement("MENU_TEXT")
    if not text:
      text = graphicElement()
      text.name = "MENU_TEXT"
      text.subX = 0
      text.subY = 0
      text.subWidth  = self.menuWidth - 2 * self.themeOptions.MENU_BORDERWIDTH
      text.width = self.menuWidth - 2 * self.themeOptions.MENU_BORDERWIDTH
    text.posX = self.posX + self.themeOptions.MENU_BORDERWIDTH

    #Controllo se avro' bisogno dei segnali prossimo/precedente:
    scroll = False
    if textH >= tempH:
      scroll = True
      textH = tempH + 2 * functions.textHeight(tFont, "I")
      tempH = tempH - (2 * self.nextImage.height + 2 * self.themeOptions.MENU_INTERLINE)
      #Dimensione testi:
      text.subHeight = self.menuHeight - 2 * self.themeOptions.MENU_BORDERWIDTH - 2 * self.prevImage.height
      text.height = text.subHeight

      #Segni di scroll:
      graph = self.getGraphicElement("MENU_PREV")
      if not graph:
        graph = graphicElement()
        graph.name = "MENU_PREV"
      if not graph.image:
        graph.image = self.prevImage
      if self.menuAlignment.upper() == "CENTER":
        tX = tempX + (tempW - self.nextImage.width) / 2
      elif self.menuAlignment.upper() == "LEFT":
        tX = tempX + self.themeOptions.MENU_BORDERWIDTH
      elif self.menuAlignment.upper() == "RIGHT":
        tX = tempX + tempW - self.nextImage.width - self.themeOptions.MENU_BORDERWIDTH
      graph.posX = tX
      graph.posY = self.posY + self.themeOptions.MENU_BORDERWIDTH
      graph.visible = False
      scrollPrev = graph

      graph = self.getGraphicElement("MENU_NEXT")
      if not graph:
        graph = graphicElement()
        graph.name = "MENU_NEXT"
      if not graph.image:
        graph.image = self.nextImage
      if self.menuAlignment.upper() == "CENTER":
        tX = tempX + (tempW - self.nextImage.width) / 2
      elif self.menuAlignment.upper() == "LEFT":
        tX = tempX +  self.themeOptions.MENU_BORDERWIDTH
      elif self.menuAlignment.upper() == "RIGHT":
        tX = tempX + tempW - self.nextImage.width - self.themeOptions.MENU_BORDERWIDTH
      graph.posX = tX
      graph.posY = self.posY + self.menuHeight - graph.image.height -  self.themeOptions.MENU_BORDERWIDTH
      graph.visible = False
      scrollNext = graph
    else:
      #Dimensione testi:
      text.subHeight = textH
      text.height = text.subHeight
      self.delGraphicElement("MENU_PREV")
      self.delGraphicElement("MENU_NEXT")

    #Allineamento verticale:
    if textH < tempH:
      if self.menuVAlignment == "CENTER":
        tempY = (tempH - textH ) / 2 + self.themeOptions.MENU_BORDERWIDTH
      elif self.menuVAlignment == "TOP":
        tempY = self.themeOptions.MENU_BORDERWIDTH
      elif self.menuVAlignment == "BOTTOM":
        tempY = tempH - textH + self.themeOptions.MENU_BORDERWIDTH
    else:
      tempY = self.themeOptions.MENU_BORDERWIDTH
      if scroll:
        tempY += scrollPrev.image.height
    text.posY = self.posY + tempY

    #Controllo il primo e l'ultimo visibile:
    if self.firstVisible: #and self.lastVisible:
      if visibleElements >= len(self.elements):
        self.firstVisible = self.elements[0]
      firstVisibleIndex = self.elements.index(self.getElement(self.firstVisible.value))
      lastVisibleIndex = firstVisibleIndex + visibleElements - 1
      if lastVisibleIndex > len(self.elements) - 1:
        lastVisibleIndex = len(self.elements) - 1
      self.lastVisible = self.elements[lastVisibleIndex]
      #lastVisibleIndex = self.elements.index(self.getElement(self.lastVisible.value))
    else:
      self.firstVisible = None
      firstVisibleIndex = None
      self.lastVisible = None
      lastVisibleIndex = None

      startY = 0
      for tElementIndex, tElement in enumerate(self.elements):
        tHeight = functions.textHeight(tElement.fontUsed, "I")
        if startY - 2 * self.themeOptions.MENU_BORDERWIDTH - self.textSubY + tHeight >= tempH \
        or tElement.posY - self.textSubY < 0:
          tElement.visible = False
        else:
          tElement.visible = True
          if not self.firstVisible:
            self.firstVisible = tElement
            firstVisibleIndex = tElementIndex
          self.lastVisible = tElement
          lastVisibleIndex = tElementIndex
        startY += tHeight + self.themeOptions.MENU_INTERLINE

    if self.firstVisible:
      firstVisibleIndex = self.elements.index(self.getElement(self.firstVisible.value))
    if self.lastVisible:
      lastVisibleIndex = self.elements.index(self.getElement(self.lastVisible.value))

    textBuffer = psp2d.Image(tempW, textH)
    if name == "psp":
      textBuffer.clear(psp2d.Color(0,0,0))
    if text.subY + text.subHeight > textH:
      text.subX = 0
      text.subY = 0
      text.subWidth  = tempW
      text.width = tempW
      text.subHeight = textH
      text.height = textH
    if firstVisibleIndex > 0:
      text.subY = self.elements[firstVisibleIndex].posY - self.elements[firstVisibleIndex - 1].posY
      self.textSubY = self.elements[firstVisibleIndex].posY
    else:
      self.textSubY = 0

    #Scroll visibili?
    if scroll:
      if firstVisibleIndex > 0:
        scrollPrev.visible = True
      else:
        scrollPrev.visible = False
      if lastVisibleIndex < len(self.elements) - 1:
        scrollNext.visible = True
      else:
        scrollNext.visible = False

    #Loop elementi:
    tempY = 0
    for tElementIndex, tElement in enumerate(self.elements):
      #Controllo se devo scrivere:
      if tElementIndex < firstVisibleIndex - 1\
      or tElementIndex > lastVisibleIndex + 1:
        tElement.visible = False
        continue

      if tElementIndex >= firstVisibleIndex and tElementIndex <= lastVisibleIndex:
        tElement.visible = True
      else:
        tElement.visible = False

      #Construisco il testo da visualizzare:
      testo = tElement.text
      if len(tElement.subMenu) > 0 \
      and tElement.value.find("|") >= 0:
        for tSubMenu in tElement.subMenu:
          if tSubMenu.value == tElement.value.split("|")[1]:
            testo = tElement.text + " " + tSubMenu.text

      #Stringa di aiuto:
      if self.selected != None \
      and self.selected == tElement.value:
        self.globalVars.set(tElement.helpText, "message")

      #Allineamento orizzontale:
      tWidth = functions.textWidth(tElement.fontUsed, testo)
      if tElement.icon:
        tWidth += tElement.icon.width

      #Limito il testo se esce:
      maxWidth = self.menuWidth
      if self.selector in ["LEFT", "RIGHT"] and tWidth > self.menuWidth + self.selectorLeftImage.width:
        maxWidth = self.menuWidth - self.selectorLeftImage.width - 3
      elif self.selector == "BOTH" and tWidth > self.menuWidth + 2 * self.selectorLeftImage.width:
        maxWidth = self.menuWidth - 2 * self.selectorLeftImage.width - 6

      if tWidth > maxWidth:
        while tWidth > maxWidth:
          testo = testo[:-1]
          tWidth = functions.textWidth(tElement.fontUsed, testo)
        testo = testo[:-3] + "..."

      if self.menuAlignment.upper() == "CENTER":
        tX = tempX + (tempW - tWidth) / 2
      elif self.menuAlignment.upper() == "LEFT":
        tX = tempX
      elif self.menuAlignment.upper() == "RIGHT":
        tX = tempX + tempW - tWidth - self.themeOptions.MENU_BORDERWIDTH

      #Selectors:
      if self.selected != None \
      and self.selected == tElement.value:
        #Left selector:
        if self.selector in ["LEFT", "BOTH"]:
          leftSelector = self.getGraphicElement("MENU_SELECTOR_LEFT")
          if not leftSelector:
            leftSelector = graphicElement()
            leftSelector.name = "MENU_SELECTOR_LEFT"
            leftSelector.image = self.selectorLeftImage
            leftSelector.data = leftSelector.image
            leftSelector.visible = True
          leftSelector.posX = tX - leftSelector.image.width - 3
          leftSelector.posY = tempY + text.posY + self.themeOptions.MENU_SELECTOR_DELTA - text.subY
          anim = animation()
          anim.name = leftSelector.name
          anim.lockKeyboard = False
          anim.data = [-0.5,0]
          anim.code = functions.selectorAnim
          leftSelector.animation = anim
        #Right selector
        if self.selector in ["RIGHT", "BOTH"]:
          rightSelector = self.getGraphicElement("MENU_SELECTOR_RIGHT")
          if not rightSelector:
            rightSelector = graphicElement()
            rightSelector.name = "MENU_SELECTOR_RIGHT"
            rightSelector.image = self.selectorRightImage
            rightSelector.data = rightSelector.image
            rightSelector.visible = True
          rightSelector.posX = tX + tWidth + 3
          rightSelector.posY = tempY + text.posY + self.themeOptions.MENU_SELECTOR_DELTA - text.subY
          anim = animation()
          anim.name = rightSelector.name
          anim.lockKeyboard = False
          anim.data = [+0.5,0]
          anim.code = functions.selectorAnim
          rightSelector.animation = anim

      #Scrivo:
      sel = self.getSelected()
      tElement.posX = tX - self.posX
      tHeight = functions.textHeight(tElement.fontUsed, "I")
      if tElement.icon:
        if self.menuAlignment in ["LEFT", "CENTER"]:
          textBuffer.blit(tElement.icon, 0, 0, tElement.icon.width, tElement.icon.height, tX - self.posX, tempY, True)
          tElement.fontUsed.drawText(textBuffer, tX - self.posX + tElement.icon.width, tempY, testo)
        elif self.menuAlignment == "RIGHT":
          textBuffer.blit(tElement.icon, 0, 0, tElement.icon.width, tElement.icon.height, self.menuWidth - tElement.icon.width - 2 * self.themeOptions.MENU_BORDERWIDTH, tempY, True)
          tElement.fontUsed.drawText(textBuffer, tX - self.posX, tempY, testo)
      else:
        tElement.fontUsed.drawText(textBuffer, tX - self.posX, tempY, testo)
      tempY += functions.textHeight(tElement.fontUsed, "I") + self.themeOptions.MENU_INTERLINE
    text.image = textBuffer

    #Controllo se servono i selector:
    if not self.selected:
      self.delGraphicElement("MENU_SELECTOR_LEFT")
      self.delGraphicElement("MENU_SELECTOR_RIGHT")

    #Aggiungo tutti gli elementi grafici:
    graphics = []
    graphicIndex = []
    graphics.append(bgImage)
    graphicIndex.append(bgImage.name)
    graphics.append(text)
    graphicIndex.append(text.name)
    if scrollPrev:
      graphics.append(scrollPrev)
      graphicIndex.append(scrollPrev.name)
    if scrollNext:
      graphics.append(scrollNext)
      graphicIndex.append(scrollNext.name)
    if leftSelector:
      graphics.append(leftSelector)
      graphicIndex.append(leftSelector.name)
    if rightSelector:
      graphics.append(rightSelector)
      graphicIndex.append(rightSelector.name)

    #Animazione di entrata:
    if pAnimate:
      #Animazione elementi grafici:
      lastAnim = ""
      for i, graph in enumerate(graphics):
        anim = animation()
        anim.name = graph.name
        anim.front = True
        anim.priority = self.animationPriority
        if i == len(graphics) - 1:
          lastAnim = graph.name
        if pAnimate == "BOTTOM":
          anim.data = [graph.posX, graph.posY, 99]
          graph.posY = graph.posY + 272
        elif pAnimate == "LEFT":
          anim.data = [graph.posX, graph.posY, 99]
          graph.posX = graph.posX - 480
        elif pAnimate == "RIGHT":
          anim.data = [graph.posX, graph.posY, 99]
          graph.posX = 480 + graph.posX
        anim.code = functions.moveFromTo
        graph.animation = anim

      #Attivo l'animazione sui selector:
      newGraph = graphicElement()
      newGraph.name = "START_SELECTOR_ANIM"
      anim = animation()
      anim.name = newGraph.name
      anim.data = lastAnim
      anim.priority = self.animationPriority - 1
      def code(globalVars, formVars, target):
        #Aspetto il termine dell'animazione precedente:
        if not self.getGraphicElement(target.animation.data).animation:
          sel = self.getGraphicElement("MENU_SELECTOR_LEFT")
          if sel:
            anim = animation()
            anim.name = sel.name
            anim.lockKeyboard = False
            anim.priority = 9999
            anim.data = [-0.5,0]
            anim.code = functions.selectorAnim
            sel.animation = anim

          sel = self.getGraphicElement("MENU_SELECTOR_RIGHT")
          if sel:
            anim = animation()
            anim.name = sel.name
            anim.lockKeyboard = False
            anim.priority = 9999
            anim.data = [+0.5,0]
            anim.code = functions.selectorAnim
            sel.animation = anim
          target.name = "DEL_" + target.name
      anim.code = code
      newGraph.animation = anim
      graphics.append(newGraph)
      graphicIndex.append(newGraph.name)

    if self.formName != self.globalVars.get("formName"):
      self.creating = False
      return

    graphicPriority = self.graphicPriority
    self.graphicIndex = []
    for graph in graphics:
      if graph not in self.formGraphics.dataDic.values():
        graph.priority = graphicPriority
        graphicPriority += 1
        graph.name = graph.name.replace("MENU_", "MENU_" +  self.name + "_")
        self.formGraphics.set(graph)
      if graph.name != "START_SELECTOR_ANIM":
        self.graphicIndex.append(graph.name)
    self.creating = False


  def delGraphics(self):
    """Del graphic elements from form"""
    for name in self.graphicIndex:
      self.delGraphicElement(name.replace(self.name+ "_", ""))


  def hideGraphics(self):
    """Hide graphic elements from form"""
    for name in self.graphicIndex:
      self.getGraphicElement(name.replace(self.name+ "_", "")).visible = False


  def showGraphics(self):
    """Hide graphic elements from form"""
    for name in self.graphicIndex:
      self.getGraphicElement(name.replace(self.name+ "_", "")).visible = True


  def selectFirstElement(self):
    """Select the first enabled element"""
    self.selected = None
    for el in self.elements:
      if el.enabled:
        self.selected = el.value
        break


  def button_down(self, pbuttons):
    #Se sto creando gli oggetti grafici non processo i tasti
    #(succede solo nel multiplayer)
    if self.creating == True:
      return

    #Gestione tasti menu
    if buttons.up in pbuttons:
      if self.selectPrevious():
        sound = self.globalVars.get("sounds").get("menuChangeSound")
        sound.start()
    elif buttons.down in pbuttons:
      if self.selectNext():
        sound = self.globalVars.get("sounds").get("menuChangeSound")
        sound.start()
    elif buttons.right in pbuttons:
      #Seleziono sottomenu:
      if self.selected \
      and self.selected.find("|") >= 0 and self.selectNextSubMenu():
        self.sound = self.globalVars.get("sounds").get("menuChangeSound")
        self.sound.start()
        selected = self.getSelected()
        selected.trigger_changeSubMenu(self.globalVars, self.formVars, self, selected)
        self.createGraphicElements()
    elif buttons.left in pbuttons:
      #Seleziono sottomenu:
      if self.selected \
      and self.selected.find("|") >= 0 and self.selectPreviousSubMenu():
        sound = self.globalVars.get("sounds").get("menuChangeSound")
        sound.start()
        selected = self.getSelected()
        selected.trigger_changeSubMenu(self.globalVars, self.formVars, self, selected)
        self.createGraphicElements()
    elif buttons.circle in pbuttons:
      self.trigger_cancel(self.globalVars, self.formVars, self, self.getSelected())
    elif buttons.cross in pbuttons:
      selected = self.getSelected()
      if selected:
        selected.trigger_select(self.globalVars, self.formVars, self, selected)


  def hideSelectors(self):
    """Hide menu selectors"""
    sel = self.getGraphicElement("MENU_SELECTOR_RIGHT")
    if sel:
      sel.visible = False
    sel = self.getGraphicElement("MENU_SELECTOR_LEFT")
    if sel:
      sel.visible = False


  def showSelectors(self):
    """Hide menu selectors"""
    sel = self.getGraphicElement("MENU_SELECTOR_RIGHT")
    if sel:
      sel.visible = True
    sel = self.getGraphicElement("MENU_SELECTOR_LEFT")
    if sel:
      sel.visible = True


  def logInfo(self):
    """Log menu's infos"""
    self.globalVars.get("logFile").log("-" * 70)
    self.globalVars.get("logFile").log("Name:" + str(self.name))
    self.globalVars.get("logFile").log("posX:" + str(self.posX))
    self.globalVars.get("logFile").log("posY:" + str(self.posY))
    self.globalVars.get("logFile").log("width:" + str(self.menuWidth))
    self.globalVars.get("logFile").log("height:" + str(self.menuHeight))
    graph = self.getGraphicElement("MENU_TEXT")
    self.globalVars.get("logFile").log("subX:" + str(graph.subX))
    self.globalVars.get("logFile").log("subY:" + str(graph.subY))
    self.globalVars.get("logFile").log("subWidth:" + str(graph.subWidth))
    self.globalVars.get("logFile").log("subHeight:" + str(graph.subHeight))
    self.globalVars.get("logFile").log("-" * 70)
    for i, el in enumerate(self.elements):
      self.globalVars.get("logFile").log("Element " + str(i) + " : " + el.value + " (" + str(el.text) + ")")



  def trigger_cancel(self, globalVars, formVars, menu, element):
    pass
