#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
from urllib import urlopen, urlcleanup
from thread import start_new_thread
from socket import setdefaulttimeout, timeout
from time import time, sleep
from os import name
from traceback import format_exc
import pspnet
import functions

class request(object):
  def __init__(self, pAddress=None, pCallback=None, pCallingForm=None):
    self.special = None
    self.address = pAddress
    self.callback = pCallback
    self.callingForm = pCallingForm


class net(object):
  """Class net"""
  def __init__(self, pGlobalVars):
    self.kill = False
    self.globalVars = pGlobalVars
    self.index = None
    self.serverAddress = None
    self.serverStatus = None
    self.session = None
    self.requests = []
    self.currentRequest = None
    self.changingForm = False
    setdefaulttimeout(20)


  def joinParameters(self, pParams):
    """Build string containing parameters"""
    if len(pParams) > 0:
      ret = "?"
      for i, p in enumerate(pParams):
        if i > 0:
          ret += "&"
        val = p[1]
        if val == None:
          val = ""
        ret += p[0] + "=" + str(val).strip()
      ret = functions.stringToServer(ret)
      return ret
    else:
      return ""


  def internalServer(self):
    """Internal server for request to server"""
    while 1:
      if self.kill:
        break
      #Loop delle richieste:
      while len(self.requests) > 0:
        if self.kill:
          break
        req = self.requests.pop(0)
        #Controllo la form:
        if req.callingForm and req.callingForm != self.globalVars.get("formName") and req.callback:
          continue
        
        if self.globalVars.get("userData").DEBUG_NET == "YES":
          self.globalVars.get("logFile").log("NET REQUEST: form: %s Address: %s Special: %s" % (req.callingForm, req.address, req.special))
        #Connessione
        if req.special == "WIFICONNECT":
          sleep(0.5)
          try:
            pspnet.disconnectAPCTL()
          except:
            pass
          try:
            pspnet.connectToAPCTL(req.address, req.callback, 15)
          except pspnet.Error, exception:
            self.globalVars.get("logFile").log("NET ERROR: ")
            self.globalVars.get("logFile").log(exception)
            try:
              self.globalVars.get("logFile").log(format_exc())
            except:
              pass
            req.callback(99)
          except Exception, exception:
            self.globalVars.get("logFile").log("NET ERROR: ")
            self.globalVars.get("logFile").log(exception)
            try:
              self.globalVars.get("logFile").log(format_exc())
            except:
              pass
        elif req.special == "WIFIDISCONNECT":
          #Disconnetto:
          try:
            pspnet.disconnectAPCTL()
          except:
            pass
        else:
          #Leggo dal server:
          self.openURL(req)
      else:
        sleep(1)
    self.kill = False


  def openURL(self, pReq):
    """Read data from an URL"""
    strResponse = ""
    strError = None
    self.currentRequest = pReq
    try:
      start = time()
      end = time()
      f = urlopen(pReq.address)
      strResponse = f.read().strip()
      f.close()
      end = time()
    except Exception, msg:
      if "TIMEOUT" in str(msg).upper()\
      or "TIMED OUT" in str(msg).upper():
        msg = "Timeout error"
      strError = (None, (None, str(msg)))
      self.globalVars.get("logFile").log("NET ERROR: " + str(msg))
    urlcleanup()
    #Controllo se la risposta e' un HTML:
    if "</HTML>" in strResponse.upper() or "<HTML>" in strResponse.upper() \
    or "</H1>" in strResponse.upper() or "<H1>" in strResponse.upper():
      strError = (None, (None, "Communication error"))

    if self.globalVars.get("userData").DEBUG_NET == "YES":
      self.globalVars.get("logFile").log("NET RESPONSE: " + pReq.address + " = " + strResponse + " (" + str(round(end - start, 1)) + " secs.)")

    if pReq.callback:
      #Controllo la form:
      if pReq.callingForm and pReq.callingForm == self.globalVars.get("formName") \
      and not self.changingForm:
        try:
          if self.globalVars.get("userData").DEBUG_NET == "YES":
            self.globalVars.get("logFile").log("NET CALLBACK: " + str(pReq.callback))
          pReq.callback(strResponse, strError)
        except Exception, exception:
          self.globalVars.get("logFile").log("NET ERROR: " + str(exception))
          self.globalVars.get("logFile").log(format_exc())
    self.currentRequest = None


  def startServer(self):
    """Start internal server"""
    #Server sincrono:
    start_new_thread(self.internalServer, ())


  def killServer(self):
    """Kill internal server"""
    self.kill = True


  def connect(self, pIndex, pCallback=None):
    #return
    tReq = request()
    tReq.special = "WIFICONNECT"
    tReq.callback = pCallback
    tReq.address = pIndex
    self.requests.append(tReq)


  def disconnect(self):
    tReq = request()
    tReq.special = "WIFIDISCONNECT"
    self.requests.append(tReq)


  def wlanSwitchState(self):
    return pspnet.wlanSwitchState()


  def getAPCTLState(self):
    return pspnet.getAPCTLState()


  def checkServerStatus(self, pVersion, pCallback=None):
    """Check server status"""
    params = [("mod", "status"),
              ("version", pVersion)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def serverInfo(self, pCallback=None):
    """Ask server info"""
    params = [("mod", "serverinfo")]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def login(self, pLang, pVersion, pUserName, pPassword, pCallback=None):
    """Login to server"""
    params = [("mod", "login"), \
              ("username", pUserName), \
              ("password", pPassword),
              ("lang", pLang), \
              ("system", name), \
              ("version", pVersion)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def playerList(self, pCity, pLocation, pCallback=None):
    """Ask player list"""
    params = [("mod", "playerlist"), \
              ("session", self.session), \
              ("city", str(pCity)), \
              ("location", str(pLocation))]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def myBuddyList(self, pCallback=None):
    """Ask complete buddy list"""
    params = [("mod", "mybuddylist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def myBlackList(self, pCallback=None):
    """Ask complete buddy list"""
    params = [("mod", "myblacklist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def playerInfo(self, pPlayer, pCallback=None):
    """Ask player list by language"""
    params = [("mod", "playerinfo"), \
              ("session", self.session), \
              ("player", pPlayer)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def addToBuddyList(self, pPlayer, pCallback=None):
    """Add a player to the buddy list"""
    params = [("mod", "addtobuddylist"), \
              ("session", self.session), \
              ("player", pPlayer)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def removeFromBuddyList(self, pPlayer, pCallback=None):
    """Add a player from the buddy list"""
    params = [("mod", "removefrombuddylist"), \
              ("session", self.session), \
              ("player", pPlayer)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def addToBlackList(self, pPlayer, pCallback=None):
    """Add a player to the buddy list"""
    params = [("mod", "addtoblacklist"), \
              ("session", self.session), \
              ("player", pPlayer)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def removeFromBlackList(self, pPlayer, pCallback=None):
    """Add a player from the buddy list"""
    params = [("mod", "removefromblacklist"), \
              ("session", self.session), \
              ("player", pPlayer)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def keepAlive(self, pCallback=None):
    """Keeps alive the session"""
    params = [("mod", "refresh"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback)
    self.requests.insert(0, tReq)


  def changeStatus(self, pStatus, pCallback=None):
    """Chenge player status"""
    params = [("mod", "changestatus"), \
              ("session", self.session), \
              ("status", pStatus)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback) #, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def registerUser(self, pUserName, pPassword, pElement, pCallback=None):
    """Register a new user"""
    params = [("mod", "register"), \
              ("username", pUserName), \
              ("password", pPassword), \
              ("element", pElement)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def myMessages(self, pCallback=None):
    """Ask for unread messages"""
    params = [("mod", "mymessages"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def messageList(self, pCallback=None):
    """Ask for message list"""
    params = [("mod", "messagelist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def readMessage(self, pMessageID, pCallback=None):
    """Ask for message list"""
    params = [("mod", "readmessage"), \
              ("session", self.session), \
              ("messageid", pMessageID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def deleteMessage(self, pMessageID, pCallback=None):
    """Ask for message list"""
    params = [("mod", "deletemessage"), \
              ("session", self.session), \
              ("messageid", pMessageID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def sendMessage(self, pTo, pHeader, pText, pCallback=None):
    """Ask for message list"""
    params = [("mod", "sendmessage"), \
              ("session", self.session), \
              ("to", pTo), \
              ("header", pHeader), \
              ("text", pText)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def cityList(self, pCallback):
    """Ask city list"""
    params = [("mod", "citylist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def allLocationList(self, pCallback):
    """Ask location list"""
    params = [("mod", "alllocationlist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def locationList(self, pCityID, pCallback):
    """Ask city list"""
    params = [("mod", "locationlist"), \
              ("session", self.session),
              ("city", pCityID),]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def enterLocation(self, pCityID, pLocationID, pCallback=None):
    """Move to a new city/location"""
    params = [("mod", "enterlocation"), \
              ("session", self.session), \
              ("cityid", pCityID), \
              ("locationid", pLocationID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def getAllCards(self, pCallback=None):
    """Gets all cards from server"""
    params = [("mod", "getcards"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def getMyCards(self, pCallback=None):
    """Gets all cards from server"""
    params = [("mod", "mycards"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def addToDeck(self, pCardID, pSlot, pCallback=None):
    params = [("mod", "addtodeck"), \
              ("session", self.session), \
              ("cardid", pCardID), \
              ("slot", pSlot)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def getMyDeck(self, pCallback=None):
    """Gets all cards from server"""
    params = [("mod", "mydeck"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def resetDeck(self, pCallback=None):
    """Reset deck and money"""
    params = [("mod", "resetdeck"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def myMoney(self, pCallback=None):
    """Gets player money"""
    params = [("mod", "mymoney"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def buyCard(self, pCardID, pCallback=None):
    params = [("mod", "buy"), \
              ("session", self.session), \
              ("itemid", pCardID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def sellCard(self, pCardID, pCallback=None):
    params = [("mod", "sell"), \
              ("session", self.session), \
              ("itemid", pCardID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def getInvites(self, pCallback=None):
    params = [("mod", "getinvites"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def sendInvite(self, pPlayerID, pRules, pElements, pCallback=None):
    params = [("mod", "sendinvite"), \
              ("session", self.session), \
              ("playerid", pPlayerID), \
              ("rules", pRules), \
              ("elem", pElements)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback)#, self.globalVars.get("formName"))
    self.requests.insert(0, tReq)


  def acceptInvite(self, pInviteID, pCallback=None):
    params = [("mod", "acceptinvite"), \
              ("session", self.session), \
              ("inviteid", pInviteID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.insert(0, tReq)


  def confirmDeck(self, pRoomID, pCallback=None):
    params = [("mod", "confirmdeck"), \
              ("session", self.session), \
              ("roomid", pRoomID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def waitOpponentsDeck(self, pRoomID, pCallback=None):
    params = [("mod", "waitopponentsdeck"), \
              ("session", self.session), \
              ("roomid", pRoomID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def getDecksAndRules(self, pRoomID, pCallback=None):
    params = [("mod", "getdecksrules"), \
              ("session", self.session), \
              ("roomid", pRoomID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def waitOpponentsMove(self, pRoomID, pTurn, pCallback=None):
    params = [("mod", "waitopponentsmove"), \
              ("session", self.session), \
              ("roomid", pRoomID), \
              ("turn", pTurn)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def makeMove(self, pRoomID, pCardID, pPosition, pCallback=None):
    params = [("mod", "makemove"), \
              ("session", self.session), \
              ("roomid", pRoomID), \
              ("cardid", pCardID), \
              ("position", pPosition)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def matchEnd(self, pRoomID, pDeck, pCallback=None):
    params = [("mod", "matchend"), \
              ("session", self.session), \
              ("roomid", pRoomID), \
              ("deck", pDeck)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def waitMatchValidation(self, pRoomID, pCallback=None):
    params = [("mod", "waitmatchvalidation"), \
              ("session", self.session), \
              ("roomid", pRoomID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def confirmTrade(self, pRoomID, pCardsID, pCallback=None):
    params = [("mod", "confirmtrade"), \
              ("session", self.session), \
              ("roomid", pRoomID), \
              ("cards", pCardsID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def waitOpponetsTrading(self, pRoomID, pCallback=None):
    params = [("mod", "waitopponentstrading"), \
              ("session", self.session), \
              ("roomid", pRoomID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def getChatMessages(self, pRoomID, pCallback=None):
    """Logout from server"""
    params = [("mod", "roomchat"), \
              ("session", self.session), \
              ("roomid", pRoomID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def sendChatMessage(self, pRoomID, pCode, pMessage, pCallback=None):
    """Logout from server"""
    params = [("mod", "sendchat"), \
              ("session", self.session), \
              ("roomid", pRoomID), \
              ("code", pCode), \
              ("string", pMessage)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def leaveRoom(self, pRoomID, pAbort="N", pCallback=None):
    params = [("mod", "leaveroom"), \
              ("session", self.session), \
              ("roomid", pRoomID), \
              ("abort", pAbort)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def locationMessages(self, pLocationID, pCallback=None):
    """Add a player from the buddy list"""
    params = [("mod", "locatiomessages"), \
              ("session", self.session), \
              ("location", pLocationID)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def shout(self, pMessage, pCallback=None):
    """Logout from server"""
    params = [("mod", "shout"), \
              ("session", self.session), \
              ("message", pMessage)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def topList(self, pCallback=None):
    """Ask toplist"""
    params = [("mod", "toplist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def topListLocation(self, pCallback=None):
    """Ask location toplist"""
    params = [("mod", "toplocationlist"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def serverNews(self, pCallback=None):
    """Ask server news"""
    params = [("mod", "servernews"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.append(tReq)


  def logout(self, pCallback=None):
    """Logout from server"""
    params = [("mod", "logout"), \
              ("session", self.session)]
    tReq = request(self.serverAddress + self.joinParameters(params), pCallback, self.globalVars.get("formName"))
    self.requests.insert(0, tReq)

