#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d, pspsnd
from core import graphicElement, functions, buttons

class toolbar(object):
  """Class toolbar"""
  def __init__(self, pGlobalVars, pFormVars):
    self.globalVars = pGlobalVars
    self.formVars = pFormVars
    self.data = None
    self.oldSelected = None
    self.selected = None
    self.lastSelected = None
    self.current = None
    self.elements = []
    self.positions = []
    self.posX = 0
    self.posY = 0
    self.width = 0
    self.height = 0
    self.background_image = None

    userData = __import__("userdata")
    #self.changeSound = pspsnd.Sound("themes/" + userData.THEME + "/menu_change.wav")
    #self.selectSound = pspsnd.Sound("themes/" + userData.THEME + "/menu_select.wav")
    #self.cancelSound = pspsnd.Sound("themes/" + userData.THEME + "/menu_cancel.wav")
    self.changeSound = pGlobalVars.get("sounds").get("menuChangeSound")
    self.selectSound = pGlobalVars.get("sounds").get("menuSelectSound")
    self.cancelSound = pGlobalVars.get("sounds").get("menuCancelSound")

    self.themeOptions = __import__("themes." + userData.THEME, None, None, userData.THEME)

  def getSelected(self):
    """Returns the selected element"""
    if self.selected and len(self.selected) > 0:
      for tElement in self.elements:
        if tElement.value == self.selected:
          return tElement
    return None


  def getElement(self, pvalue):
    """Returns the an element"""
    for tElement in self.elements:
      if tElement.value.split("|")[0] == pvalue or tElement.value == pvalue:
        return tElement
    return None


  def selectPrevious(self):
    """Selects previous menu"""
    next = False
    tElements = self.elements[:]
    tElements.extend(self.elements[:])
    tElements.reverse()
    for i, tElement in enumerate(tElements):
      #Controllo se e' abilitato:
      if next and tElement.enabled:
        #Controllo se e' visibile:
        if not tElement.visible:
          self.createGraphicElements(self.formVars.get("formGraphics"))
          #self.scrollDown()

        self.oldSelected = self.selected
        self.selected = tElement.value
        tElement.trigger_enter(self, self.globalVars, self.formVars, tElement)
        return True

      if tElement.value == self.selected:
        if i < len(tElements) - 1:
          next = True
        else:
          return False


  def selectNext(self):
    """Selects next menu"""
    next = False
    tElements = self.elements[:]
    tElements.extend(self.elements[:])
    for i, tElement in enumerate(tElements):
      #Controllo se e' abilitato:
      if next and tElement.enabled:
        #Controllo se e' visibile:
        if not tElement.visible:
          self.createGraphicElements(self.formVars.get("formGraphics"))
          #self.scrollUp()

        self.oldSelected = self.selected
        self.selected = tElement.value
        tElement.trigger_enter(self, self.globalVars, self.formVars, tElement)
        return True

      if tElement.value == self.selected:
        if i < len(tElements) - 1:
          next = True
        else:
          return False


  def createGraphicElements(self, pGraphics):
    """Create toolbar's graphic elements"""
    #Rettangolo:
    graph = graphicElement()
    graph.name = "TOOLBAR"
    graph.priority = 9999
    if self.background_image:
      graph.image = psp2d.Image(self.background_image)
    else:
      graph.data = [self.width, self.height, self.themeOptions.MENU_OUTERCOLOR, self.themeOptions.MENU_INNERCOLOR, self.themeOptions.MENU_CORNERCOLOR, self.themeOptions.MENU_BORDERWIDTH, self.themeOptions.MENU_ALPHA]
      graph.image = functions.drawRect(self.parent, graph)
    graph.posX = self.posX
    graph.posY = self.posY

    #Calcolo la distanza tra gli elementi:
    totWidth = 0
    for element in self.elements:
      #Controllo che font usare:
      if element.value in [self.selected, self.current]:
        tFont = self.globalVars.get("fonts").get("fontMenuSel")
      else:
        tFont = self.globalVars.get("fonts").get("fontMenu")
      totWidth += functions.textWidth(tFont, element.text)
      if element.icon:
        totWidth += element.icon.width
    dist = (self.width - totWidth - 2 * self.themeOptions.MENU_BORDERWIDTH) / (len(self.elements) - 1)

    #Loop tra gli elementi:
    tX = self.themeOptions.MENU_BORDERWIDTH
    tY = (self.height - functions.textHeight(self.globalVars.get("fonts").get("fontMenu"), "I")) / 2 - 2
    for element in self.elements:
      element.visible = True
      #Icona:
      if element.icon:
        graph.image.blit(element.icon, 0, 0, element.icon.width, element.icon.height, tX, (self.height - element.icon.height) / 2, True)
        tX += element.icon.width
      #Controllo che font usare:
      if element.value in [self.selected, self.current]:
        tFont = self.globalVars.get("fonts").get("fontMenuSel")
        #Testo di aiuto:
        if self.selected != None \
        and self.selected == element.value:
          self.globalVars.set(element.helpText, "message")
      else:
        tFont = self.globalVars.get("fonts").get("fontMenu")
      testo = element.text
      tFont.drawText(graph.image, tX, tY + self.themeOptions.MENU_INTERLINE, testo)

      #Mi salvo la posizione:
      element.posX = tX
      tX += functions.textWidth(tFont, testo) + dist
    pGraphics.set(graph)


  def button_down(self, pbuttons):
    #Gestione tasti toolbar
    if buttons.left in pbuttons:
      if self.selectPrevious():
        sound = self.changeSound
        sound.start()
        self.createGraphicElements(self.formVars.get("formGraphics"))
    elif buttons.right in pbuttons:
      if self.selectNext():
        sound = self.changeSound
        sound.start()
        self.createGraphicElements(self.formVars.get("formGraphics"))
    elif buttons.circle in pbuttons:
      self.trigger_cancel(self, self.globalVars, self.formVars, self.getSelected())
    elif buttons.cross in pbuttons:
      selected = self.getSelected()
      if selected:
        selected.trigger_select(self, self.globalVars, self.formVars, selected)


  def trigger_cancel(self, globalVars, formVars, menu, element):
    pass

