#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d, pspsnd
from core import graphicElement, animation, functions, buttons
from time import time
from os import name

class vKeyboard(object):
  """Class virtual Kayboard"""
  def __init__(self, pGlobalVars, pFormVars, pFormGraphics):
    self.globalVars = pGlobalVars
    self.formVars = pFormVars
    self.formGraphics = pFormGraphics
    self.mode = 0
    self.posX = 0
    self.posY = 0
    self.current_x = 1
    self.current_y = 1
    self.graphicIndex = []
    self.showTextBox = True
    self.maskChar = None
    self.text = ""
    self.maxLength = None
    self.lastLetterTime = 0
    self.returnGraphicElement = None
    self.destroy = False
    self.callbackFunction = None

    self.userData = self.globalVars.get("userData")
    self.selectedImages = ["vk_keys_s.png",	"vk_keys_s_c.png",	"vk_nums_s.png", "vk_nums_s_c.png"]
    self.selectedImage = psp2d.Image('themes/' + self.userData.THEME + "/" + self.selectedImages[self.mode])

    self.unselectedImages = ["vk_keys.png",	"vk_keys_c.png",	"vk_nums.png", "vk_nums_c.png"]
    self.unselectedImage = psp2d.Image('themes/' + self.userData.THEME + "/" + self.unselectedImages[self.mode])

    self.confirmSound = pspsnd.Sound("themes/" + self.userData.THEME + "/vk_confirm.wav")
    self.clickSound = pspsnd.Sound("themes/" + self.userData.THEME + "/vk_click.wav")
    self.errorSound = pspsnd.Sound("themes/" + self.userData.THEME + "/vk_error.wav")
    self.deleteSound = pspsnd.Sound("themes/" + self.userData.THEME + "/vk_delete.wav")

    self.modeChar = [[[ ",abc",  ".def","!ghi" ],
                      [ "-jkl","\bm n", "?opq" ],
                      [ "(rst",  ":uvw",")xyz" ]],
                    [[ "^ABC",  "@DEF","*GHI" ],
                      [ "_JKL","\bM N", "\"OPQ" ],
                      [ "=RST",  ";UVW","/XYZ" ]],
                    [[ "1111","2222","3333" ],
                      [ "4444",  "\b5 5","6666" ],
                      [ "7777","8888", "9909" ]],
                    [[ "'(.)",  "\"<'>","-[_]" ],
                      [ "!{?}","\b\0 \0", "+\\=/" ],
                      [ ":@;#",  "~$`%","*^|&" ]]]


  def createGraphicElement(self, pAnimate = None):
    """Creates the graphic element"""
    graphicElements = []
    graphicIndex = []

    graph = self.formGraphics.get("V_KEYBOARD")
    if not graph:
      graph = graphicElement()
      graph.name = "V_KEYBOARD"
      graph.priority = 99999
      graph.posX = self.posX
      graph.posY = self.posY

    #Preparo l'immagine:
    graph.image = None
    image = psp2d.Image(164,164)
    if name == "psp":
      image.clear(psp2d.Color(0,0,0))
    image.blit(self.unselectedImage, 0, 0, self.unselectedImage.width, self.unselectedImage.height, 7, 7, True)

    sezione = psp2d.Image(64, 64)
    if name == "psp":
      sezione.clear(psp2d.Color(0,0,0))
    sezione.blit(self.selectedImage, self.current_x * 64, self.current_y * 64, 64, 64, 0, 0, True)
    image.blit(sezione, 0, 0, -1, -1, self.current_x * 50, self.current_y * 50, True)

    graph.image = image
    graphicElements.append(graph)
    graphicIndex.append(graph.name)

    #Rettangolo:
    graph = self.formGraphics.get("V_KEYBOARD_SELECTOR")
    if not graph:
      graph = graphicElement()
      graph.name = "V_KEYBOARD_SELECTOR"
      graph.priority = 99999
      graph.posX = self.posX + self.current_x * 50
      graph.posY = self.posY + self.current_y * 50
      graph.image = psp2d.Image('themes/' + self.userData.THEME + "/vk_selector.png")
    else:
      anim = animation()
      anim.name = graph.name
      anim.lockKeyboard = False
      anim.data = [self.posX + self.current_x * 50, self.posY + self.current_y * 50, 99]
      anim.code = functions.moveFromTo
      graph.animation = anim
    graphicElements.append(graph)
    graphicIndex.append(graph.name)

    #Text box:
    if self.showTextBox:
      graph = self.formGraphics.get("V_KEYBOARD_TEXTBOX")
      if not graph:
        graph = graphicElement()
        graph.name = "V_KEYBOARD_TEXTBOX"
        graph.priority = 99990
        graph.posX = self.posX
        graph.posY = self.posY + 164
        graph.image = psp2d.Image('themes/' + self.userData.THEME + '/vk_text.png')
      graphicElements.append(graph)
      graphicIndex.append(graph.name)

    #Text:
    if self.showTextBox:
      graph = self.formGraphics.get("V_KEYBOARD_TEXT")
      if not graph:
        graph = graphicElement()
        graph.priority = 99999
        graph.name = "V_KEYBOARD_TEXT"
        graph.posX = self.posX + self.globalVars.get("themeOptions").MENU_BORDERWIDTH
        graph.posY = self.posY + 164 + self.globalVars.get("themeOptions").MENU_BORDERWIDTH
        graph.font = self.globalVars.get("fonts").get("fontMessage")
      if graph.text != self.text:
        graph.image = None
        #Controllo che ci stia:
        if self.maskChar:
          tText = ''.rjust(len(self.text), self.maskChar)
        else:
          tText = self.text
        tWidth = functions.textWidth(graph.font, tText)
        if tWidth > 164 - 2 * self.globalVars.get("themeOptions").MENU_BORDERWIDTH:
          while tWidth > 158:
            tText = tText[1:]
            tWidth = functions.textWidth(graph.font, "..." + tText[3:])
          graph.text = "..." + tText[3:]
        else:
          graph.text = tText
      graphicElements.append(graph)
      graphicIndex.append(graph.name)

    #Animazione di entrata:
    if pAnimate:
      for i, graph in enumerate(graphicElements):
        anim = animation()
        anim.name = graph.name
        anim.front = True
        anim.priority = 100
        if i == len(graphicElements) - 1:
          anim.exclusive = True
          anim.priority = -100
        if pAnimate == "BOTTOM":
          anim.data = [graph.posX, graph.posY, 99]
          graph.posY = graph.posY + 272
        elif pAnimate == "LEFT":
          anim.data = [graph.posX, graph.posY, 99]
          graph.posX = graph.posX - 480
        elif pAnimate == "RIGHT":
          anim.data = [graph.posX, graph.posY, 99]
          graph.posX = 480 + graph.posX
        anim.code = functions.moveFromTo
        graph.animation = anim

    for graph in graphicElements:
      self.formGraphics.set(graph)
    self.graphicIndex = graphicIndex


  def button_down(self, pButtons):
    """Button down on virtual keyboard"""
    crea = False
    oldText = self.text
    #Annulla:
    if buttons.select in pButtons:
      self.deleteSound.start()
      for name in self.graphicIndex:
        tGraph = self.formGraphics.get(name)
        if tGraph:
          tGraph.name = "DEL_" + tGraph.name
      self.graphicIndex = []
      self.formVars.set(None, "formKeyboard")
      if self.returnGraphicElement:
        self.returnGraphicElement.image = None
        self.returnGraphicElement.text = None
      #Chiamo la funzione di callback:
      if self.callbackFunction:
        self.callbackFunction(False, self.text)
      return

    #Conferma testo:
    if buttons.start in pButtons:
      self.confirmSound.start()
      if self.returnGraphicElement:
        self.returnGraphicElement.image = None
        self.returnGraphicElement.text = self.text
      for name in self.graphicIndex:
        tGraph = self.formGraphics.get(name)
        if tGraph:
          tGraph.name = "DEL_" + tGraph.name
      self.graphicIndex = []
      self.formVars.set(None, "formKeyboard")
      #Chiamo la funzione di callback:
      if self.callbackFunction:
        self.callbackFunction(True, self.text)
      return

    if buttons.left_trigger in pButtons and time() - self.lastLetterTime >= 0.3:
      #Cambio tastiera:
      if self.mode > 0:
        self.mode -= 1
      else:
        self.mode = 3
      self.selectedImage = None
      self.selectedImage = psp2d.Image('themes/' + self.userData.THEME + "/" + self.selectedImages[self.mode])
      self.unselectedImage = None
      self.unselectedImage = psp2d.Image('themes/' + self.userData.THEME + "/" + self.unselectedImages[self.mode])
      self.lastLetterTime = time()
      self.createGraphicElement()
    elif buttons.right_trigger in pButtons and time() - self.lastLetterTime >= 0.3:
      if self.mode < 3:
        self.mode += 1
      else:
        self.mode = 0
      self.selectedImage = psp2d.Image('themes/' + self.userData.THEME + "/" + self.selectedImages[self.mode])
      self.unselectedImage = psp2d.Image('themes/' + self.userData.THEME + "/" + self.unselectedImages[self.mode])
      self.lastLetterTime = time()
      self.createGraphicElement()

    #Posizionamento:
    old_x = self.current_x
    old_y = self.current_y
    if buttons.left in pButtons:
      self.current_x = 0
    elif buttons.right in pButtons:
      self.current_x = 2
    else:
      self.current_x = 1

    if buttons.up in pButtons:
      self.current_y = 0
    elif buttons.down in pButtons:
      self.current_y = 2
    else:
      self.current_y = 1

    if self.current_x <> old_x \
    or self.current_y <> old_y:
      crea = True

    #Controllo il testo:
    if self.text == None:
      self.text = ""

    #Selezione lettera:
    if buttons.cross in pButtons and time() - self.lastLetterTime >= 0.3:
      #Controllo lunghezza massime:
      if self.maxLength and len(self.text) >= self.maxLength:
        self.errorSound.start()
      else:
        self.clickSound.start()
        self.text = self.text + self.modeChar[self.mode][self.current_y][self.current_x][2]
        crea = True
      self.lastLetterTime = time()
    elif buttons.square in pButtons and time() - self.lastLetterTime >= 0.3:
      #Controllo lunghezza massima:
      if self.maxLength and len(self.text) >= self.maxLength:
        self.errorSound.start()
      else:
        self.clickSound.start()
        self.text = self.text + self.modeChar[self.mode][self.current_y][self.current_x][1]
        crea = True
      self.lastLetterTime = time()
    elif buttons.triangle in pButtons and time() - self.lastLetterTime >= 0.3:
      #Controllo lunghezza massima:
      if self.maxLength and len(self.text) >= self.maxLength:
        self.errorSound.start()
      else:
        if self.modeChar[self.mode][self.current_x][self.current_y][0] == "\b":
          if len(self.text) > 0:
            self.deleteSound.start()
            self.text = self.text[:-1]
        else:
          self.clickSound.start()
          self.text = self.text + self.modeChar[self.mode][self.current_y][self.current_x][0]
        crea = True
      self.lastLetterTime = time()
    elif buttons.circle in pButtons and time() - self.lastLetterTime >= 0.3:
      #Controllo lunghezza massime:
      if self.maxLength and len(self.text) >= self.maxLength:
        self.errorSound.start()
      else:
        self.clickSound.start()
        self.text = self.text + self.modeChar[self.mode][self.current_y][self.current_x][3]
        crea = True
      self.lastLetterTime = time()

    #Aggiorno l'oggetto grafico:
    if self.text != oldText and self.returnGraphicElement:
      self.returnGraphicElement.image = None
      self.returnGraphicElement.text = self.text

    if crea:
      self.createGraphicElement()

