#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#Common functions:
import psp2d, pspsnd
import sys
import types
from math import sqrt, pow
from time import sleep
from os import name

def get_refcounts():
  d = {}
  sys.modules
  # collect all classes
  for m in sys.modules.values():
      for sym in dir(m):
          o = getattr (m, sym)
          if type(o) is types.ClassType:
              d[o] = sys.getrefcount (o)
  # sort by refcount
  pairs = map (lambda x: (x[1],x[0]), d.items())
  pairs.sort()
  pairs.reverse()
  return pairs


def dump_refcounts():
  #Cerco il primo nome libero:
  i = 0
  while True:
    i += 1
    try:
      refcountsLog  = file("gb_refcounts_" + str(i) + ".log", "r")
      refcountsLog.close()
    except IOError:
      break
  refcountsLog = file("gb_refcounts_" + str(i) + ".log", "w")

  for n, c in get_refcounts():
      refcountsLog.write('%10d %s' % (n, c.__name__) + "\n")
  refcountsLog.close()


def dump_garbage(pParent):
  """Show garbage"""
  #Cerco il primo nome libero:
  i = 0
  while True:
    i += 1
    try:
      garbageLog  = file("gb_coll_" + str(i) + ".log", "r")
      garbageLog.close()
    except IOError:
      break
  garbageLog = file("gb_coll_" + str(i) + ".log", "w")
  # force collection
  pParent.gc.collect()

  if len(pParent.gc.garbage) > 0:
    print "-" * 78
    garbageLog.write("-" * 78 + "\n")
    print "GARBAGE OBJECTS:"
    garbageLog.write("GARBAGE OBJECTS:")
    for x in pParent.gc.garbage:
        s = str(x)
        if len(s) > 80: s = s[:80]
        print type(x),"\n  ", s
        garbageLog.write("\n")
        garbageLog.write(str(type(x)) + "\n")
        garbageLog.write(s + "\n")
    garbageLog.flush()
    print "-" * 78
  garbageLog.close()


def dump_objects(pParent):
  """Dumps all objects"""
  #Cerco il primo nome libero:
  i = 0
  while True:
    i += 1
    try:
      objectDump = file("objDump_" + str(i) + ".log", "r")
      objectDump.close()
    except IOError:
      break

  objectDump = file("objDump_" + str(i) + ".log", "w")
  objectDump.write("-" * 78 + "\n")
  objs = pParent.gc.get_objects()
  for o in objs:
    try:
      objectDump.write(repr(o) + "\n")
    except:
      pass
  objectDump.close()


def dump_memory(pParent):
  import code, sys
  from sizer import scanner, formatting, operations
  #Cerco il primo nome libero:
  i = 0
  while True:
    i += 1
    try:
      memDump = file("memDump_" + str(i) + ".log", "r")
      memDump.close()
    except IOError:
      break
  memDump = file("memDump_" + str(i) + ".log", "w")

  objs = scanner.Objects()
  sys.stdout = memDump
  formatting.printsizes(objs)
  try:
    if pParent.lastObjs:
      print "-" * 80
      print " DIFFERENZA CON DUMP PRECEDENTE"
      print "-" * 80
      diffObjs = operations.diff(objs, pParent.lastObjs)
      formatting.printsizes(diffObjs)
      #for obj in diffObjs:
      #  print objs[id(obj)].obj
  except AttributeError:
    pass
  pParent.lastObjs = objs
  sys.stdout = sys.__stdout__
  memDump.close()


def textWidth(pfont, ptext):
  """Calcolo la larghezza di un testo"""
  if ptext:
    return pfont.textWidth(ptext)
  else:
    return 0


def textHeight(pfont, ptext):
  """Calcolo la altezza di un testo"""
  if ptext:
    return pfont.textHeight(ptext)
  else:
    return 0


def drawRect(target):
  """Draws a rectangle"""
  w = target.data[0]
  h = target.data[1]
  outerColor = target.data[2]
  innerColor = target.data[3]
  cornerColor = target.data[4]
  borderWidth = target.data[5]
  alpha = target.data[6]
  if len(target.data) >= 8:
    corner = target.data[7]
  else:
    corner = True
  #Immagine esterna:
  outerImage = psp2d.Image(w, h)
  outerImage.clear(psp2d.Color(outerColor[0], outerColor[1], outerColor[0], alpha))

  #Immagine interna:
  innerImage = psp2d.Image(w - 2 * borderWidth, h - 2 * borderWidth)
  innerImage.clear(psp2d.Color(innerColor[0], innerColor[1], innerColor[0], alpha))
  outerImage.blit(innerImage, 0, 0, innerImage.width, innerImage.height, borderWidth, borderWidth)
  del innerImage

  #Angoli:
  if corner == True:
    cornerImage = psp2d.Image(borderWidth * 2, borderWidth * 2)
    cornerImage.clear(psp2d.Color(cornerColor[0], cornerColor[1], cornerColor[0], alpha))
    outerImage.blit(cornerImage, 0,0, cornerImage.width, cornerImage.height, 0, 0)
    outerImage.blit(cornerImage, 0,0, cornerImage.width, cornerImage.height, outerImage.width - cornerImage.width, outerImage.height - cornerImage.height)
    outerImage.blit(cornerImage, 0,0, cornerImage.width, cornerImage.height, outerImage.width - cornerImage.width, 0)
    outerImage.blit(cornerImage, 0,0, cornerImage.width, cornerImage.height, 0, outerImage.height - cornerImage.height)
    del cornerImage

    #Angolo Orizzontale:
    horLineImage = psp2d.Image(borderWidth * 3, borderWidth)
    horLineImage.clear(psp2d.Color(cornerColor[0], cornerColor[1], cornerColor[0], alpha))
    outerImage.blit(horLineImage, 0, 0, horLineImage.width, horLineImage.height, 0, 0)
    outerImage.blit(horLineImage, 0, 0, horLineImage.width, horLineImage.height, outerImage.width - horLineImage.width, outerImage.height - horLineImage.height)
    outerImage.blit(horLineImage, 0, 0, horLineImage.width, horLineImage.height, outerImage.width - horLineImage.width, 0)
    outerImage.blit(horLineImage, 0, 0, horLineImage.width, horLineImage.height, 0, outerImage.height - horLineImage.height)
    del horLineImage

    #Angolo Verticale:
    verLineImage = psp2d.Image(borderWidth, borderWidth * 3)
    verLineImage.clear(psp2d.Color(cornerColor[0], cornerColor[1], cornerColor[0], alpha))
    outerImage.blit(verLineImage, 0, 0, verLineImage.width, verLineImage.height, 0, 0)
    outerImage.blit(verLineImage, 0, 0, verLineImage.width, verLineImage.height, outerImage.width - verLineImage.width, outerImage.height - verLineImage.height)
    outerImage.blit(verLineImage, 0, 0, verLineImage.width, verLineImage.height, outerImage.width - verLineImage.width, 0)
    outerImage.blit(verLineImage, 0, 0, verLineImage.width, verLineImage.height, 0, outerImage.height - verLineImage.height)
    del verLineImage
  return outerImage


def fromCellToX(pCell):
  toX = (116 + 77 * (pCell % 3)) + 9
  return toX


def fromCellToY(pCell):
  if pCell in [0,1,2]:
    toY = 12
  elif pCell in [3,4,5]:
    toY = 88
  elif pCell in [6,7,8]:
    toY = 165
  return toY


def backToMainMenu(globalVars, formVars, menu, element):
  #Torno al main menu
  sound = globalVars.get("sounds").get("menuCancelSound")
  sound.start()
  globalVars.set("mdlMainMenu", "formName")
  globalVars.set("YES", "breakLoop")


def saveOptions(userData):
  #Salvo le opzioni:
  f = file("userdata/__init__.py", "w")
  f.write("#Config file:\n")
  f.write("import serverList\n")
  f.write("import customMsg\n")
  f.write("DEBUG=\"" + userData.DEBUG + "\"\n")
  f.write("DEBUG_SCREEN=\"" + userData.DEBUG_SCREEN + "\"\n")
  f.write("DEBUG_NET=\"" + userData.DEBUG_NET + "\"\n")
  f.write("LANG=\"" + userData.LANG + "\"\n")
  f.write("THEME=\"" + userData.THEME + "\"\n")
  f.write("SHOW_FPS=\"" + userData.SHOW_FPS + "\"\n")
  f.write("\n")
  f.write("#Quick match options:\n")
  f.write("QUICK_CARDSET=\"" + userData.QUICK_CARDSET + "\"\n")
  f.write("QUICK_LEVEL=\"" + userData.QUICK_LEVEL + "\"\n")
  f.write("QUICK_SAME=\"" + userData.QUICK_SAME + "\"\n")
  f.write("QUICK_SAMEWALL=\"" + userData.QUICK_SAMEWALL + "\"\n")
  f.write("QUICK_PLUS=\"" + userData.QUICK_PLUS + "\"\n")
  f.write("QUICK_COMBO=\"" + userData.QUICK_COMBO + "\"\n")
  f.write("QUICK_ELEMENTAL=\"" + userData.QUICK_ELEMENTAL + "\"\n")
  f.write("QUICK_OPEN=\"" + userData.QUICK_OPEN + "\"\n")
  f.write("QUICK_SUDDENDEATH=\"" + userData.QUICK_SUDDENDEATH + "\"\n")
  f.write("\n")
  f.write("#NET OPTIONS:\n")
  f.write("CONNECTION=\"" + userData.CONNECTION + "\"\n")
  f.write("SERVER=\"" + userData.SERVER + "\"\n")
  f.write("USERNAME=\"" + userData.USERNAME + "\"\n")
  f.write("PASSWORD=\"" + userData.PASSWORD + "\"\n")
  f.write("SAVEPASSWORD=\"" + userData.SAVEPASSWORD + "\"\n")
  f.write("CHATPOPUP=\"" + userData.CHATPOPUP + "\"\n")
  f.write("TIMEOUT_DECK=" + str(userData.TIMEOUT_DECK) + "\n")
  f.write("TIMEOUT_TRADE=" + str(userData.TIMEOUT_TRADE) + "\n")
  f.write("TIMEOUT_MOVE=" + str(userData.TIMEOUT_MOVE) + "\n")
  f.close()


def saveCustomMessages(customMsg):
  #Salvo i messagggi custom:
  f = file("userdata/customMsg.py", "w")
  f.write("#Custom messages to shout in a location:\n")
  f.write("SHOUTMSG=[]\n")
  for tMsg in customMsg.SHOUTMSG:
    f.write("SHOUTMSG.append(" + repr(tMsg) + ")\n")

  f.write("#Custom messages for chat:\n")
  f.write("CHATMSG=[]\n")
  for tMsg in customMsg.CHATMSG:
    f.write("CHATMSG.append(" + repr(tMsg) + ")\n")
  f.close()


def stringToServer(pString):
  """Preparo la stringa per il server"""
  pString = pString.replace(" ", "%20")
  pString = pString.replace("'", "&#39;")
  return pString


def stringFromServer(pString):
  """Preparo la stringa per il client"""
  #Tolgo pubblicita' server americano:
  pString = pString.strip()
  pString = pString.replace("<meta http-equiv=\"refresh\" content=\"400;url=http://www.hostico.net\">", "")
  pString = pString.replace("\n", "")

  pString = pString.replace("%20", " ")
  pString = pString.replace("&#39;", "'")
  return pString


def analyzeServerString(pString):
  """Analyze a server string"""
  if pString:
    pString = stringFromServer(pString)
    if pString[-5:] == "//%//":
      pString = pString[:-5]
    if pString[-2:] == "||":
      pString = pString[:-2]
    return pString.split("//%//")
  else:
    return []


def formatDateTime(pDateTime, pSeconds=False):
  """Format a datetime yyyymmddhhmmss"""
  if pDateTime:
    ret = pDateTime[6:8] + "/" + pDateTime[4:6] + "/" + pDateTime[0:4] + " "
    ret += pDateTime[8:10] + ":" + pDateTime[10:12]
    if pSeconds:
      ret += ":" + pDateTime[12:14]
    return ret
  else:
    return ""


def decodeCardSetID(pCardsetID):
  """Decode cardset id in multiplayer"""
  cardSetsID = [7, 8, 9, 10]
  cardSetsName = ["Final Fantasy VII", "Final Fantasy VIII", "Final Fantasy IX", "Final Fantasy X"]
  return cardSetsName[cardSetsID.index(pCardsetID)]


def drawLine(pImage, pColor, pFromX, pFromY, pToX, pToY):
  """Draw a line on an image"""
  image = pImage
  pixel = psp2d.Image(2,2)
  if name == "psp":
    pixel.clear(psp2d.Color(0,0,0))
  pixel.clear(pColor)
  deltaX = abs(pToX - pFromX)
  deltaY = abs(pToY - pFromY)
  posX = 0.0
  posY = 0.0
  if deltaX >= deltaY:
    for i in xrange(0, deltaX + 1):
      if pToX > pFromX:
        posX = pFromX + i
      else:
        posX = pFromX - i
      posY = round((pToY - pFromY) * (posX - pFromX) / (pToX - pFromX) + pFromY, 0)
      image.blit(pixel, 0, 0, pixel.width, pixel.height, int(posX), int(posY), True)
  else:
    for i in xrange(0, deltaY + 1):
      if pToY > pFromY:
        posY = pFromY + i
      else:
        posY = pFromY - i
      posX = round((posY - pFromY) * (pToX - pFromX) / (pToY - pFromY) + pFromX, 0)
      image.blit(pixel, 0, 0, pixel.width, pixel.height, int(posX), int(posY), True)


def waitSafeMenu(pMenu):
  """Wait for safe creating menu (for multiplayer)"""
  #Aspetto lo scroll:
  if pMenu:
    graph = pMenu.getGraphicElement("MENU_TEXT")
    while graph and graph.animation:
      sleep(0.005)
  #Aspetto la creazione:
  while pMenu.creating:
    sleep(0.005)


#-------------------------------------------------------------------------------------------------------------------------------------------------#
# Animations                                                                                                                                      #
#-------------------------------------------------------------------------------------------------------------------------------------------------#
def moveFromTo(globalVars, formVars, target):
  #Controllo se sono arrivato:
  if target.posX == target.animation.data[0] \
  and target.posY == target.animation.data[1]:
    #Fine animazione:
    target.animation.name = "DEL_" + target.animation.name
  else:
    #Calcolo la distanza:
    dist = sqrt(pow(target.animation.data[0] - target.posX, 2) + pow(target.animation.data[1] - target.posY, 2))

    #Controllo la direzione:
    dX = 0
    dY = 0
    deltaX = abs(target.posX - target.animation.data[0])
    deltaY = abs(target.posY - target.animation.data[1])
    if deltaX >= deltaY:
      if target.posX < target.animation.data[0]:
        dX = sqrt(dist)
        if dX == 0:
          dX = 1
      elif target.posX > target.animation.data[0]:
        dX = -sqrt(dist)
        if dX == 0:
          dX = -1
    else:
      if target.posY < target.animation.data[1]:
        dY = sqrt(dist)
        if dY == 0:
          dY = 1
      elif target.posY > target.animation.data[1]:
        dY = -sqrt(dist)
        if dY == 0:
          dY = -1

    #Limito per fluidita' i valori:
    if len(target.animation.data) == 3:
      max = target.animation.data[2]
    else:
      max = 9

    if dX > max:
      dX = max
    elif dX < -max:
      dX = -max
    elif dY > max:
      dY = max
    elif dY < -max:
      dY = -max

    #Calcolo nuova posizione:
    if dX <> 0:
      newY = (target.animation.data[1] - target.posY) / (target.animation.data[0] - target.posX) * (target.posX + dX - target.posX) + target.posY
      newY = newY - dY
      newX = target.posX + dX
    elif dY <> 0:
      newX = dY / (target.animation.data[1] - target.posY) * (target.animation.data[0] - target.posX) + target.posX
      newX = newX - dX
      newY = target.posY + dY
    target.posY = round(newY, 0)
    target.posX = round(newX, 0)


def moveFromToAndDelete(globalVars, formVars, target):
  """Move from x,y to x1,y1 and delete graphicElement"""
  moveFromTo(globalVars, formVars, target)
  #Controllo se sono arrivato:
  if target.posX == target.animation.data[0] \
  and target.posY == target.animation.data[1]:
    target.name = "DEL_" + target.name 


#Messaggi partita:
def specialMessage(globalVars, formVars, target):
  #Messaggio speciale
  if target.animation.data == None:
    sound = pspsnd.Sound("themes/" + globalVars.get("userData").THEME + "/special.wav")
    sound.start()
    target.animation.data = globalVars.get("currentFPSTime")
  elif globalVars.get("currentFPSTime") - target.animation.data > 0.5:
    target.name = "DEL_" + target.name


def message(globalVars, formVars, target):
  #Messaggio generico:
  if target.animation.data == None:
    target.animation.data = globalVars.get("currentFPSTime")
  elif globalVars.get("currentFPSTime") - target.animation.data > 0.5:
    target.visible = False
    #target.animation.name = "DEL_" + target.animation.name
    target.name = "DEL_" + target.name


def endMessage(globalVars, formVars, target):
  #Messaggio finale:
  if target.animation.data[4] == None:
    target.animation.data[4] = globalVars.get("currentFPSTime")

  if target.animation.data[0] > target.animation.data[1] and target.animation.data[2] == None:
    #Vinto:
    globalVars.set(pspsnd.Sound("themes/" + globalVars.get("userData").THEME + "/win.wav"), "playingSound")
    globalVars.get("playingSound").start()
    target.animation.data[2] = True
  elif target.animation.data[2] == False:
    target.animation.data[2] = True
    target.animation.data.append(globalVars.get("currentFPSTime"))
  elif target.animation.data[3] == "SUDDEN" and globalVars.get("currentFPSTime") - target.animation.data[4] > 0.5:
    #Sudden death:
    globalVars.set(pspsnd.Sound("themes/" + globalVars.get("userData").THEME + "/special.wav"), "playingSound")
    globalVars.get("playingSound").start()
    target.animation.data[5]()
    target.name = "DEL_" + target.name
  elif target.animation.data[3] == "TRADE" and globalVars.get("currentFPSTime") - target.animation.data[4] > 0.5:
    #Trading carte
    target.animation.data[5]()
    target.name = "DEL_" + target.name


#Rotazione carta + suono:
def changeOwnerSound(globalVars, formVars, target):
  vel = 8
  if len(target.animation.data) == 4:
    target.animation.data.append(0)
    target.animation.data.append(target.posX)

    target.width = target.image.width
    target.rotating = 0
    target.direction = -1

    #Suono:
    sound = pspsnd.Sound("themes/" + globalVars.get("userData").THEME + "/flip.wav")
    sound.start()

  #Ruoto la carta:
  if target.width > vel and target.direction == -1 \
  or target.width < 74 and target.direction == 1:
    target.posX += -target.direction * (vel / 2)
    target.width += vel * target.direction
  elif (target.width <= vel or target.width == 74):
    if target.rotating < 3:
      target.rotating += 1
      target.direction = target.direction * -1
    else:
      target.animation.name = "DEL_" + target.animation.name

    target.animation.data[4] = 0
    target.image = target.animation.data[target.rotating]


def changeOwnerVerticalSound(globalVars, formVars, target):
  vel = 8
  if len(target.animation.data) == 4:
    target.animation.data.append(0)
    target.animation.data.append(target.posX)

    target.height = target.image.height
    target.rotating = 0
    target.direction = -1

    #Suono:
    sound = pspsnd.Sound("themes/" + globalVars.get("userData").THEME + "/flip.wav")
    sound.start()

  #Ruoto la carta:
  if target.height > vel and target.direction == -1 \
  or target.height < 74 and target.direction == 1:
    target.posY += -target.direction * (vel / 2)
    target.height += vel * target.direction
  elif (target.height <= vel or target.height == 74):
    if target.rotating < 3:
      target.rotating += 1
      target.direction = target.direction * -1
    else:
      target.animation.name = "DEL_" + target.animation.name

    target.animation.data[4] = 0
    target.image = target.animation.data[target.rotating]


#Rotazione carta:
def changeOwner(globalVars, formVars, target):
  vel = 8
  if len(target.animation.data) == 4:
    target.animation.data.append(0)
    target.animation.data.append(target.posX)

    target.width = target.image.width
    target.rotating = 0
    target.direction = -1

  #Ruoto la carta:
  if target.width > vel and target.direction == -1 \
  or target.width < 74 and target.direction == 1:
    target.posX += -target.direction * (vel / 2)
    target.width += vel * target.direction
  elif (target.width <= vel or target.width == 74):
    if target.rotating < 3:
      target.rotating += 1
      target.direction = target.direction * -1
    else:
      target.animation.name = "DEL_" + target.animation.name

    if target.animation.data[target.rotating]:
      target.animation.data[4] = 0
      target.image = target.animation.data[target.rotating]
    else:
      target.animation.name = "DEL_" + target.animation.name


def changeOwnerVertical(globalVars, formVars, target):
  vel = 8
  if len(target.animation.data) == 4:
    target.animation.data.append(0)
    target.animation.data.append(target.posX)

    target.height = target.image.height
    target.rotating = 0
    target.direction = -1

  #Ruoto la carta:
  if target.height > vel and target.direction == -1 \
  or target.height < 74 and target.direction == 1:
    target.posY += -target.direction * (vel / 2)
    target.height += vel * target.direction
  elif (target.height <= vel or target.height == 74):
    if target.rotating < 3:
      target.rotating += 1
      target.direction = target.direction * -1
    else:
      target.animation.name = "DEL_" + target.animation.name

    if target.animation.data[target.rotating]:
      target.animation.data[4] = 0
      target.image = target.animation.data[target.rotating]
    else:
      target.animation.name = "DEL_" + target.animation.name


def selectorAnim(globalVars, formVars, target):
  """Animation for menu selector"""
  target.posX += target.animation.data[0]
  target.animation.data[1] += target.animation.data[0]
  if abs(target.animation.data[1]) >= 5:
    target.animation.data = [-1 * target.animation.data[0], 0]


def selectorTopAnim(globalVars, formVars, target):
  """Animation for trade card selector"""
  target.posY += target.animation.data[0]
  target.animation.data[1] += target.animation.data[0]
  if abs(target.animation.data[1]) >= 5:
    target.animation.data = [-1 * target.animation.data[0], 0]
