#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d
from core import graphicElement, dataStore, animation, functions, menu, menuElement, buttons, deck, cardSet, TTplayer

from time import time, sleep
from os import name
from traceback import format_exc

def show(pGlobalVars, pFormVars, pButtonDown):
  """Show form"""
  screen = psp2d.Screen()
  controller = psp2d.Controller
  blitScreen = screen.blit

  lastFPSTime = None
  lastFPSUpdate = time()
  targetFPSTime = 1.0/60.0

  oldPad = None
  padTime = time()

  msg = pGlobalVars.get("commonGraphics").get("CE_FORM_MESSAGE")
  fps = pGlobalVars.get("commonGraphics").get("CE_FORM_FPS")
  while 1:
    lockKeyboard = False
    currentFPSTime = time()
    pGlobalVars.set(lockKeyboard, "lockKeyboard")
    pGlobalVars.set(currentFPSTime, "currentFPSTime")
    #Sfondo:
    bkg = pFormVars.get("bkgImage")
    bkgPosX = pFormVars.get("bkgImagePosX")
    bkgPosY = pFormVars.get("bkgImagePosY")
    if bkg != None:
      blitScreen(bkg, 0, 0, bkg.width, bkg.height, bkgPosX, bkgPosY)

    graphicElementsBlitted = 0
    animationExecuted = 0

    #Messaggio:
    if msg.text != pGlobalVars.get("message"):
      tWidth = functions.textWidth(pGlobalVars.get("fonts").get("fontMessage"), pGlobalVars.get("message"))
      #Centro la scritta nello spazio a disposizione:
      tX = (480 - tWidth) / 2
      msg.posX = tX
      msg.text = pGlobalVars.get("message")
      msg.image = None

    #Elementi grafici:
    animations = []
    graphics = [(graph.priority, graph.name, graph) for graph in pFormVars.get("formGraphics").dataDic.values() + pGlobalVars.get("commonGraphics").dataDic.values()]
    graphics.sort()
    for tGraph in graphics:
      graph = tGraph[2]
      if graph.visible \
      and (not graph.animation or not graph.animation.front):
        graphicElementsBlitted += 1
        if graph.image:
          blit(screen, graph)
        elif graph.text:
          graph.buildTextImage()
          blit(screen, graph)

      #Rimuovo gli elementi da rimuovere:
      if str(graph.name)[0:4] == "DEL_":
        graph.animation = None
        if not pFormVars.get("formGraphics").remove(graph.name[4:]):
          pGlobalVars.get("commonGraphics").remove(graph.name[4:])
      #Mi salvo l'animazione:
      elif graph.animation:
        animations.append((-graph.animation.priority, graph.name, graph))

    #Animazioni:
    animations.sort()
    for tGraph in animations:
      animationExecuted += 1
      graph = tGraph[2]
      #Se e' in primo piano rivisualizzo l'immagine:
      if graph.animation.front and graph.visible:
        if graph.image:
          blit(screen, graph)
        elif graph.text:
          graph.buildTextImage()
          blit(screen, graph)

      #Controllo il blocco della tastiera:
      if graph.animation.lockKeyboard and not lockKeyboard:
        pGlobalVars.set(True, "lockKeyboard")
        lockKeyboard = True

      #Eseguo animazione:
      exclusive = graph.animation.exclusive
      try:
        graph.animation.code(pGlobalVars, pFormVars, graph)
      except:
        if pGlobalVars.get("debug"):
          pGlobalVars.get("logFile").log(format_exc())
        graph.animation.name = "DEL_" + graph.animation.name
      #Controllo se e' da rimuovere:
      if str(graph.name)[0:4] == "DEL_":
        graph.animation = None
        if not pFormVars.get("formGraphics").remove(graph.name[4:]):
          pGlobalVars.get("commonGraphics").remove(graph.name[4:])
        graph.__dict__.clear()
      elif str(graph.animation.name)[0:4] == "DEL_":
        graph.animation = None
      #Se l'animazione e' esclusiva non eseguo le altre:
      if exclusive:
        break

    #FPS:
    if lastFPSTime:
      delta = time() - lastFPSTime
      #Limito gli FPS:
      if delta < targetFPSTime:
        toSleep = targetFPSTime - delta
        sleep(toSleep)
      #Aggiorno a video:
      if pGlobalVars.get("userData").SHOW_FPS == "YES":
        if currentFPSTime - lastFPSUpdate >= 0.5:
          delta = time() - lastFPSTime
          FPS = 1.0 / delta
          fps.text = "FPS: " + str(int(FPS))
          fps.image = None
          lastFPSUpdate = time()

    lastFPSTime = time()
    screen.swap()

    #Controllo il break:
    if pGlobalVars.get("breakLoop") == "YES":
      pGlobalVars.set("NO", "breakLoop")
      break

    #Con le animazioni non leggo i tasti:
    if lockKeyboard:
      continue

    #Gestione tasti:
    pad = controller()
    #Chiamo la gestione dei tasti:
    tButtons = []
    if pad.up:
      tButtons.append(buttons.up)
    if pad.down:
      tButtons.append(buttons.down)
    if pad.left:
      tButtons.append(buttons.left)
    if pad.right:
      tButtons.append(buttons.right)
    if pad.circle:
      tButtons.append(buttons.circle)
    if pad.square:
      tButtons.append(buttons.square)
    if pad.cross:
      tButtons.append(buttons.cross)
    if pad.triangle:
      tButtons.append(buttons.triangle)
    if pad.select:
      tButtons.append(buttons.select)
    if pad.start:
      tButtons.append(buttons.start)
    if pad.l:
      tButtons.append(buttons.left_trigger)
    if pad.r:
      tButtons.append(buttons.right_trigger)

    if pad.analogX < 85 - 127:
      tButtons.append(buttons.left)
    elif pad.analogX > 170 - 127:
      tButtons.append(buttons.right)

    if pad.analogY < 85 - 127:
      tButtons.append(buttons.up)
    elif pad.analogY  > 170 - 127:
      tButtons.append(buttons.down)

    #Screenshot:
    if buttons.left_trigger in tButtons \
    and buttons.right_trigger in tButtons\
    and (oldPad ==  None or oldPad != tButtons or currentFPSTime - padTime >= 0.2):
      #Cerco il primo nome libero:
      i = 0
      if name == "psp":
        ext = ".png"
      else:
        ext = ".bmp"
      while True:
        i += 1
        try:
          f = file("screenshots/SCR_" + str(i) + ext, "r")
          f.close()
        except IOError:
          break
      screen.saveToFile("screenshots/SCR_" + str(i) + ext)
      oldPad = tButtons
      padTime = time()
      continue

    formKeyboard = pFormVars.get("formKeyboard")
    formHelp = pFormVars.get("helpItem")
    if formHelp and buttons.square in tButtons and not formKeyboard \
    or formHelp and formHelp.active:
      formHelp.button_down(tButtons)
    elif formKeyboard:
      formKeyboard.button_down(tButtons)
    else:
      if len(tButtons) > 0 \
      and (oldPad ==  None or oldPad != tButtons or currentFPSTime - padTime >= 0.2):
        oldPad = tButtons
        padTime = time()
        common_button_down(tButtons, pButtonDown, pGlobalVars, pFormVars)


def blit(screen, graph):
  """Blit a graphic element to screen"""
  #Controllo se esce dallo schermo:
  if graph.posX + graph.subWidth > 480 and graph.posX < 480:
    subWidth = int(480 - graph.posX)
    width = int(480 - graph.posX)
  else:
    subWidth = graph.subWidth
    width = graph.width

  if graph.posY + graph.subHeight > 272 and graph.posY < 272:
    subHeight = int(272 - graph.posY)
    height = int(272 - graph.posY)
  else:
    subHeight = graph.subHeight
    height = graph.height

  #Disegno a video:
  try:
    screen.blit(graph.image, graph.subX, graph.subY, subWidth, subHeight, int(graph.posX), int(graph.posY), graph.blend, width, height)
  except:
    try:
      globalVars.get("logFile").log("ERROR BLITTING: " + graph.name)
      globalVars.get("logFile").log("          posX: " + str(graph.posX))
      globalVars.get("logFile").log("          posY: " + str(graph.posY))
      globalVars.get("logFile").log("         width: " + str(width) + " (" + str(graph.image.width) + ")")
      globalVars.get("logFile").log("        height: " + str(height) + " (" + str(graph.image.height) + ")")
      globalVars.get("logFile").log("          subX: " + str(graph.subX))
      globalVars.get("logFile").log("          subY: " + str(graph.subY))
      globalVars.get("logFile").log("      subWidth: " + str(subWidth))
      globalVars.get("logFile").log("     subHeight: " + str(subHeight))
    except:
      pass


def common_button_down(pButtons, pButtonDown, pGlobalVars, pFormVars):
  #Gestione tasti form:
  if not pButtonDown(pButtons):
    #Dumps:
    if pGlobalVars.get("debug") and buttons.triangle in pButtons:
      #self.functions.dump_memory(self)
      #self.functions.dump_garbage(self)
      #self.functions.dump_objects(self)
      #functions.dump_refcounts()
      return

    formToolbar = pFormVars.get("formToolbar")
    formMenu = pFormVars.get("formMenu")
    if formToolbar and formToolbar.selected:
      #Gestione tasti toolbar:
      formToolbar.button_down(pButtons)
    elif formMenu:
      #Gestione tasti menu:
      formMenu.button_down(pButtons)


def confirm(globalVars, formVars, formGraphics, pWidth=150, pHeight=100, pPosX=-1, pPosY=-1):
  #Costruisco un menu di scelta si/no:
  tMenu = menu(formGraphics, globalVars, formVars)
  tMenu.name = "CONFIRM"

  tElement = menuElement()
  tElement.text = globalVars.get("messages").YES
  tElement.icon = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/gm_t_yes.png')
  tElement.value = "YES"
  tMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").NO
  tElement.icon = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/gm_t_no.png')
  tElement.value = "NO"
  tMenu.elements.append(tElement)

  tMenu.name = "CONFIRMMENU"
  tMenu.menuWidth = pWidth
  tMenu.menuHeight = pHeight
  if pPosX < 0:
    tMenu.posX = (480 - tMenu.menuWidth) / 2
  else:
    tMenu.posX = pPosX

  if pPosY < 0:
    tMenu.posY = (272 - tMenu.menuHeight) / 2
  else:
    tMenu.posY = pPosY

  tMenu.selected = tMenu.elements[0].value
  tMenu.menuAlignment = globalVars.get("themeOptions").MENU_ALIGN
  tMenu.menuVAlignment = globalVars.get("themeOptions").MENU_VALIGN
  return tMenu


def doLockKeyboard(globalVars, formVars):
  """Lock keyboard"""
  formGraphics = formVars.get("formGraphics")
  #Blocco la tastiera all'utente:
  globalVars.set(True, "lockKeyboard")
  wGraph = formGraphics.get("LOCKKEYBOARD")
  if not wGraph:
    wGraph = graphicElement()
    wGraph.name = "LOCKKEYBOARD"
  anim = animation()
  anim.name = wGraph.name
  anim.priority = 9999
  anim.lockKeyboard = True
  def code(globalVars, formVars, target):
    pass
  anim.code = code
  wGraph.animation = anim
  formGraphics.set(wGraph)


def doUnlockKeyboard(globalVars, formVars):
  """Unlock keyboard"""
  tGraph = formVars.get("formGraphics").get("LOCKKEYBOARD")
  if tGraph:
    tGraph.name = "DEL_" + tGraph.name


def waitMessage(globalVars, formVars, pMessage, pLockKeyboard=True, pTimeout=0, pToX=5):
  """Display a wait message"""
  formGraphics = formVars.get("formGraphics")
  #Messaggio di attesa:
  globalVars.set(True, "messageShown")
  graph = formGraphics.get("WAIT")
  if not graph:
    graph = graphicElement()
    graph.name = "WAIT"
    graph.priority = 100
  graph.font = globalVars.get("fonts").get("fontMessage")
  graph.image = None
  graph.text = pMessage

  #Nascondo il messaggio dopo n secondi:
  if pTimeout > 0:
    wGraph = formGraphics.get("HIDEWAIT")
    if not wGraph:
      wGraph = graphicElement()
      wGraph.name = "HIDEWAIT"
    anim = animation()
    anim.name = wGraph .name
    anim.priority = 999
    anim.data = [pTimeout, globalVars.get("currentFPSTime")]
    anim.lockKeyboard = False
    def code(globalVars, formVars, target):
      if globalVars.get("currentFPSTime") - target.animation.data[1] > target.animation.data[0]:
        hideWaitMessage(globalVars, formVars)
    anim.code = code
    wGraph.animation = anim
    formGraphics.set(wGraph)
  else:
    hide = formGraphics.get("HIDEWAIT")
    if hide:
      hide.name = "DEL_" + hide.name

  #Blocco la tastiera:
  if pLockKeyboard == True:
    doLockKeyboard(globalVars, formVars)
  else:
    doUnlockKeyboard(globalVars, formVars)

  tHeight = functions.textHeight(graph.font, graph.text)
  graph.posY = 249 - tHeight
  graph.posX = -functions.textWidth(graph.font, graph.text)
  anim = animation()
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.priority = 999
  anim.data = [pToX, graph.posY]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)


def hideWaitMessage(globalVars, formVars):
  """Hide the wait message"""
  formGraphics = formVars.get("formGraphics")
  globalVars.set(False, "messageShown")
  graph = formGraphics.get("WAIT")
  anim = animation()
  anim.priority = 9999
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.data = [-functions.textWidth(graph.font, graph.text), graph.posY]
  anim.code = functions.moveFromTo
  graph.animation = anim
  toDel = formGraphics.get("LOCKKEYBOARD")
  if toDel:
    toDel.name = "DEL_" + toDel.name
  toDel = formGraphics.get("HIDEWAIT")
  if toDel:
    toDel.name = "DEL_" + toDel.name


def updateWaitMessage(globalVars, formVars, pMessage):
  """Update the wait message"""
  waitM = formVars.get("formGraphics").get("WAIT")
  if waitM.text != pMessage:
    waitM.image = None
    waitM.text = pMessage


def createMessagesGE(globalVars, formVars):
  """Create messages graphic element"""
  graph = graphicElement()
  graph.name = "MESSAGES"
  graph.priority = 0
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_bg_statusbar.png')
  graph.posX = 0
  graph.posY = 272
  anim = animation()
  anim.name = graph.name
  anim.data = [graph.posX, 250, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  return graph


def backupMenu(globalVars, formVars):
  #Backup del menu corrente:
  if formVars.get("formMenu"):
    functions.waitSafeMenu(formVars.get("formMenu"))
    sel = formVars.get("formMenu").getGraphicElement("MENU_SELECTOR_RIGHT")
    if sel:
      sel.visible = False
    sel = formVars.get("formMenu").getGraphicElement("MENU_SELECTOR_LEFT")
    if sel:
      sel.visible = False
    if not formVars.get("lastMenues"):
      formVars.set([], "lastMenues")
    formVars.get("lastMenues").append(formVars.get("formMenu"))
    formVars.set(None, "formMenu")


def restoreMenu(globalVars, formVars):
  #Restore dell'ultimo menu:
  if len(formVars.get("lastMenues")) > 0:
    lastMenu = formVars.get("lastMenues").pop()
    if lastMenu:
      formVars.set(lastMenu, "formMenu")
      formVars.set(lastMenu.name, "status")
      sel = lastMenu.getGraphicElement("MENU_SELECTOR_RIGHT")
      if sel:
        sel.visible = True
      sel = lastMenu.getGraphicElement("MENU_SELECTOR_LEFT")
      if sel:
        sel.visible = True


def delGraphic(pFormGraphics, pGraphName):
  """Delete a graphic element"""
  graph = pFormGraphics.get(pGraphName)
  if graph and graph.name[:4] != "DEL_":
    graph.name = "DEL_" + graph.name
    #graph.animation = None
    graph.image = None


def addTimeoutRestore(globalVars, formVars, pTimeout, pMessage, pShowFunction=None, pHideFunction=None):
  """Add a timeout object with confirm and restore form"""
  formGraphics = formVars.get("formGraphics")
  graph = graphicElement()
  graph.name = "WAITTIMEOUT"
  graph.data = [globalVars.get("currentFPSTime"), globalVars.get("currentFPSTime"), pMessage, pTimeout, True, None]
  anim = animation()
  def code(globalVars, formVars, target):
    #Aggiorno i secondi:
    if target.data[4]:
      updateWaitMessage(globalVars, formVars,  target.data[2] + " [" + str(target.data[3] - int(globalVars.get("currentFPSTime") -  target.data[1])) + " secs.]")

    if not target.data[0] \
    or globalVars.get("currentFPSTime") - target.data[0] >= pTimeout:
      if target.visible:
        target.visible = False
        updateWaitMessage(globalVars, formVars, pMessage)
        doUnlockKeyboard(globalVars, formVars)
        target.data[4] = False
        if pShowFunction:
          pShowFunction()
        #Chiedo conferma:
        tMenu = confirm(globalVars, formVars, formGraphics)
        tMenu.graphicPriority = 1000
        tMenu.animationPriority = 2000
        tMenu.background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirm.png')
        tMenu.name = "CONFIRMABORT"
        def code(globalVars, formVars, menu, element):
          #Chiudo il menu:
          globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
          globalVars.get("playingSound").start()
          delGraphic(formGraphics, "CONFIRM")
          delGraphic(formGraphics, "CONFIRM")
          menu.delGraphics()
          formVars.set(None, "formMenu")
          target.data[0] = globalVars.get("currentFPSTime")
          target.data[1] = globalVars.get("currentFPSTime")
          target.data[4] = True
          target.visible = True
          if pHideFunction:
            pHideFunction()
        tMenu.elements[1].trigger_select = code
        tMenu.trigger_cancel = code
        def code(globalVars, formVars, menu, element):
          #Torno alla form di gioco:
          globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
          globalVars.get("playingSound").start()
          globalVars.get("net").changeStatus("N")
          globalVars.set(["RESTORE"], "formParams")
          globalVars.set("mdlGame", "formName")
          globalVars.set("YES", "breakLoop")
        tMenu.elements[0].trigger_select = code
        formVars.set(tMenu, "formMenu")
        tMenu.createGraphicElements("RIGHT")

        graph = graphicElement()
        graph.name = "CONFIRM"
        graph.priority = 1000
        graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirmch_title.png')
        text = globalVars.get("messages").GM_CONFIRMABORT
        globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, (graph.image.width - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), text)) / 2, 2, text)
        graph.posX = tMenu.posX - (graph.image.width - tMenu.menuWidth) / 2
        graph.posY = tMenu.posY - graph.image.height - 2
        anim = animation()
        anim.name = graph.name
        anim.lockKeyboard = False
        anim.data = [graph.posX, graph.posY]
        anim.code = functions.moveFromTo
        graph.posX = -graph.image.width
        graph.animation = anim
        formGraphics.set(graph)
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def getStatusIcon(globalVars, pStatus):
  """Return icon for given status"""
  icon = None
  if pStatus == "N":
    icon = globalVars.get("iconNormal")
  elif pStatus == "P":
    icon = globalVars.get("iconPlaying")
  elif pStatus == "M":
    icon = globalVars.get("iconMessaging")
  elif pStatus == "W":
    icon = globalVars.get("iconWorldMap")
  elif pStatus == "D":
    icon = globalVars.get("iconDeck")
  elif pStatus == "S":
    icon = globalVars.get("iconShopping")
  elif pStatus == "O":
    icon = globalVars.get("iconOffline")
  return icon


def analyzePlayerInfo(response):
  """Analyze player info. Return TTplayer instance"""
  player = TTplayer()
  player.name = response[0]
  player.status = response[1]
  player.language = response[2]
  player.played = response[3]
  player.won = response[4]
  player.lost = response[5]
  player.draw = response[6]
  player.systemName = response[7]
  player.clientVersion = response[8]
  player.element = response[9]
  return player


def drawPlayerInfo(globalVars, player, imageName):
  """Draw player info and returns graphic element"""
  graph = graphicElement()
  graph.name = "PLAYER_INFO"
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/' + imageName)
  posX = 5
  posY = 2
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenuSel"), "I")
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_NAME)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 1 * tHeight, globalVars.get("messages").GM_STATUS)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 2 * tHeight, globalVars.get("messages").GM_LANG)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 3 * tHeight, globalVars.get("messages").GM_SYSTEM)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 4 * tHeight, globalVars.get("messages").GM_VERSION)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 5 * tHeight, globalVars.get("messages").GM_PLAY)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, 60 - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_WIN), posY + 6 * tHeight, globalVars.get("messages").GM_WIN)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, 60 * 2 - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_DRAW), posY + 6 * tHeight, globalVars.get("messages").GM_DRAW)
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, 60 * 3 - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_LOSE), posY + 6 * tHeight, globalVars.get("messages").GM_LOSE)

  elementImage = eval("globalVars.get(\"images\").get(\"element" + player.element.title() + "Image\")")
  graph.image.blit(elementImage, 0, 0, elementImage.width, elementImage.height, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_NAME + " "), posY + 0 * tHeight, True)
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_NAME + " ") + elementImage.width, posY + 0 * tHeight, player.name)
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_STATUS + " "), posY + 1 * tHeight, eval("globalVars.get(\"messages\").GM_STATUS_" + player.status[:1]))
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_LANG + " "), posY + 2 * tHeight, player.language)
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_SYSTEM + " "), posY + 3 * tHeight, player.systemName)
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_VERSION + " "), posY + 4 * tHeight, player.clientVersion)
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenu"), globalVars.get("messages").GM_PLAY + " "), posY + 5 * tHeight, player.played)
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, 60 - functions.textWidth(globalVars.get("fonts").get("fontMenu"), player.won), posY + 7 * tHeight, str(player.won))
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, 60 * 2 - functions.textWidth(globalVars.get("fonts").get("fontMenu"), player.draw), posY + 7 * tHeight, str(player.draw))
  globalVars.get("fonts").get("fontMenu").drawText(graph.image, 60 * 3 - functions.textWidth(globalVars.get("fonts").get("fontMenu"), player.lost), posY + 7 * tHeight, str(player.lost))
  return graph
