#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d
from core import functions, animation, graphicElement, dataStore, menu, menuElement, buttons, city
from formsFunction import *
from math import sqrt, pow

formGraphics = dataStore()
formVars = dataStore()
formVars.set(formGraphics, "formGraphics")
globalVars = None

def init(pGlobalVars, pParams):
  global formVars, globalVars, formGraphics

  formGraphics = dataStore()
  formVars = dataStore()
  formVars.set(formGraphics, "formGraphics")
  globalVars = pGlobalVars

  formVars.set([], "cityList")
  formVars.set(None, "selectedCity")
  formVars.set(psp2d.Image('themes/' + globalVars.get("userData").THEME + '/gm_city.png'), "cityImage")
  formVars.set(psp2d.Image('themes/' + globalVars.get("userData").THEME + '/gm_citycurrent.png'), "currentCityImage")

  formVars.set(psp2d.Image('themes/' + globalVars.get("userData").THEME + '/gm_bg_worldmap.png'), "bkgImage")
  formVars.set(globalVars.get("userData").THEME + '/gm_bg_worldmap.png', "bkgImageName")
  formVars.set(0, "bkgImagePosX")
  formVars.set(0, "bkgImagePosY")

  formVars.set("SELECT", "status")

  #Mappa:
  graph = graphicElement()
  graph.name = "WORLDMAP"
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/gm_worldmap.png')
  graph.priority = 0
  graph.posX = 480
  graph.posY = 4
  graph.data = [480 - graph.image.width - 4, graph.posY, 99]
  anim = animation()
  anim.name = graph.name
  anim.priority = 10
  anim.data = graph.data[:]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Menu location:
  locationMenu = menu(formGraphics, globalVars, formVars)
  locationMenu.name = "LOCATION_MENU"
  locationMenu.menuHeight = 200
  locationMenu.menuWidth = 150
  locationMenu.imageBuffer = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_wm_location.png')
  locationMenu.posX = 480 - graph.image.width - 4 + graph.image.width - locationMenu.menuWidth
  locationMenu.posY = graph.posY
  locationMenu.menuAlignment = "RIGHT"
  locationMenu.selector = "LEFT"
  formVars.set(locationMenu, "locationMenu")

  #Riquadro info:
  graph = graphicElement()
  graph.name = "CITYINFO"
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_wm_cityinfo.png')
  graph.imageBuffer = psp2d.Image(graph.image)
  graph.posX = -graph.image.width
  graph.posY = 4
  anim = animation()
  anim.name = graph.name
  anim.priority = 10
  anim.data = [4, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Riquadro Regole:
  graph = graphicElement()
  graph.name = "CITYRULES"
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_wm_cityrules.png')
  graph.imageBuffer = psp2d.Image(graph.image)
  graph.posX = -graph.image.width
  graph.posY = 118
  anim = animation()
  anim.name = graph.name
  anim.exclusive = True
  anim.data = [4, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  formVars.set(globalVars.get("cityList"), "cityList")
  drawCities()

  #Visualizzo il selector
  graph = graphicElement()
  graph.name = "CHECK_SELECTOR"
  anim = animation()
  def code(globalVars, formVars, target):
    graph = formGraphics.get("SELECTOR")
    graph.visible = True
    target.name = "DEL_" + target.name
  anim.name = graph.name
  anim.code = code
  anim.priority = -200
  graph.animation = anim
  formGraphics.set(graph)


def drawCities():
  """Loads city list"""
  worldMap = formGraphics.get("WORLDMAP")
  for tCity in formVars.get("cityList"):
    #Disegno la citta':
    if globalVars.get("player").city == tCity.cityID:
      worldMap.image.blit(psp2d.Image(formVars.get("currentCityImage")), 0, 0, psp2d.Image(formVars.get("currentCityImage")).width, psp2d.Image(formVars.get("currentCityImage")).height, tCity.posX, tCity.posY, True)
      selectorImage = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/selector_left.png')
      formVars.set(tCity, "selectedCity")

      graph = graphicElement()
      graph.name = "SELECTOR"
      graph.image = selectorImage
      graph.visible = False
      graph.data = graph.image
      graph.posX = worldMap.data[0] + tCity.posX - graph.image.width
      graph.posY = worldMap.data[1] + tCity.posY
      anim = animation()
      anim.name = graph.name
      anim.front = True
      anim.lockKeyboard = False
      anim.data = [-0.5,0]
      anim.code = functions.selectorAnim
      graph.animation = anim
      formGraphics.set(graph)
    else:
      worldMap.image.blit(psp2d.Image(formVars.get("cityImage")), 0, 0, psp2d.Image(formVars.get("cityImage")).width, psp2d.Image(formVars.get("cityImage")).height, tCity.posX, tCity.posY, True)

  updateCityInfo()


def leftCity():
  """Select city left the selected"""
  #Calcolo le distanze:
  minDist = 99999
  nearCity = None
  for tCity in formVars.get("cityList"):
    if tCity.posX < formVars.get("selectedCity").posX:
      dist = abs(sqrt(pow(formVars.get("selectedCity").posX - tCity.posX, 2) + pow(formVars.get("selectedCity").posY - tCity.posY, 2)))
      if dist < minDist:
        nearCity = tCity
        minDist = dist
  if nearCity:
    selectCity(nearCity)


def rightCity():
  """Select city right the selected"""
  #Calcolo le distanze:
  minDist = 99999
  nearCity = None
  for tCity in formVars.get("cityList"):
    if tCity.posX > formVars.get("selectedCity").posX:
      dist = abs(sqrt(pow(formVars.get("selectedCity").posX - tCity.posX, 2) + pow(formVars.get("selectedCity").posY - tCity.posY, 2)))
      if dist < minDist:
        nearCity = tCity
        minDist = dist
  if nearCity:
    selectCity(nearCity)


def upCity():
  """Select city up the selected"""
  #Calcolo le distanze:
  minDist = 99999
  nearCity = None
  for tCity in formVars.get("cityList"):
    if tCity.posY < formVars.get("selectedCity").posY:
      dist = abs(sqrt(pow(formVars.get("selectedCity").posX - tCity.posX, 2) + pow(formVars.get("selectedCity").posY - tCity.posY, 2)))
      if dist < minDist:
        nearCity = tCity
        minDist = dist
  if nearCity:
    selectCity(nearCity)


def downCity():
  """Select city down the selected"""
  #Calcolo le distanze:
  minDist = 99999
  nearCity = None
  for tCity in formVars.get("cityList"):
    if tCity.posY > formVars.get("selectedCity").posY:
      dist = abs(sqrt(pow(formVars.get("selectedCity").posX - tCity.posX, 2) + pow(formVars.get("selectedCity").posY - tCity.posY, 2)))
      if dist < minDist:
        nearCity = tCity
        minDist = dist
  if nearCity:
    selectCity(nearCity)


def selectCity(pCity):
  """Select one city"""
  formVars.set(pCity, "selectedCity")
  worldMap = formGraphics.get("WORLDMAP")
  city = formVars.get("selectedCity")
  graph = formGraphics.get("SELECTOR")
  anim = animation()
  anim.name = graph.name
  anim.front = True
  anim.data = [worldMap.data[0] + city.posX - graph.image.width, worldMap.data[1] + city.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim

  #Riattivo animazione selector:
  graph = graphicElement()
  graph.name = "WAIT_SELECTOR"
  anim = animation()
  anim.name = graph.name
  anim.front = True
  anim.lockKeyboard = False
  def code(globalVars, formVars, target):
    graph = formGraphics.get("SELECTOR")
    if not graph.animation:
      anim = animation()
      anim.name = graph.name
      anim.front = True
      anim.lockKeyboard = False
      anim.data = [-0.5,0]
      anim.code = functions.selectorAnim
      graph.animation = anim
      target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)
  updateCityInfo()


def updateCityInfo():
  """Update city information"""
  info = formGraphics.get("CITYINFO")
  info.image = psp2d.Image(info.imageBuffer)

  posX = 5
  posY = 5
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenuSel"), "I")
  globalVars.get("fonts").get("fontMenuSel").drawText(info.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_NAME)
  globalVars.get("fonts").get("fontMenu").drawText(info.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_NAME + " "), posY + 0 * tHeight, formVars.get("selectedCity").name)
  globalVars.get("fonts").get("fontMenuSel").drawText(info.image, posX, posY + 1 * tHeight, globalVars.get("messages").GM_REGION)
  globalVars.get("fonts").get("fontMenu").drawText(info.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_REGION + " "), posY + 1 * tHeight, formVars.get("selectedCity").region)

  for i, riga in enumerate(formVars.get("selectedCity").description.split("//n//")):
    globalVars.get("fonts").get("fontMenu").drawText(info.image, posX, posY + (2 + i) * tHeight, riga)

  #Regole:
  posX = 5
  posY = 5
  info = formGraphics.get("CITYRULES")
  info.image = psp2d.Image(info.imageBuffer)
  globalVars.get("fonts").get("fontMenuSel").drawText(info.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_RULES)
  for i, riga in enumerate(formVars.get("selectedCity").rules.split("//n//")):
    globalVars.get("fonts").get("fontMenu").drawText(info.image, posX, posY + (1 + i) * tHeight, eval("globalVars.get(\"messages\").GM_" + riga))


def enterLocationCallback(pResponse, pError):
  """Change city"""
  if pError:
    waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if response[0] == "OK":
    globalVars.get("net").changeStatus("N")
    globalVars.set(int(response[1]), "newCity")
    globalVars.set(int(response[2]), "newLocation")
    formVars.remove("cityList", pClear=False)
    formVars.remove("selectedCity", pClear=False)
    globalVars.set(["RESTORE"], "formParams")
    globalVars.set("mdlGame", "formName")
    globalVars.set("YES", "breakLoop")


def locationListCallback(pResponse, pError):
  """Loads location list"""
  if pError:
    waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  hideWaitMessage(globalVars, formVars)
  response = functions.analyzeServerString(pResponse)

  graph = formGraphics.get("WORLDMAP")
  locationMenu = formVars.get("locationMenu")
  locationMenu.clear()
  locationMenu.name = "LOCATION_MENU"
  locationMenu.graphicPriority = 10
  locationMenu.background_image = psp2d.Image(locationMenu.imageBuffer)
  locationMenu.menuHeight = graph.image.height
  locationMenu.menuWidth = 150
  locationMenu.posX = 480 - graph.image.width - 4 + graph.image.width - locationMenu.menuWidth
  locationMenu.posY = graph.posY
  locationMenu.menuAlignment = "RIGHT"
  locationMenu.selector = "LEFT"
  locationMenu.elements = []
  for l in response:
    cL = l.split("||")
    loc = city()
    loc.cityID = int(cL[0])
    loc.name = cL[1]
    loc.description = cL[2]
    loc.rules = cL[3]
    loc.players = int(cL[4])
    tElement = menuElement()
    tElement.text = loc.name
    tElement.value = loc.name
    tElement.data = loc
    def code(globalVars, formVars, menu, element):
      updateLocationInfo()
    tElement.trigger_enter = code

    def code(globalVars, formVars, menu, element):
      #Chiedo conferma:
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      askConfirm()
    tElement.trigger_select = code
    if globalVars.get("player").city == formVars.get("selectedCity").cityID \
    and globalVars.get("player").location == loc.cityID:
      tElement.enabled = False
    if tElement.enabled \
    and not formVars.get("locationMenu").selected:
      formVars.get("locationMenu").selected = tElement.value
    locationMenu.elements.append(tElement)

  def code(globalVars, formVars, menu, element):
    #Chiudo il menu:
    globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
    globalVars.get("playingSound").start()
    menu.delGraphics()
    formVars.set(None, "formMenu")
    formGraphics.get("SELECTOR").visible = True
    formVars.set("SELECT", "status")
    updateCityInfo()
  locationMenu.trigger_cancel = code

  formVars.set(locationMenu, "formMenu")
  locationMenu.createGraphicElements("RIGHT")
  if locationMenu.selected:
    updateLocationInfo()


def updateLocationInfo():
  """Update location information"""
  if formVars.get("formMenu").selected:
    formVars.set(formVars.get("formMenu").getSelected().data, "selectedLocation")
    info = formGraphics.get("CITYINFO")
    info.image = psp2d.Image(info.imageBuffer)

    posX = 5
    posY = 5
    tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenuSel"), "I")
    globalVars.get("fonts").get("fontMenuSel").drawText(info.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_NAME)
    globalVars.get("fonts").get("fontMenu").drawText(info.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_NAME + " "), posY + 0 * tHeight, formVars.get("selectedLocation").name)

    for i, riga in enumerate(formVars.get("selectedLocation").description.split("//n//")):
      globalVars.get("fonts").get("fontMenu").drawText(info.image, posX, posY + (1 + i) * tHeight, riga)

    globalVars.get("fonts").get("fontMenuSel").drawText(info.image, posX, posY + (2 + i) * tHeight, globalVars.get("messages").GM_PLAYERS)
    globalVars.get("fonts").get("fontMenu").drawText(info.image, posX + functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_PLAYERS + " "), posY + (2 + i) * tHeight, str(formVars.get("selectedLocation").players))

    #Regole:
    posX = 5
    posY = 5
    info = formGraphics.get("CITYRULES")
    info.image = psp2d.Image(info.imageBuffer)
    globalVars.get("fonts").get("fontMenuSel").drawText(info.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_RULES)

    #Della locazione:
    lastRow = 1
    for i, riga in enumerate(formVars.get("selectedLocation").rules.split("//n//")):
      if riga.find("_") >= 0 and riga[:riga.find("_")] in globalVars.get("elements"):
        globalVars.get("fonts").get("fontMenu").drawText(info.image, posX, posY + (lastRow + i) * tHeight, riga[riga.find("_") + 1:] + " x " + globalVars.get("cardSets")[0].elementals[globalVars.get("elements").index(riga[:riga.find("_")])])
      else:
        try:
          globalVars.get("fonts").get("fontMenu").drawText(info.image, posX, posY + (lastRow + i) * tHeight, eval("globalVars.get(\"messages\").GM_" + riga))
        except:
          globalVars.get("fonts").get("fontMenu").drawText(info.image, posX, posY + (lastRow + i) * tHeight, riga)


def askConfirm():
  #Ask confirm to move to new location
  formGraphics.get("SELECTOR").visible = False
  #Chiedo conferma della scelta:
  formVars.set("CONFIRM", "status")
  formVars.get("formMenu").delGraphics()
  tMenu = confirm(globalVars, formVars, formGraphics)
  tMenu.graphicPriority = 11
  tMenu.background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirm.png')
  def code(globalVars, formVars, menu, element):
    #Cambio citta':
    globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
    globalVars.get("playingSound").start()
    delGraphic(formGraphics, "CONFIRM")
    menu.delGraphics()
    formVars.set(None, "formMenu")
    formVars.set("MOVING", "status")
    globalVars.set(formVars.get("selectedCity").cityID, "newCity")
    globalVars.set(formVars.get("selectedLocation").cityID, "newLocation")
    formVars.remove("cityList", pClear=False)
    formVars.remove("selectedCity", pClear=False)
    globalVars.set(["RESTORE"], "formParams")
    globalVars.set("mdlGame", "formName")
    globalVars.set("YES", "breakLoop")

  tMenu.elements[0].trigger_select = code

  def code(globalVars, formVars, menu, element):
    #Chiudo il menu:
    globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
    globalVars.get("playingSound").start()
    delGraphic(formGraphics, "CONFIRM")
    menu.delGraphics()
    formVars.set(None, "formMenu")
    formVars.set("SELECT", "status")
    menu.formGraphics.get("SELECTOR").visible = True
  tMenu.elements[1].trigger_select = code
  tMenu.trigger_cancel = code
  tMenu.createGraphicElements("BOTTOM")
  formVars.set(tMenu, "formMenu")

  graph = graphicElement()
  graph.name = "CONFIRM"
  graph.priority = 1
  #Controllo la larghezza:
  width = functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_CONFIRMCITY + " " + formVars.get("selectedCity").name + "?") + 20
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirmch_title.png')
  text = globalVars.get("messages").GM_CONFIRMCITY + " " + formVars.get("selectedCity").name + "?"
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, (graph.image.width - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), text)) / 2, 2, text)
  graph.posX = tMenu.posX - (graph.image.width - tMenu.menuWidth) / 2
  graph.posY = tMenu.posY - graph.image.height - 2
  formGraphics.set(graph)


def button_down(pbuttons):
  #Gestione pressione tasti:
  if formVars.get("status") == "SELECT":
    if buttons.circle in pbuttons:
      #Torno al gioco:
      globalVars.get("net").changeStatus("N")
      formVars.remove("cityList", pClear=False)
      formVars.remove("selectedCity", pClear=False)
      globalVars.set(["RESTORE"], "formParams")
      globalVars.set("mdlGame", "formName")
      globalVars.set("YES", "breakLoop")
      return True
    elif buttons.cross in pbuttons and formVars.get("selectedCity"):
        #Visualizzo la lista delle locazioni:
        formVars.set("LOCATIONLIST", "status")
        formGraphics.get("SELECTOR").visible = False
        waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAIT)
        globalVars.get("net").locationList(formVars.get("selectedCity").cityID, locationListCallback)
        return True
    elif buttons.left in pbuttons:
      #Cambio citta':
      globalVars.set(globalVars.get("sounds").get("menuChangeSound"), "playingSound")
      globalVars.get("playingSound").start()
      leftCity()
      return True
    elif buttons.right in pbuttons:
      #Cambio citta':
      globalVars.set(globalVars.get("sounds").get("menuChangeSound"), "playingSound")
      globalVars.get("playingSound").start()
      rightCity()
      return True
    elif buttons.up in pbuttons:
      #Cambio citta':
      globalVars.set(globalVars.get("sounds").get("menuChangeSound"), "playingSound")
      globalVars.get("playingSound").start()
      upCity()
      return True
    elif buttons.down in pbuttons:
      #Cambio citta':
      globalVars.set(globalVars.get("sounds").get("menuChangeSound"), "playingSound")
      globalVars.get("playingSound").start()
      downCity()
      return True
