#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""Form match options"""
import psp2d, pspsnd
from core import graphicElement, dataStore, animation, functions, menu, menuElement, buttons, gameType
from formsFunction import *

formGraphics = dataStore()
formVars = dataStore()
formVars.set(formGraphics, "formGraphics")
globalVars = None

def init(pGlobalVars, pParams):
  global formVars, globalVars, formGraphics

  formGraphics = dataStore()
  formVars = dataStore()
  formVars.set(formGraphics, "formGraphics")
  globalVars = pGlobalVars

  formVars.set(globalVars.get("images").get("defaultBkgImage"), "bkgImage")
  formVars.set(0, "bkgImagePosX")
  formVars.set(0, "bkgImagePosY")

  startOptionsMenu = menu(formGraphics, pGlobalVars, formVars)
  startOptionsMenu.name = "STARTOPTIONS"

  lang = pGlobalVars.get("userData").LANG
  theme = pGlobalVars.get("userData").THEME

  #Titolo:
  graph = graphicElement()
  graph.name = "TITLE"
  graph.image = psp2d.Image('themes/' + pGlobalVars.get("userData").THEME + '/formsbg/gm_generic_title.png')
  graph.posX = -440
  graph.posY = 8
  anim = animation()
  anim.name = "TITLE"
  anim.data = [20, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  graph = graphicElement()
  graph.name = "TITLET"
  graph.font = pGlobalVars.get("fonts").get("fontTitle")
  graph.text = pGlobalVars.get("messages").STARTOPTIONS
  graph.posY = 10
  graph.posX = -functions.textWidth(graph.font, graph.text)
  anim = animation()
  anim.name = "TITLET"
  anim.data = [(480 - functions.textWidth(graph.font, graph.text)) / 2, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  tElement = menuElement()
  tElement.text = pGlobalVars.get("messages").START_MATCH
  tElement.helpText = pGlobalVars.get("messages").START_MATCH
  tElement.value = "STARTMATCH"
  def code(globalVars, formVars, menu, element):
    #Inizio partita singola:
    globalVars.set(pspsnd.Sound("themes/" + pGlobalVars.get("userData").THEME + "/start.wav"), "playingSound")
    globalVars.get("playingSound").start()
    #Controllo le opzioni scelte:
    gType = gameType()
    gType.type = "SINGLE"
    gType.levels = menu.elements[2].value.split("|")[1]
    pGlobalVars.get("userData").QUICK_LEVEL=gType.levels

    if menu.elements[3].value.split("|")[1] == "NO":
      gType.same = False
      globalVars.get("userData").QUICK_SAME="NO"
    else:
      globalVars.get("userData").QUICK_SAME="YES"

    if menu.elements[4].value.split("|")[1] == "NO":
      gType.sameWall = False
      globalVars.get("userData").QUICK_SAMEWALL="NO"
    else:
      globalVars.get("userData").QUICK_SAMEWALL="YES"

    if menu.elements[5].value.split("|")[1] == "NO":
      gType.plus = False
      globalVars.get("userData").QUICK_PLUS="NO"
    else:
      globalVars.get("userData").QUICK_PLUS="YES"

    if menu.elements[6].value.split("|")[1] == "NO":
      gType.combo = False
      globalVars.get("userData").QUICK_COMBO="NO"
    else:
      globalVars.get("userData").QUICK_COMBO="YES"

    if menu.elements[7].value.split("|")[1] == "NO":
      gType.elemental = False
      globalVars.get("userData").QUICK_ELEMENTAL="NO"
    else:
      globalVars.get("userData").QUICK_ELEMENTAL="YES"

    if menu.elements[8].value.split("|")[1] == "NO":
      gType.open = False
      globalVars.get("userData").QUICK_OPEN="NO"
    else:
      globalVars.get("userData").QUICK_OPEN="YES"

    if menu.elements[9].value.split("|")[1] == "NO":
      gType.suddenDeath = False
      globalVars.get("userData").QUICK_SUDDENDEATH="NO"
    else:
      globalVars.get("userData").QUICK_SUDDENDEATH="YES"

    #Cardset:
    for i, cs in enumerate(globalVars.get("cardSets")):
      if cs.name.lower() == menu.elements[1].value.split("|")[1]:
        gType.cardSet = globalVars.get("cardSets")[i]
        globalVars.get("userData").QUICK_CARDSET = cs.name.lower()

    #Salvo le opzioni:
    functions.saveOptions(globalVars.get("userData"))
    globalVars.set("mdlTripleTriad", "formName")
    globalVars.set([gType], "formParams")
    globalVars.set("YES", "breakLoop")
  tElement.trigger_select = code
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").CARDSET
  tElement.helpText = globalVars.get("messages").CARDSETH
  tElement.value = "CARDSET|" + globalVars.get("userData").QUICK_CARDSET
  for i, tCardSet in enumerate(globalVars.get("cardSets")):
    tSubElement = menuElement()
    tSubElement.text = tCardSet.name
    tSubElement.value = tCardSet.name.lower()
    tSubElement.helpText = tCardSet.shortDesc
    tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").CARDS_LEVEL
  tElement.helpText = globalVars.get("messages").CARDS_LEVELH
  tElement.value = "CARDSLEVEL|" + globalVars.get("userData").QUICK_LEVEL
  tSubElement = menuElement()
  tSubElement.text = "1-3"
  tSubElement.value = "1-3"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = "4-6"
  tSubElement.value = "4-6"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = "7-10"
  tSubElement.value = "7-10"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").SAME_RULE
  tElement.helpText = globalVars.get("messages").SAME_RULEH
  tElement.value = "SAME|" + globalVars.get("userData").QUICK_SAME
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").SAMEWALL_RULE
  tElement.helpText = globalVars.get("messages").SAMEWALL_RULEH
  tElement.value = "SAMEWALL|" + globalVars.get("userData").QUICK_SAMEWALL
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").PLUS_RULE
  tElement.helpText = globalVars.get("messages").PLUS_RULEH
  tElement.value = "PLUS|" + globalVars.get("userData").QUICK_PLUS
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").COMBO_RULE
  tElement.helpText = globalVars.get("messages").COMBO_RULEH
  tElement.value = "COMBO|" + globalVars.get("userData").QUICK_COMBO
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").ELEMENTAL_RULE
  tElement.helpText = globalVars.get("messages").ELEMENTAL_RULEH
  tElement.value = "ELEMENTAL|" + globalVars.get("userData").QUICK_ELEMENTAL
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").OPEN_RULE
  tElement.helpText = globalVars.get("messages").OPEN_RULEH
  tElement.value = "OPEN|" + globalVars.get("userData").QUICK_OPEN
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").SUDDENDEATH_RULE
  tElement.helpText = globalVars.get("messages").SUDDENDEATH_RULEH
  tElement.value = "SUDDENDEATH|" + globalVars.get("userData").QUICK_SUDDENDEATH
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  startOptionsMenu.elements.append(tElement)

  def code(globalVars, formVars, menu, element):
    #Torno al menu start:
    globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
    globalVars.get("playingSound").start()
    globalVars.set("mdlStart", "formName")
    globalVars.set(["STARTQUICK"], "formParams")
    globalVars.set("YES", "breakLoop")
  startOptionsMenu.trigger_cancel = code

  startOptionsMenu.background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_generic_menu.png')
  startOptionsMenu.posX = globalVars.get("themeOptions").MENU_X
  startOptionsMenu.posY = globalVars.get("themeOptions").MENU_Y
  startOptionsMenu.menuWidth = globalVars.get("themeOptions").MENU_W
  startOptionsMenu.menuHeight = globalVars.get("themeOptions").MENU_H
  startOptionsMenu.selected = startOptionsMenu.elements[0].value
  startOptionsMenu.menuAlignment = globalVars.get("themeOptions").MENU_ALIGN
  startOptionsMenu.menuVAlignment = globalVars.get("themeOptions").MENU_VALIGN

  formMenu = startOptionsMenu
  formMenu.createGraphicElements("BOTTOM")
  formVars.set(formMenu, "formMenu")
  formVars.set(None, "formToolbar")


def button_down(pbuttons):
  return False
