#    Triple Triad PSP
#    Copyright (C) 2006, 2007 Sakya
#    sakya_tg@yahoo.it
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import psp2d, pspsnd
from core import graphicElement, dataStore, animation, functions, menu, menuElement, buttons, deck, cardSet, helpItem, gameType, vKeyboard
from formsFunction import *
from random import _random

formGraphics = dataStore()
formVars = dataStore()
formVars.set(formGraphics, "formGraphics")
globalVars = None

"""Form Triple Triad"""
class move(object):
  def __init__(self):
    self.value = 0
    self.card = None
    self.cardDelta = 0
    self.cell = None
    self.getCards = []
    self.getCardsNormal = []
    self.getCardsSame = []
    self.getCardsSameWall = []
    self.getCardsPlus = []
    self.getCardsCombo = []
    self.same = False
    self.sameWall = False
    self.plus = False
    self.combo = False
    self.deckCell = None
    self.exposed = [None,None,None]


  def stringValue(self):
    return "Move: " + self.card.name.ljust(20) + " Pos: " + str(self.cell).rjust(1) + " Value: " + str(self.value).rjust(3)  + " Exposed: " + str(self.exposed).rjust(1)

def init(pGlobalVars, pParams):
  global formVars, globalVars, formGraphics

  formGraphics = dataStore()
  formVars = dataStore()
  formVars.set(formGraphics, "formGraphics")
  globalVars = pGlobalVars
  pGameType = pParams[0]

  formVars.set(psp2d.Image('themes/' + pGlobalVars.get("userData").THEME + '/backgrounddeck.png'), "bkgImage")
  formVars.set(0, "bkgImagePosX")
  formVars.set(0, "bkgImagePosY")

  #Help
  formHelp = helpItem(formGraphics, globalVars, formVars)
  formHelp.images = [globalVars.get("images").get("buttonCross"), globalVars.get("images").get("buttonCircle"), globalVars.get("images").get("buttonStart"), globalVars.get("images").get("buttonSquare")]
  formHelp.rows = [globalVars.get("messages").MH_CROSS, globalVars.get("messages").MH_CIRLCE, globalVars.get("messages").MH_START, globalVars.get("messages").MH_SQUARE]
  formHelp.preShowFunc = showHelp
  formHelp.postHideFunc = hideHelp
  formVars.set(formHelp, "helpItem")


  formVars.set(deck(), "deck")
  formVars.set(pGameType, "gameType")
  formVars.set(None, "result")
  formVars.set("", "firstTurn")
  formVars.set(pspsnd.Sound("themes/" + globalVars.get("userData").THEME + "/card.wav"), "cardSound")

  scoreImages = [psp2d.Image('themes/' + globalVars.get("userData").THEME + '/0.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/1.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/2.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/3.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/4.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/5.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/6.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/7.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/8.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/9.png'), \
                 psp2d.Image('themes/' + globalVars.get("userData").THEME + '/10.png')]
  formVars.set(scoreImages, "scoreImages")

  formVars.set(None, "tradingCards")
  formVars.set(None, "toBe3d")
  formVars.set([], "choosenCards")

  #Carico il font
  globalVars.get("fonts").set(psp2d.Font('themes/' + globalVars.get("userData").THEME + '/font_result.png'), "fontResult")

  gameMenu = menu(formGraphics, pGlobalVars, formVars)
  gameMenu.name = "INGAMEMENU"
  gameMenu.graphicPriority = 200
  gameMenu.animationPriority= 2000
  formVars.set(gameMenu, "gameMenu")

  chatMessageMenu = menu(formGraphics, pGlobalVars, formVars)
  chatMessageMenu.name = "CHATMESSAGEMENU"
  chatMessageMenu.background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_shout_menu.png')
  chatMessageMenu.graphicPriority = 200
  chatMessageMenu.animationPriority = 2000
  chatMessageMenu.menuWidth = 290
  chatMessageMenu.menuHeight = 145
  chatMessageMenu.posX = 95
  chatMessageMenu.posY = 25
  chatMessageMenu.menuAlignment = "LEFT"
  chatMessageMenu.selector = "RIGHT"
  formVars.set(chatMessageMenu, "chatMessageMenu")

  #Controllo il tipo di gioco:
  if formVars.get("gameType").type == "SINGLE":
    initSingle()
  elif formVars.get("gameType").type == "MULTI":
    initMulti()


def initSingle():
  #Inizializzazioni per partita single-player:
  #Chi inizia?
  random = _random.Random()
  random.seed()
  rnd = int(round(random.random() * 1))
  if rnd == 0:
    formVars.set("P1", "firstTurn")
  else:
    formVars.set("P2", "firstTurn")

  tGameType = formVars.get("gameType")
  allCards = tGameType.cardSet

  #Creo gli elementi:
  if tGameType.elemental:
    createElements()

  #Calcolo il massimo per i numeri casuali:
  cardsUsed = []
  if tGameType.levels == "1-3":
    cardsUsed.extend(allCards.levels[1])
    cardsUsed.extend(allCards.levels[2])
    cardsUsed.extend(allCards.levels[3])
  elif tGameType.levels == "4-6":
    cardsUsed.extend(allCards.levels[4])
    cardsUsed.extend(allCards.levels[5])
    cardsUsed.extend(allCards.levels[6])
  elif tGameType.levels == "7-10":
    cardsUsed.extend(allCards.levels[7])
    cardsUsed.extend(allCards.levels[8])
    cardsUsed.extend(allCards.levels[9])
    cardsUsed.extend(allCards.levels[10])

  #Mazzi dei due giocatori:
  formVars.set(cardSet(), "cardsP1")
  tGameType.initialDeck_Player1 = []
  tGameType.initialDeck_Player2 = []
  #Cinque carte a caso:
  values = []
  for i in xrange(0,5):
    while True:
      rnd = int(round(random.random() * (len(cardsUsed) - 1)))
      if rnd in values:
        continue
      values.append(rnd)
      tGameType.initialDeck_Player1.append(i)
      tCard = cardsUsed[rnd].copy()
      tCard.ID = i
      tCard.owner = "P1"
      tCard.data = str(i)
      tCard.deckCell = None
      tCard.buffImageP1 = psp2d.Image(tCard.imageP1)
      tCard.buffImageP2 = psp2d.Image(tCard.imageP2)
      formVars.get("cardsP1").elements.append(tCard)
      #Creo l'oggetto grafico:
      graph = graphicElement()
      graph.name = str(i)
      graph.image = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      graph.posX = 10
      graph.posY = -100
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      anim.data = [graph.posX, 3 + 43 * i]
      anim.priority = 99 - i
      anim.code = functions.moveFromTo
      graph.animation = anim
      formGraphics.set(graph)
      break

  formVars.set(cardSet(), "cardsP2")
  #Cinque carte a caso:
  values = []
  for i in xrange(0,5):
    while True:
      rnd = int(round(random.random() * (len(cardsUsed) - 1)))
      if rnd in values:
        continue
      values.append(rnd)
      tGameType.initialDeck_Player2.append(100 + i)
      tCard = cardsUsed[rnd].copy()
      tCard.ID = 100 + i
      tCard.owner = "P2"
      tCard.data = str(100 + i)
      tCard.deckCell = None
      tCard.buffImageP1 = psp2d.Image(tCard.imageP1)
      tCard.buffImageP2 = psp2d.Image(tCard.imageP2)
      formVars.get("cardsP2").elements.append(tCard)
      #Creo l'oggetto grafico:
      graph = graphicElement()
      graph.name = str(100 + i)
      if tGameType.open:
        graph.image = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
      else:
        graph.image = tCard.imageBack
      graph.posX = 396
      graph.posY = -100
      anim = animation()
      anim.name = "MOVECARD"
      anim.data = [graph.posX, 3 + 43 * i]
      if i == 4:
        anim.exclusive = True
      anim.priority = 89 - i
      anim.code = functions.moveFromTo
      graph.animation = anim
      formGraphics.set(graph)
      break
  #Messaggio CHALLENGE:
  formVars.set("SINGLE", "status")
  graph = graphicElement()
  graph.name = "CHALLENGE"
  graph.font = globalVars.get("fonts").get("fontResult")
  msg = globalVars.get("messages").CHALLENGE
  tWidth = functions.textWidth(graph.font, msg)
  tX = round((480 - tWidth) / 2)
  graph.posX = tX
  graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
  graph.text = msg
  anim = animation()
  anim.name = graph.name
  anim.priority = 50
  anim.exclusive = True
  anim.front = True
  anim.code = functions.message
  graph.animation = anim
  formGraphics.set(graph)

  #Selector:
  graph = graphicElement()
  graph.name = "CARD_SELECTOR"
  graph.visible = False
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/selector_right.png')
  graph.posX = 10 + 74 + 10
  graph.posY = 3 + 74 / 2 - graph.image.height / 2
  graph.visible = False
  graph.priority = 10
  anim = animation()
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.data = [+0.5,0]
  anim.code = functions.selectorAnim
  graph.animation = anim
  formGraphics.set(graph)

  postInit()


def initMulti():
  #Inizializzazioni per partita multi-player:
  waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAIT)
  formVars.get("gameType").lastTurn = -1
  formVars.set([], "chatMessages")
  formVars.set([], "oldChatMessages")

  #Chiedo le info dei due giocatori:
  globalVars.get("net").playerInfo(formVars.get("gameType").player1_Name, player1InfoCallback)
  globalVars.get("net").playerInfo(formVars.get("gameType").player2_Name, player2InfoCallback)

  #Chiedo i mazzi e le regole:
  formVars.set(cardSet(), "cardsP1")
  formVars.set(cardSet(), "cardsP2")
  globalVars.get("net").getDecksAndRules(formVars.get("gameType").roomID, getDecksAndRulesCallback)

  #Selector:
  graph = graphicElement()
  graph.name = "CARD_SELECTOR"
  graph.visible = False
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/selector_right.png')
  graph.data = graph.image
  graph.posX = 10 + 74 + 10
  graph.posY = 3 + 74 / 2 - graph.image.height / 2
  graph.visible = False
  graph.priority = 10
  anim = animation()
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.data = [+0.5,0]
  anim.code = functions.selectorAnim
  graph.animation = anim
  formGraphics.set(graph)

  #Timer per chat message:
  graph = graphicElement()
  graph.name = "CHATMESSAGE"
  graph.data = globalVars.get("currentFPSTime")
  anim = animation()
  def code(globalVars, formVars, target):
    if not target.data \
    or globalVars.get("currentFPSTime") - target.data >= 5:
      if target.visible == True:
        globalVars.get("net").getChatMessages(formVars.get("gameType").roomID, getChatMessagesCallback)
        target.visible = False
      target.data = globalVars.get("currentFPSTime")
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def player1InfoCallback(pResponse, pError):
  """Player 1 info callback"""
  if pError:
    waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) >= 10:
    player = analyzePlayerInfo(response)
    formVars.set(player, "Player_1_info")


def player2InfoCallback(pResponse, pError):
  """Player 2 info callback"""
  if pError:
    waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) >= 10:
    player = analyzePlayerInfo(response)
    formVars.set(player, "Player_2_info")


def postInit():
  """Post-init operations"""
  graph = graphicElement()
  graph.name = "START_MATCH"
  graph.posX = 0
  graph.posY = 0
  anim = animation()
  anim.name = graph.name
  anim.priority = -20
  def code(globalVars, formVars, target):
    #Controllo chi deve iniziare:
    if formVars.get("firstTurn") == "P1":
     selectFirstCardP1()
    else:
      if formVars.get("gameType").type == "SINGLE":
        computerMove()
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Mi metto in stato selezione carta:
  if formVars.get("status") in ["MENU", "VIEWRULES"]:
    formVars.get("data")[0] = "SELECTING"
  else:
    formVars.set("SELECTING", "status")
  updateScore()


def createElements():
  """Creates elements on deck"""
  #Cancello quelli gia' presenti:
  tDeck = formVars.get("deck")
  tDeck.elements = [None] * 9

  #Creo gli elementi:
  random = _random.Random()
  random.seed()
  positions = []
  allCards = formVars.get("gameType").cardSet
  numEl = int(round(random.random() * 9))
  for i in xrange(0, numEl):
    el = int(round(random.random() * (len(allCards.elementals) - 1)))
    #Posizione:
    while True:
      pos = int(round(random.random() * 8))
      if pos not in positions:
        positions.append(pos)
        break
    tDeck.elements[pos] = el
    formVars.get("bkgImage").blit(formVars.get("gameType").cardSet.elementalsImages[el], 0, 0, -1, -1, functions.fromCellToX(pos) + 38, functions.fromCellToY(pos) + 38, True)


def getSelectedP1():
  """Returns the selected card for Player 1"""
  for tCard in formVars.get("cardsP1").elements:
    if tCard.status == "SELECTED":
      return tCard
  return None


def selectFirstCardP1():
  """Selects the first card not used for Player 1"""
  for i, tCard in enumerate(formVars.get("cardsP1").elements):
    if tCard.deckCell == None:
      tCard.status = "SELECTED"
      graph = formGraphics.get(tCard.data)
      graph.status = "SELECTED"
      anim = animation()
      anim.name = "MOVECARD"
      anim.exclusive = True
      anim.priority = 50
      anim.data = [graph.posX + 20, graph.posY]
      anim.code = functions.moveFromTo
      graph.animation = anim
      formVars.set(tCard.stringValue(), "message")

      graph = graphicElement()
      graph.name = "CARD_SOUND"
      anim = animation()
      anim.name = "CARD_SOUND"
      anim.priority = 10
      def code(globalVars, formVars, target):
        globalVars.set(formVars.get("cardSound"), "playingSound")
        globalVars.get("playingSound").start()
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)

      #Selector:
      graph = graphicElement()
      graph.name = "CHANGE_SELECTOR_ANIM"
      anim = animation()
      anim.name = graph.name
      anim.priority = -200
      def code(globalVars, formVars, target):
        graph = formGraphics.get("CARD_SELECTOR")
        if not formVars.get("status") in ["MENU", "VIEWRULES"]:
          graph.visible = True
        graph.posY = 3 + formVars.get("cardsP1").elements[0].imageBack.height / 2 - graph.image.height / 2 + 43 * i
        anim = animation()
        anim.name = graph.name
        anim.lockKeyboard = False
        anim.data = [+0.5,0]
        anim.code = functions.selectorAnim
        graph.animation = anim
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)
      break


def selectNextCardP1():
  """Selects the next card for Player 1"""
  next = False
  for i, tCard in enumerate(formVars.get("cardsP1").elements):
    if next and tCard.deckCell == None:
      globalVars.set(formVars.get("cardSound"), "playingSound")
      globalVars.get("playingSound").start()

      #Deseleziono anche l'elemento grafico:
      graph = formGraphics.get(last.data)
      last.status = None
      graph.status = None
      anim = animation()
      anim.name = "MOVECARD"
      anim.data = [graph.posX - 20, graph.posY]
      anim.code = functions.moveFromTo
      graph.animation = anim
      tCard.status = "SELECTED"

      #Seleziono anche l'elemento grafico:
      graph = formGraphics.get(tCard.data)
      graph.status = "SELECTED"
      anim = animation()
      anim.name = "MOVECARD"
      anim.data = [graph.posX + 20, graph.posY]
      anim.code = functions.moveFromTo
      graph.animation = anim
      globalVars.set(tCard.stringValue(), "message")

      #Animazione selector:
      graph = formGraphics.get("CARD_SELECTOR")
      anim = animation()
      anim.name = graph.name
      anim.data = [10 + formVars.get("cardsP1").elements[0].imageBack.width + 10, 3 + formVars.get("cardsP1").elements[0].imageBack.height / 2 - graph.image.height / 2 + 43 * i, 99]
      anim.exclusive = True
      anim.code = functions.moveFromTo
      graph.animation = anim

      graph = graphicElement()
      graph.name = "CHANGE_SELECTOR_ANIM"
      anim = animation()
      anim.name = graph.name
      anim.priority = -200
      def code(globalVars, formVars, target):
        graph = formGraphics.get("CARD_SELECTOR")
        anim = animation()
        anim.name = graph.name
        anim.lockKeyboard = False
        anim.data = [+0.5,0]
        anim.code = functions.selectorAnim
        graph.animation = anim
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)
      break


    if tCard.status == "SELECTED" and i < len(formVars.get("cardsP1").elements) - 1:
      last = tCard
      next = True


def selectPreviousCardP1():
  """Selects the previous card for Player 1"""
  cards = formVars.get("cardsP1").elements[:]
  cards.reverse()
  next = False
  for i, tCard in enumerate(cards):
    if next and tCard.deckCell == None:
      globalVars.set(formVars.get("cardSound"), "playingSound")
      globalVars.get("playingSound").start()

      #Deseleziono anche l'elemento grafico:
      graph = formGraphics.get(last.data)
      last.status = None
      graph.status = None
      anim = animation()
      anim.name = "MOVECARD"
      anim.data = [graph.posX - 20, graph.posY]
      anim.code = functions.moveFromTo
      graph.animation = anim

      tCard.status = "SELECTED"
      #Seleziono anche l'elemento grafico:
      graph = formGraphics.get(tCard.data)
      graph.status = "SELECTED"
      anim = animation()
      anim.name = "MOVECARD"
      anim.data = [graph.posX + 20, graph.posY]
      anim.code = functions.moveFromTo
      graph.animation = anim
      globalVars.set(tCard.stringValue(), "message")

      #Animazione selector:
      graph = formGraphics.get("CARD_SELECTOR")
      anim = animation()
      anim.name = graph.name
      anim.data = [10 + formVars.get("cardsP1").elements[0].imageBack.width + 10, 3 + formVars.get("cardsP1").elements[0].imageBack.height / 2 - graph.image.height / 2 + 43 * (4 - i), 99]
      anim.exclusive = True
      anim.code = functions.moveFromTo
      graph.animation = anim

      graph = graphicElement()
      graph.name = "CHANGE_SELECTOR_ANIM"
      anim = animation()
      anim.name = graph.name
      anim.priority = -200
      def code(globalVars, formVars, target):
        graph = formGraphics.get("CARD_SELECTOR")
        anim = animation()
        anim.name = graph.name
        anim.lockKeyboard = False
        anim.data = [+0.5,0]
        anim.code = functions.selectorAnim
        graph.animation = anim
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)
      break

    if tCard.status == "SELECTED" and i < len(cards) - 1:
      next = True
      last = tCard


def selectCardP1():
  """Selects the card for Player 1"""
  globalVars.set(formVars.get("cardSound"), "playingSound")
  globalVars.get("playingSound").start()

  #Cerco il primo slot libero:
  for i, tCard in enumerate(formVars.get("deck").cells):
    if tCard == None:
      break

  tCard = getSelectedP1()
  graph = formGraphics.get(tCard.data)
  formVars.set(graph.name, "selectedCard")
  tCard.deckCell = i
  formVars.get("deck").cells[i] = tCard
  toX = functions.fromCellToX(i)
  toY = functions.fromCellToY(i)
  anim = animation()
  anim.front = True
  anim.name = "MOVECARD"
  anim.data = [toX, toY]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formVars.set("POSITIONING", "status")

  #Nascondo il selector:
  graph = formGraphics.get("CARD_SELECTOR")
  graph.visible = False


def MoveCardP1Left():
  """Moves selectedcard to the left"""
  tCard = getSelectedP1()

  initPos = tCard.deckCell
  nextPos = formVars.get("deck").previousFreeCell(tCard.deckCell)
  if nextPos != None:
    graph = formGraphics.get(formVars.get("selectedCard"))
    toX = functions.fromCellToX(nextPos)
    toY = functions.fromCellToY(nextPos)

    anim = animation()
    anim.front = True
    anim.name = "MOVECARD"
    anim.data = [toX, toY]
    anim.code = functions.moveFromTo
    graph.animation = anim

    formVars.get("deck").cells[initPos] = None
    formVars.get("deck").cells[nextPos] = tCard
    tCard.deckCell = nextPos


def MoveCardP1Right():
  """Moves selectedcard to the right"""
  tCard = getSelectedP1()

  initPos = tCard.deckCell
  nextPos = formVars.get("deck").nextFreeCell(tCard.deckCell)
  if nextPos != None:
    graph = formGraphics.get(formVars.get("selectedCard"))
    toX = functions.fromCellToX(nextPos)
    toY = functions.fromCellToY(nextPos)
    anim = animation()
    anim.front = True
    anim.name = "MOVECARD"
    anim.data = [toX, toY]
    anim.code = functions.moveFromTo
    graph.animation = anim

    formVars.get("deck").cells[initPos] = None
    formVars.get("deck").cells[nextPos] = tCard
    tCard.deckCell = nextPos


def MoveCardP1Up():
  """Moves selectedcard up"""
  tCard = getSelectedP1()
  if tCard.deckCell in [0,1,2]:
    return

  graph = formGraphics.get(formVars.get("selectedCard"))
  cell = tCard.deckCell - 3
  while True:
    if formVars.get("deck").cells[cell] == None:
      break
    if cell < 3:
      cell = None
      break
    cell -= 3

  if cell != None:
    toY = functions.fromCellToY(cell)
    anim = animation()
    anim.front = True
    anim.name = "MOVECARD"
    anim.data = [graph.posX, toY]
    anim.code = functions.moveFromTo
    graph.animation = anim

    formVars.get("deck").cells[tCard.deckCell] = None
    formVars.get("deck").cells[cell] = tCard
    tCard.deckCell = cell


def MoveCardP1Down():
  """Moves selectedcard down"""
  tCard = getSelectedP1()
  if tCard.deckCell in [6,7,8]:
    return

  graph = formGraphics.get(formVars.get("selectedCard"))
  cell = tCard.deckCell + 3
  while True:
    if formVars.get("deck").cells[cell] == None:
      break
    if cell > 5:
      cell = None
      break
    cell += 3

  if cell != None:
    toY = functions.fromCellToY(cell)
    anim = animation()
    anim.front = True
    anim.name = "MOVECARD"
    anim.data = [graph.posX, toY]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formVars.get("deck").cells[tCard.deckCell] = None
    formVars.get("deck").cells[cell] = tCard
    tCard.deckCell = cell


def releaseCardP1():
  """Release on the deck the selected card for Player 1"""
  tCard = getSelectedP1()
  graph = formGraphics.get(formVars.get("selectedCard"))
  tCard.status = None
  formVars.set("WAITING", "status")

  #Valuto la mossa:
  tMove = evaluateMove(tCard, tCard.deckCell)
  tCard.delta = tMove.cardDelta
  return tMove


def abortCardP1():
  """Abort the selected card for Player 1"""
  globalVars.set(formVars.get("cardSound"), "playingSound")
  globalVars.get("playingSound").start()

  tCard = getSelectedP1()
  graph = formGraphics.get(formVars.get("selectedCard"))
  graph.status = None
  formVars.get("deck").cells[tCard.deckCell] = None
  tCard.deckCell = None
  toX = 30
  toY = 3 + 43 * formVars.get("cardsP1").elements.index(tCard)
  anim = animation()
  anim.front = True
  anim.name = "MOVECARD"
  anim.data = [toX, toY]
  anim.exclusive = True
  anim.code = functions.moveFromTo
  graph.animation = anim
  formVars.set("SELECTING", "status")

  #Visualizzo il selector:
  graph = graphicElement()
  graph.name = "CHANGE_SELECTOR_ANIM"
  anim = animation()
  anim.name = graph.name
  anim.priority = -200
  def code(globalVars, formVars, target):
    graph = formGraphics.get("CARD_SELECTOR")
    graph.visible = True
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def evaluateNormal(pDeck, pCard, pCell, pDelta):
  """Evaluates normal rule"""
  tCell, rCell, bCell, lCell = pDeck.getNearCells(pCell)

  getCards = []
  if rCell != None:
    #Carta a destra:
    if rCell.owner <> pCard.owner and rCell.left + rCell.delta < pCard.right + pDelta:
      getCards.append((False, pCell + 1))

  if lCell != None:
    #Carta a sinistra:
    if lCell.owner <> pCard.owner and lCell.right + lCell.delta < pCard.left + pDelta:
      getCards.append((False, pCell - 1))

  if tCell != None:
    #Carta sopra:
    if tCell.owner <> pCard.owner and tCell.bottom + tCell.delta < pCard.top + pDelta:
      getCards.append((True, pCell - 3))

  if bCell != None:
    #Carta sotto:
    if bCell.owner <> pCard.owner and bCell.top + bCell.delta < pCard.bottom + pDelta:
      getCards.append((True, pCell + 3))
  return getCards


def evaluateSameWall(pDeck, pCard, pCell, pDelta):
  """Evaluates same-wall rule"""
  tCell, rCell, bCell, lCell = pDeck.getNearCells(pCell)

  getCards = []
  if pCell in [0,3,6] and pCard.left + pDelta == 10:
    if rCell != None and rCell.left + rCell.delta == pCard.right + pDelta and rCell.owner != pCard.owner:
      getCards.append((False, pCell + 1))

    if tCell != None and tCell.bottom + tCell.delta == pCard.top + pDelta and tCell.owner != pCard.owner:
      getCards.append((True, pCell - 3))

    if bCell != None and bCell.top + bCell.delta == pCard.bottom + pDelta and bCell.owner != pCard.owner:
      getCards.append((True, pCell + 3))
  elif pCell in [0,1,2] and pCard.top + pDelta == 10:
    if rCell != None and rCell.left + rCell.delta == pCard.right + pDelta and rCell.owner != pCard.owner:
      getCards.append((False, pCell + 1))

    if lCell != None and lCell.right + lCell.delta == pCard.left + pDelta and lCell.owner != pCard.owner:
      getCards.append((False, pCell - 1))

    if bCell != None and bCell.top + bCell.delta == pCard.bottom + pDelta and bCell.owner != pCard.owner:
      getCards.append((True, pCell + 3))
  elif pCell in [6,7,8] and pCard.bottom + pDelta == 10:
    if rCell != None and rCell.left + rCell.delta == pCard.right + pDelta and rCell.owner != pCard.owner:
      getCards.append((False, pCell + 1))

    if lCell != None and lCell.right + lCell.delta == pCard.left + pDelta and lCell.owner != pCard.owner:
      getCards.append((False, pCell - 1))

    if tCell != None and tCell.bottom + tCell.delta == pCard.top + pDelta and tCell.owner != pCard.owner:
      getCards.append((True, pCell - 3))
  elif pCell in [2,5,8] and pCard.right + pDelta == 10:
    if lCell != None and lCell.right + lCell.delta == pCard.left + pDelta and lCell.owner != pCard.owner:
      getCards.append((False, pCell - 1))

    if tCell != None and tCell.bottom + tCell.delta == pCard.top + pDelta and tCell.owner != pCard.owner:
      getCards.append((True, pCell - 3))

    if bCell != None and bCell.top + bCell.delta == pCard.bottom + pDelta and bCell.owner != pCard.owner:
      getCards.append((True, pCell + 3))

  if len(getCards) > 0:
    return getCards
  else:
    return []


def evaluateSame(pDeck, pCard, pCell, pDelta):
  """Evaluates same rule"""
  tCell, rCell, bCell, lCell = pDeck.getNearCells(pCell)

  getCards = []
  if rCell != None and rCell.left + rCell.delta == pCard.right + pDelta:
    getCards.append((False, pCell + 1))

  if lCell != None and lCell.right + lCell.delta == pCard.left + pDelta:
    getCards.append((False, pCell - 1))

  if tCell != None and tCell.bottom + tCell.delta == pCard.top + pDelta:
    getCards.append((True, pCell - 3))

  if bCell != None and bCell.top + bCell.delta == pCard.bottom + pDelta:
    getCards.append((True, pCell + 3))

  if len(getCards) >= 2:
    return [card for card in getCards if pDeck.cells[card[1]].owner != pCard.owner]
  else:
    return []


def evaluatePlus(pDeck, pCard, pCell, pDelta):
  """Evaluates plus rule"""
  tCell, rCell, bCell, lCell = pDeck.getNearCells(pCell)

  getCards = []
  if rCell != None and bCell != None \
  and rCell.owner != pCard.owner\
  and rCell.left + rCell.delta + pCard.right + pDelta == bCell.top + bCell.delta + pCard.bottom + pDelta:
    getCards.append((False, pCell + 1))
    getCards.append((True, pCell + 3))

  if rCell != None and lCell != None \
  and rCell.owner != pCard.owner\
  and lCell.right + lCell.delta + pCard.left + pDelta == rCell.left + rCell.delta + pCard.right + pDelta:
    getCards.append((False, pCell + 1))
    getCards.append((False, pCell - 1))

  if bCell != None and lCell != None \
  and bCell.owner != pCard.owner\
  and bCell.top + bCell.delta + pCard.bottom + pDelta == lCell.right + lCell.delta + pCard.left + pDelta:
    getCards.append((True, pCell + 3))
    getCards.append((False, pCell - 1))

  if tCell != None and rCell != None \
  and tCell.owner != pCard.owner\
  and rCell.left + rCell.delta + pCard.right + pDelta == tCell.bottom + tCell.delta + pCard.top + pDelta:
    getCards.append((True, pCell - 3))
    getCards.append((False, pCell + 1))

  if tCell != None and bCell != None \
  and tCell.owner != pCard.owner\
  and bCell.top + bCell.delta + pCard.bottom + pDelta == tCell.bottom + tCell.delta + pCard.top + pDelta:
    getCards.append((True, pCell - 3))
    getCards.append((True, pCell + 3))

  if tCell != None and lCell != None \
  and tCell.owner != pCard.owner\
  and lCell.right + lCell.delta + pCard.left + pDelta == tCell.bottom + tCell.delta + pCard.top + pDelta:
    getCards.append((True, pCell - 3))
    getCards.append((False, pCell - 1))

  if len(getCards) >= 2:
    return [card for card in getCards if pDeck.cells[card[1]].owner != pCard.owner]
  else:
    return []


def evaluateCombo(pDeck, pCards):
  """Evaluates combo rule"""
  getCards = []
  for tCard in pCards:
    pCard = pDeck.cells[tCard[1]]
    getCards.extend(evaluateNormal(pDeck, pCard, pCard.deckCell, 0))
  return getCards


def evaluateMove(pCard, pCell):
  """Evaluates a move"""
  def getMoveCards(pDeck, pCards, pOwner):
    for cell in pCards:
      tCard = pDeck.cells[cell[1]]
      tCard.owner = pOwner
  tMove = move()
  getCards = []
  getNormal = []

  tDeck = formVars.get("deck").copy()

  #Controllo regola ELEMENTALE:
  if formVars.get("gameType").elemental:
    if tDeck.elements[pCell] != None:
      if tDeck.elements[pCell] == pCard.elemental:
        tMove.cardDelta = 1
      else:
        tMove.cardDelta = -1

  #Regola coppia-muro:
  sameWall = False
  getSameWall = []
  if formVars.get("gameType").sameWall:
    getSameWall = evaluateSameWall(tDeck, pCard, pCell, tMove.cardDelta)
    if len(getSameWall) > 0:
      sameWall = True
      getCards.extend(getSameWall)
      #Prendo le carte:
      getMoveCards(tDeck, getSameWall, pCard.owner)

  #Regola coppia:
  same = False
  getSame = []
  if formVars.get("gameType").same:
    getSame = evaluateSame(tDeck, pCard, pCell, tMove.cardDelta)
    if len(getSame) > 0:
      same = True
      getCards.extend(getSame)
      #Prendo le carte:
      getMoveCards(tDeck, getSame, pCard.owner)

  #Regola PLUS:
  plus = False
  getPlus = []
  if formVars.get("gameType").plus:
    getPlus = evaluatePlus(tDeck, pCard, pCell, tMove.cardDelta)
    if len(getPlus) > 0:
      plus = True
      getCards.extend(getPlus)
      #Prendo le carte:
      getMoveCards(tDeck, getPlus, pCard.owner)

  #Regola normale:
  getNormal = evaluateNormal(tDeck, pCard, pCell, tMove.cardDelta)

  #Tolgo dalla regola normale le carte gia' prese con altre regole:
  remove = []
  for i, cell in enumerate(getNormal):
    if cell in getSame:
      remove.append(cell)
    elif cell in getSameWall:
      remove.append(cell)
    elif cell in getPlus:
      remove.append(cell)

  for i in remove:
    getNormal.remove(i)

  #Prendo le carte:
  getMoveCards(tDeck, getNormal, pCard.owner)
  getCards.extend(getNormal)

  tDeck.cells[pCell] = pCard

  #Regola COMBO:
  combo = False
  getCombo = []
  if formVars.get("gameType").combo and (same or plus):
      getCombo = evaluateCombo(tDeck, getCards)
      if len(getCombo) > 0:
        combo = True
        getCards.extend(getCombo)
        #Prendo le carte:
        getMoveCards(tDeck, getCombo, pCard.owner)

  tMove.exposed = tDeck.exposedCells(pCard.owner)

  #Calcolo valore mossa:
  for tCard in tDeck.cells:
    if tCard and tCard.owner == pCard.owner:
      tMove.value += tCard.value()
  tMove.getCards = getCards

  tMove.getCardsSame = getSame
  tMove.same = same
  tMove.getCardsSameWall = getSameWall
  tMove.sameWall = sameWall
  tMove.getCardsPlus = getPlus
  tMove.plus = plus
  tMove.getCardsCombo = getCombo
  tMove.combo = combo
  tMove.getCardsNormal = getNormal

  #Penalizzo il valore piu' basso:
  tMove.value = tMove.value - (10 - tMove.exposed[2])

  #Penalizzo i lati esposti:
  tMove.value = tMove.value - (10 * tMove.exposed[0] - tMove.exposed[1])

  #Bonus per ogni carta in piu' dell'avversario:
  if pCard.owner == "P1":
    tMove.value = tMove.value + (len(tDeck.playerCards("P1")) - len(tDeck.playerCards("P2"))) * 4
  else:
    tMove.value = tMove.value + (len(tDeck.playerCards("P2")) - len(tDeck.playerCards("P1"))) * 4

  tMove.card = pCard
  tMove.cell = pCell
  tMove.deckCell = pCell

  del tDeck
  return tMove


def suddenDeath():
  """Sudden death rule"""
  #Ricarico lo sfondo senza elementi:
  formVars.set(psp2d.Image('themes/' + globalVars.get("userData").THEME + '/backgrounddeck.png'), "bkgImage")

  #Creo gli elementi:
  createElements()

  #Controllo la carta non giocata:
  leftCard = None
  for tCard in formVars.get("cardsP1").elements + formVars.get("cardsP2").elements:
    if tCard.deckCell == None:
      leftCard = tCard
      tCard.status = None
      break

  formVars.get("cardsP1").elements = []
  formVars.get("cardsP2").elements = []
  for i, tCard in enumerate(formVars.get("deck").cells):
    tCard.deckCell = None
    tCard.status = None
    tCard.delta = 0
    formVars.get("deck").cells[i] = None
    graph = formGraphics.get(tCard.data)
    if graph:
      graph.priority = i
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      if i % 2 == 0:
        anim.code = functions.changeOwner
      else:
        anim.code = functions.changeOwnerVertical

      if tCard.owner == "P1":
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, None]
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
        formVars.get("cardsP1").elements.append(tCard)
      else:
        if formVars.get("gameType").open:
          anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, None]
          anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
        else:
          anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, None, None]
        formVars.get("cardsP2").elements.append(tCard)
      #L'ultima e' esclusiva:
      if i == len(formVars.get("deck").cells) - 1:
        anim.priority = 50
        anim.exclusive = True
      graph.animation = anim

  #Carta non giocata:
  graph = formGraphics.get(leftCard.data)
  if graph:
    if leftCard.owner == "P1":
      formVars.get("cardsP1").elements.append(leftCard)
    else:
      formVars.get("cardsP2").elements.append(leftCard)
    graph.priority = i + 1

  #Pausa:
  graph = graphicElement()
  graph.name = "WAIT"
  anim = animation()
  anim.name = "WAIT"
  anim.priority = -50
  anim.exclusive = True
  anim.lockKeyboard = True
  anim.code = functions.message
  graph.animation = anim
  formGraphics.set(graph)

  #Spostamento carte:
  graph = graphicElement()
  graph.name = "MOVE_CARDS"
  anim = animation()
  anim.name = "MOVE_CARDS"
  anim.priority = -100
  anim.exclusive = True
  def code(globalVars, formVars, target):
    #Animo le carte per farle tornare in posizione:
    contP1 = 0
    contP2 = 0
    for i, tCard in enumerate(formVars.get("cardsP1").elements+formVars.get("cardsP2").elements):
      graph = formGraphics.get(tCard.data)
      if tCard.owner == "P1":
        graph.priority = contP1
        anim = animation()
        anim.name = "MOVECARD_" + str(tCard.ID)
        anim.data = [10, 3 + 43 * contP1, 99]
        anim.priority = 99 - i
        anim.code = functions.moveFromTo
        graph.animation = anim
        contP1 += 1
      elif tCard.owner == "P2":
        graph.priority = contP2
        anim = animation()
        anim.name = "MOVECARD_" + str(tCard.ID)
        anim.data = [396, 3 + 43 * contP2, 99]
        anim.priority = 99 - i
        anim.code = functions.moveFromTo
        graph.animation = anim
        contP2 += 1
      if i == len(formVars.get("cardsP1").elements+formVars.get("cardsP2").elements) - 1:
        anim.priority = -100
        anim.exclusive = True
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Ricomincio a giocare:
  graph = graphicElement()
  graph.name = "START_GAME_SUDDEN"
  anim = animation()
  anim.name = "START_GAME_SUDDEN"
  anim.priority = -200
  anim.exclusive = True
  def code(globalVars, formVars, target):
    if formVars.get("gameType").type == "MULTI":
      formVars.get("gameType").lastTurn = -1
      #Mi metto in attesa (per capire che deve iniziare):
      waitOpponentsMove()
    elif formVars.get("gameType").type == "SINGLE":
      #Chi inizia?
      random = _random.Random()
      random.seed()
      rnd = int(round(random.random() * 1))
      if rnd == 0:
        formVars.set("P1", "firstTurn")
      else:
        formVars.set("P2", "firstTurn")

    #Messaggio CHALLENGE:
    formVars.set("SINGLE", "status")
    graph = graphicElement()
    graph.name = "CHALLENGE"
    graph.font = globalVars.get("fonts").get("fontResult")
    msg = globalVars.get("messages").CHALLENGE
    tWidth = functions.textWidth(graph.font, msg)
    tX = round((480 - tWidth) / 2)
    graph.posX = tX
    graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
    graph.text = msg
    anim = animation()
    anim.name = "CHALLENGE"
    anim.priority = 50
    anim.exclusive = True
    anim.front = True
    anim.data = None
    anim.code = functions.message
    graph.animation = anim
    formGraphics.set(graph)

    formVars.set("SELECTING", "status")

    #Selezione prima carta:
    graph = graphicElement()
    graph.name = "START_MATCH"
    graph.data = False
    graph.posX = 0
    graph.posY = 0
    anim = animation()
    anim.name = "START_MATCH"
    anim.priority = 10
    def code(globalVars, formVars, target):
      #Controllo chi deve iniziare:
      if target.data == False:
        target.data = True
        target.name = "DEL_" + target.name
        if formVars.get("firstTurn") == "P1":
          selectFirstCardP1()
        else:
          computerMove()
    anim.code = code
    graph.animation = anim
    formGraphics.set(graph)
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def showHelp():
  """Shows help on buttons"""
  if formVars.get("status") == "VIEWRULES":
    return False

  if formVars.get("status") == "SELECTING":
    formGraphics.get("CARD_SELECTOR").visible = False
  else:
    sel = formVars.get("formMenu").get("MENU_SELECTOR_LEFT")
    if sel:
      sel.visible = False
    sel = formVars.get("formMenu").get("MENU_SELECTOR_RIGHT")
    if sel:
      sel.visible = False
  return True


def hideHelp():
  """Hides help"""
  if formVars.get("status") == "SELECTING":
    formGraphics.get("CARD_SELECTOR").visible = True
  else:
    sel = formVars.get("formMenu").get("MENU_SELECTOR_LEFT")
    if sel:
      sel.visible = True
    sel = formVars.get("formMenu").get("MENU_SELECTOR_RIGHT")
    if sel:
      sel.visible = True


def score():
  """Returns [CardsP1, CardsP2]"""
  p1 = 0
  p2 = 0
  for tCard in formVars.get("deck").cells:
    if tCard and tCard.owner == "P1":
      p1 += 1
    elif tCard and tCard.owner == "P2":
      p2 += 1

  #Carte in mano:
  for tCard in formVars.get("cardsP1").elements:
    if tCard.deckCell == None:
      p1 += 1

  for tCard in formVars.get("cardsP2").elements:
    if tCard.deckCell == None:
      p2 += 1
  return (p1, p2)


def updateScore():
  """Updates the scores on video"""

  tScore = score()
  graph = formGraphics.get("SCORE_P1")
  if not graph:
    graph = graphicElement()
    graph.name = "SCORE_P1"
    graph.posX = 100
    graph.posY = 215
    formGraphics.set(graph)
  graph.image = formVars.get("scoreImages")[tScore[0]]

  graph = formGraphics.get("SCORE_P2")
  if not graph:
    graph = graphicElement()
    graph.name = "SCORE_P2"
    graph.posX = 354
    graph.posY = 215
    formGraphics.set(graph)
  graph.image = formVars.get("scoreImages")[tScore[1]]


def initInGameMenu():
  """Init in-game menu"""
  graph = formGraphics.get("TITLE")
  if not graph:
    graph = graphicElement()
    graph.name = "TITLE"
    graph.priority = 100
    width = 200
    graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirmch_title.png')
    graph.posX = -width
    graph.posY = 50
    anim = animation()
    anim.name = "TITLE"
    anim.priority = 100
    anim.data = [(480 - width) / 2, graph.posY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formGraphics.set(graph)

  graph = formGraphics.get("TITLET")
  if not graph:
    graph = graphicElement()
    graph.name = "TITLET"
    graph.priority = 101
    graph.font = globalVars.get("fonts").get("fontMenu")
    graph.text = globalVars.get("messages").INGAMEMENU
    graph.posY = 53
    graph.posX = -functions.textWidth(graph.font, graph.text)
    anim = animation()
    anim.name = "TITLET"
    anim.priority = 100
    anim.data = [(480 - functions.textWidth(graph.font, graph.text)) / 2, graph.posY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formGraphics.set(graph)
  else:
    graph.image = None
    graph.text = globalVars.get("messages").INGAMEMENU
    graph.posX = (480 - functions.textWidth(graph.font, graph.text)) / 2
  formVars.get("gameMenu").clear()
  formVars.get("gameMenu").menuWidth = 150
  formVars.get("gameMenu").menuHeight = 100
  formVars.get("gameMenu").graphicPriority = 200
  formVars.get("gameMenu").animationPriority = 2000
  formVars.get("gameMenu").posX = (480 - formVars.get("gameMenu").menuWidth) / 2
  formVars.get("gameMenu").posY = (272 - formVars.get("gameMenu").menuHeight) / 2
  formVars.get("gameMenu").menuAlignment = globalVars.get("themeOptions").MENU_ALIGN
  formVars.get("gameMenu").menuVAlignment = globalVars.get("themeOptions").MENU_VALIGN
  formVars.get("gameMenu").background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirm.png')


def inGameMenu():
  #Costruisco il menu di gioco:
  initInGameMenu()
  menu = formVars.get("gameMenu")
  if formVars.get("data")[0] ==  "ENDED":
    tElement = menuElement()
    tElement.text = globalVars.get("messages").REPLAY
    tElement.helpText = globalVars.get("messages").REPLAYH
    tElement.value = "REPLAY"
    def code(globalVars, formVars, menu, element):
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      globalVars.set("mdlTripleTriad", "formName")
      globalVars.set([formVars.get("gameType")], "formParams")
      formVars.remove("gameType", False)
      globalVars.set("YES", "breakLoop")
    tElement.trigger_select = code
    menu.elements.append(tElement)
  else:
    tElement = menuElement()
    tElement.text = globalVars.get("messages").CONTINUE
    tElement.helpText = globalVars.get("messages").CONTINUEH
    tElement.value = "CONTINUE"
    def code(globalVars, formVars, menu, element):
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      formVars.set(formVars.get("data")[0],"status")
      globalVars.set(formVars.get("data")[1], "message")
      formVars.set(None, "data")
      formVars.get("formMenu").delGraphics()
      formVars.set(None, "formMenu")
      delGraphic(formGraphics, "TITLE")
      delGraphic(formGraphics, "TITLET")
      if formVars.get("status") == "SELECTING":
        graph = menu.formGraphics.get("CARD_SELECTOR")
        graph.visible = True
    tElement.trigger_select = code
    menu.elements.append(tElement)

    tElement = menuElement()
    tElement.text = globalVars.get("messages").VIEW_RULES
    tElement.helpText = globalVars.get("messages").VIEW_RULESH
    tElement.value = "VIEW_RULES"
    def code(globalVars, formVars, menu, element):
      showMatchRules()
    tElement.trigger_select = code
    menu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").EXIT
  tElement.helpText = globalVars.get("messages").EXITH
  tElement.value = "EXIT"
  def code(globalVars, formVars, menu, element):
    graph = menu.formGraphics.get("TITLET")
    graph.priority = 100
    graph.image = None
    graph.text = globalVars.get("messages").CONFIRMEXIT
    graph.posX = (480 - functions.textWidth(graph.font, graph.text)) / 2

    graph = menu.formGraphics.get("TITLE")
    width = 200
    graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirmch_title.png')
    graph.posX = (480 - width) / 2

    globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
    globalVars.get("playingSound").start()
    formVars.get("gameMenu").delGraphics()
    formVars.set(confirm(globalVars, formVars, formGraphics), "formMenu")
    formVars.get("formMenu").graphicPriority = 200
    formVars.get("formMenu").background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirm.png')
    def code(globalVars, formVars, menu, element):
      #Torno al menu di gioco:
      globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
      globalVars.get("playingSound").start()
      menu.delGraphics()
      formVars.set(inGameMenu(), "formMenu")
      formVars.get("formMenu").selected = "EXIT"
      formVars.get("formMenu").createGraphicElements("BOTTOM")
    formVars.get("formMenu").trigger_cancel = code

    tElement = formVars.get("formMenu").elements[0]
    def code(globalVars, formVars, menu, element):
      #Scarico il font:
      globalVars.get("fonts").remove("fontResult")

      #Passo al main menu:
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      globalVars.set("mdlMainMenu", "formName")
      globalVars.set("YES", "breakLoop")
    tElement.trigger_select = code
    tElement = formVars.get("formMenu").elements[1]
    def code(globalVars, formVars, menu, element):
      #Torno al menu precedente:
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      graph = menu.formGraphics.get("TITLET")
      graph.image = None
      graph.text = globalVars.get("messages").INGAMEMENU
      graph.posX = (480 - functions.textWidth(graph.font, graph.text)) / 2
      formVars.get("formMenu").delGraphics()
      formVars.set(inGameMenu(), "formMenu")
      formVars.get("formMenu").selected = "EXIT"
      formVars.get("formMenu").createGraphicElements("BOTTOM")
    tElement.trigger_select = code
    formVars.get("formMenu").createGraphicElements("BOTTOM")
  tElement.trigger_select = code
  menu.elements.append(tElement)

  def code(globalVars, formVars, menu, element):
    if formVars.get("data")[0] != "ENDED":
      globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
      globalVars.get("playingSound").start()
      formVars.set(formVars.get("data")[0], "status")
      globalVars.set(formVars.get("data")[1], "message")
      formVars.set(None, "data")
      formVars.get("formMenu").delGraphics()
      formVars.set(None, "formMenu")
      delGraphic(formGraphics, "TITLE")
      delGraphic(formGraphics, "TITLET")
      if formVars.get("status") == "SELECTING":
        graph = menu.formGraphics.get("CARD_SELECTOR")
        graph.visible = True
  menu.trigger_cancel = code
  menu.selected = menu.elements[0].value
  return menu


def inGameMenuMulti():
  """In-game menu for multiplayer game"""
  initInGameMenu()
  menu = formVars.get("gameMenu")
  tElement = menuElement()
  tElement.text = globalVars.get("messages").CONTINUE
  tElement.helpText = globalVars.get("messages").CONTINUEH
  tElement.value = "CONTINUE"
  def code(globalVars, formVars, menu, element):
    globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
    globalVars.get("playingSound").start()
    formVars.set(formVars.get("data")[0], "status")
    globalVars.set(formVars.get("data")[1], "message")
    formVars.set(None, "data")
    formVars.get("formMenu").delGraphics()
    formVars.set(None, "formMenu")
    delGraphic(formGraphics, "TITLE")
    delGraphic(formGraphics, "TITLET")
    if formVars.get("status") == "SELECTING":
      graph = menu.formGraphics.get("CARD_SELECTOR")
      graph.visible = True
    elif formVars.get("status") == "TRADING":
      graph = menu.formGraphics.get("TRADE_SELECTOR")
      if graph:
        graph.visible = True
  tElement.trigger_select = code
  menu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").VIEW_RULES
  tElement.helpText = globalVars.get("messages").VIEW_RULESH
  tElement.value = "VIEW_RULES"
  def code(globalVars, formVars, menu, element):
    showMatchRules()
  tElement.trigger_select = code
  menu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").CHATMSG
  tElement.helpText = globalVars.get("messages").CHATMSGH
  tElement.value = "CHATMSG"
  def code(globalVars, formVars, menu, element):
    globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
    globalVars.get("playingSound").start()
    menu.delGraphics()
    delGraphic(formGraphics, "TITLE")
    delGraphic(formGraphics, "TITLET")
    showChatMessageMenu()
  tElement.trigger_select = code
  menu.elements.append(tElement)

  tElement = menuElement()
  tElement.text = globalVars.get("messages").ABORT_MATCH
  tElement.helpText = globalVars.get("messages").ABORT_MATCHH
  tElement.value = "ABORT_MATCH"
  def code(globalVars, formVars, menu, element):
    #Scarico il font:
    globalVars.get("fonts").remove("fontResult")
    #Esco dalla stanza:
    globalVars.get("net").leaveRoom(formVars.get("gameType").roomID, "S")
    #Cambio lo status:
    globalVars.get("net").changeStatus("N")
    #Ritorno al gioco:
    globalVars.set(["RESTORE"], "formParams")
    globalVars.set("mdlGame", "formName")
    globalVars.set("YES", "breakLoop")
  tElement.trigger_select = code
  menu.elements.append(tElement)

  def code(globalVars, formVars, menu, element):
    if formVars.get("data")[0] != "ENDED":
      globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
      globalVars.get("playingSound").start()
      formVars.set(formVars.get("data")[0], "status")
      globalVars.set(formVars.get("data")[1], "message")
      formVars.set(None, "data")
      menu.delGraphics()
      formVars.set(None, "formMenu")
      delGraphic(formGraphics, "TITLE")
      delGraphic(formGraphics, "TITLET")
      if formVars.get("status") == "SELECTING":
        graph = menu.formGraphics.get("CARD_SELECTOR")
        graph.visible = True
      elif formVars.get("status") == "TRADING":
        graph = menu.formGraphics.get("TRADE_SELECTOR")
        if graph:
          graph.visible = True
  menu.trigger_cancel = code
  menu.selected = menu.elements[0].value
  return menu


def hideAllMenues():
  """Hide all menuse, rules info..."""
  #Nascondo il selector se c'e':
  graph = formGraphics.get("CARD_SELECTOR")
  if graph:
    graph.visible = False
  graph = formGraphics.get("TRADE_SELECTOR")
  if graph:
    graph.visible = False

  #Nascondo le regole se ci sono:
  if formVars.get("status") == "VIEWRULES":
    hideMatchRules()

  #Nascondo l'help se c'e':
  if formVars.get("status") == "HELP":
    formVars.get("helpItem").hide()

  #Nascondo il menu se c'e':
  menu = formVars.get("formMenu")
  if menu == formVars.get("gameMenu") \
  or menu and menu.name == "CONFIRMABORT":
    delGraphic(formGraphics, "CONFIRM")
    delGraphic(formGraphics, "TITLE")
    delGraphic(formGraphics, "TITLET")
    menu.delGraphics()
  elif menu == formVars.get("chatMessageMenu"):
    hideChatMessageMenu(pReturn=False)


def showChatMessageMenu():
  """Show chat message menu"""
  hideAllMenues()
  formVars.set("MENU", "status")
  chatMessageMenu = formVars.get("chatMessageMenu")
  chatMessageMenu.clear()
  chatMessageMenu.name = "CHATMESSAGEMENU"
  chatMessageMenu.background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_chat_menu.png')
  chatMessageMenu.graphicPriority = 200
  chatMessageMenu.animationPriority = 2000
  chatMessageMenu.menuWidth = 290
  chatMessageMenu.menuHeight = 127
  chatMessageMenu.posX = 95
  chatMessageMenu.posY = 25
  chatMessageMenu.menuAlignment = "LEFT"
  chatMessageMenu.selector = "RIGHT"

  #Titolo
  title = graphicElement()
  title.name = "CHATMESSAGETITLE"
  title.priority = 100
  title.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_shout_title.png')
  globalVars.get("fonts").get("fontMenuSel").drawText(title.image, (title.image.width - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_T_CHAT_TITLE)) / 2, 3, globalVars.get("messages").GM_T_CHAT_TITLE)
  title.posX = chatMessageMenu.posX
  title.posY = chatMessageMenu.posY - title.image.height - 2
  if formVars.get("formMenu") != chatMessageMenu:
    anim = animation()
    anim.name = title.name
    anim.priority = chatMessageMenu.animationPriority
    anim.data = [title.posX, title.posY, 99]
    title.posX = title.posX - 480
    anim.code = functions.moveFromTo
    title.animation = anim
  formGraphics.set(title)

  #Area chat
  area = graphicElement()
  area.name = "CHATAREA"
  area.priority = 100
  area.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_chat_area.png')

  posX = 2
  posY = 2
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenu"), "I")
  for i, msg in enumerate(formVars.get("chatMessages")):
    globalVars.get("fonts").get("fontMenu").drawText(area.image, posX, area.image.height - (posY + (i + 1) * tHeight), msg[1])

  globalVars.get("fonts").get("fontMenuSel").drawText(title.image, (title.image.width - functions.textWidth(globalVars.get("fonts").get("fontMenuSel"), globalVars.get("messages").GM_T_CHAT_TITLE)) / 2, 3, globalVars.get("messages").GM_T_CHAT_TITLE)
  area.posX = chatMessageMenu.posX
  area.posY = 25
  if formVars.get("formMenu") != chatMessageMenu:
    anim = animation()
    anim.name = area.name
    anim.priority = chatMessageMenu.animationPriority
    anim.data = [area.posX, area.posY, 99]
    area.posX = 480
    anim.code = functions.moveFromTo
    area.animation = anim
  formGraphics.set(area)

  chatMessageMenu.posY = area.posY + area.image.height + 2
  def code(globalVars, formVars, menu, element):
    hideChatMessageMenu()
  chatMessageMenu.trigger_cancel = code

  chatMessageMenu.elements = []
  #Opzione popup:
  tElement = menuElement()
  tElement.text = globalVars.get("messages").CHATPOPUP
  tElement.helpText = globalVars.get("messages").CHATPOPUPH
  tElement.value = "POPUP|" + globalVars.get("userData").CHATPOPUP
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").NO
  tSubElement.value = "NO"
  tElement.subMenu.append(tSubElement)
  tSubElement = menuElement()
  tSubElement.text = globalVars.get("messages").YES
  tSubElement.value = "YES"
  tElement.subMenu.append(tSubElement)
  chatMessageMenu.elements.append(tElement)

  #Tastiera:
  tElement = menuElement()
  tElement.text = globalVars.get("messages").CHATWRITE
  tElement.helpText = globalVars.get("messages").CHATWRITEH
  tElement.value = "WRITE"
  def code(globalVars, formVars, menu, element):
    """write message"""
    hideChatMessageMenu(False)
    formVars.set(None, "formMenu")
    virtualKeyboard = vKeyboard(globalVars, formVars, formVars.get("formGraphics"))
    virtualKeyboard.maxLength = 50
    virtualKeyboard.posX = 480 / 2 - 164 / 2
    virtualKeyboard.posY = 270 / 2 - 164 / 2
    virtualKeyboard.text = ""
    virtualKeyboard.callbackFunction = writeChatCallback
    #virtualKeyboard.returnGraphicElement = graph
    virtualKeyboard.createGraphicElement("BOTTOM")
    formVars.set(virtualKeyboard, "formKeyboard")

  tElement.trigger_select = code
  chatMessageMenu.elements.append(tElement)

  #Custom message:
  for chatMSG in globalVars.get("userData").customMsg.CHATMSG:
    tElement = menuElement()
    tElement.text = chatMSG
    tElement.value = chatMSG
    def code(globalVars, formVars, menu, element):
      """chat message"""
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      globalVars.get("net").sendChatMessage(formVars.get("gameType").roomID, "", element.value)
      hideChatMessageMenu(True)
    tElement.trigger_select = code
    chatMessageMenu.elements.append(tElement)

  #Standard message:
  numMsg = 0
  while True:
    numMsg += 1
    try:
      msg = eval("globalVars.get(\"messages\").STDCHATMSG_" + str(numMsg).zfill(2))
    except AttributeError:
      break
    tElement = menuElement()
    tElement.text = msg
    tElement.value = str(numMsg).zfill(2)
    def code(globalVars, formVars, menu, element):
      """chat message"""
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      globalVars.get("net").sendChatMessage(formVars.get("gameType").roomID, element.value, "")
      hideChatMessageMenu(True)
    tElement.trigger_select = code
    chatMessageMenu.elements.append(tElement)

  chatMessageMenu.selectFirstElement()
  if formVars.get("formMenu") != chatMessageMenu:
    formVars.set(chatMessageMenu, "formMenu")
    chatMessageMenu.createGraphicElements("LEFT")
  else:
    chatMessageMenu.createGraphicElements()


def hideChatMessageMenu(pReturn=True):
  """Hide chat message menu"""
  globalVars.get("userData").CHATPOPUP = formVars.get("formMenu").getElement("POPUP").value.split("|")[1]
  functions.saveOptions(globalVars.get("userData"))
  delGraphic(formGraphics, "CHATMESSAGETITLE")
  delGraphic(formGraphics, "CHATAREA")
  formVars.get("chatMessageMenu").delGraphics()
  if pReturn:
    formVars.set(inGameMenuMulti(), "formMenu")
    formVars.get("formMenu").createGraphicElements("BOTTOM")


def writeChatCallback(okCancel, text):
  """Callback for virtual keyboard"""
  if okCancel == True and len(text) > 0:
    #Invio il messaggio:
    globalVars.get("net").sendChatMessage(formVars.get("gameType").roomID, "", text)
    #Chiedo conferma per salvarlo:
    graph = graphicElement()
    graph.name = "TITLET"
    graph.priority = 101
    graph.font = globalVars.get("fonts").get("fontMenu")
    graph.text = globalVars.get("messages").SAVECHATMESSAGE
    graph.posY = 53
    graph.posX = -functions.textWidth(graph.font, graph.text)
    anim = animation()
    anim.name = "TITLET"
    anim.priority = 100
    anim.data = [(480 - functions.textWidth(graph.font, graph.text)) / 2, graph.posY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formGraphics.set(graph)

    graph = graphicElement()
    graph.name = "TITLE"
    graph.priority = 100
    width = 200
    graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirmch_title.png')
    graph.posX = -width
    graph.posY = 50
    anim = animation()
    anim.name = "TITLE"
    anim.priority = 100
    anim.data = [(480 - width) / 2, graph.posY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formGraphics.set(graph)

    formVars.set(confirm(globalVars, formVars, formGraphics), "formMenu")
    formVars.get("formMenu").graphicPriority = 200
    formVars.get("formMenu").background_image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_confirm.png')
    def code(globalVars, formVars, menu, element):
      #Torno al menu di chat:
      globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
      globalVars.get("playingSound").start()
      delGraphic(formGraphics, "TITLE")
      delGraphic(formGraphics, "TITLET")
      formVars.get("formMenu").delGraphics()
      showChatMessageMenu()
    formVars.get("formMenu").trigger_cancel = code
    tElement = formVars.get("formMenu").elements[1]
    tElement.trigger_select = code

    tElement = formVars.get("formMenu").elements[0]
    def code(globalVars, formVars, menu, element):
      #Salvo il messaggio nei custom messages:
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      globalVars.get("userData").customMsg.CHATMSG.append(text)
      functions.saveCustomMessages(globalVars.get("userData").customMsg)
      delGraphic(formGraphics, "TITLE")
      delGraphic(formGraphics, "TITLET")
      formVars.get("formMenu").delGraphics()
      showChatMessageMenu()

    tElement.trigger_select = code
    formVars.get("formMenu").createGraphicElements("BOTTOM")
  else:
    showChatMessageMenu()


def getChatMessagesCallback(pResponse, pError):
  """Get chat messages callback"""
  formGraphics.get("CHATMESSAGE").visible = True
  if pError:
    waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  messages = []
  for numMsg, p in enumerate(response):
    cp = p.split("||")
    if len(cp) >= 2:
      try:
        msg = cp[0] + ": " + eval("globalVars.get(\"messages\").STDCHATMSG_" + cp[1].zfill(2))
      except:
        msg = cp[0] + ": " + cp[1]
      messages.append((cp[0], msg))
  messages.reverse()

  #Controllo se ce ne sono di nuovi:
  if formVars.get("oldChatMessages") != messages:
    formVars.set(messages, "chatMessages")
    newM = ["",""]
    #Cerco il messaggio nuovo:
    for tNewM in messages:
      if tNewM not in formVars.get("oldChatMessages"):
        newM = tNewM
        break

    if newM[0] != globalVars.get("player").name \
    or formVars.get("formMenu") == formVars.get("chatMessageMenu"):
      #Controllo opzione di popup:
      if globalVars.get("userData").CHATPOPUP == "YES":
        if not formVars.get("data"):
          formVars.set([formVars.get("status"), globalVars.get("message")], "data")
        showChatMessageMenu()
      else:
        waitMessage(globalVars, formVars, newM[1], False, 3)
    formVars.set(messages, "oldChatMessages")


def makeMove(pMove, pAnimPriority=100):
  """Makes a move"""
  def getCards(pMove, pCards, pAnimPriority):
    #Prendo le carte con animazione:
    for i, cell in enumerate(pCards):
      tCard = formVars.get("deck").cells[cell[1]]
      tCard.owner = pMove.card.owner
      graph = formGraphics.get(tCard.data)
      anim = animation()
      anim.name = "CHANGEOWNER"
      if i == len(pCards) - 1:
        anim.exclusive = True
        if not cell[0]:
          anim.code = functions.changeOwnerSound
        else:
          anim.code = functions.changeOwnerVerticalSound
      else:
        if not cell[0]:
          anim.code = functions.changeOwner
        else:
          anim.code = functions.changeOwnerVertical
      pAnimPriority -= 1
      anim.priority = pAnimPriority
      if pMove.card.owner == "P1":
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, tCard.writeValues(psp2d.Image(tCard.buffImageP1))]
      else:
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, tCard.writeValues(psp2d.Image(tCard.buffImageP2))]
      graph.animation = anim
    return pAnimPriority

  #Ridisegno i valori della carta:
  pAnimPriority -= 1
  graph = graphicElement()
  graph.name = "CHANGEIMAGECARD"
  anim = animation()
  anim.name = graph.name
  anim.priority = pAnimPriority
  anim.data = [pMove.card]
  def code(globalVars, formVars, target):
    card = target.animation.data[0]
    graph = formGraphics.get(card.data)
    if card.owner == "P1":
      graph.image = card.writeValues(psp2d.Image(card.buffImageP1))
    elif card.owner == "P2":
      graph.image = card.writeValues(psp2d.Image(card.buffImageP2))
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Regola COPPIA-MURO:
  if pMove.sameWall:
    #Messaggio:
    graph = graphicElement()
    graph.name = "SAMEWALL"
    graph.font = globalVars.get("fonts").get("fontResult")
    msg = globalVars.get("messages").SAMEWALL
    tWidth = functions.textWidth(graph.font, msg)
    tX = round((480 - tWidth) / 2)
    graph.posX = tX
    graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
    graph.text = msg
    anim = animation()
    anim.name = "SAMEWALL"
    anim.front = True
    anim.exclusive = True
    pAnimPriority -= 1
    anim.priority = pAnimPriority
    anim.data = None
    anim.code = functions.specialMessage
    graph.animation = anim
    formGraphics.set(graph)

    #Mi prendo le carte che mi spettano:
    pAnimPriority = getCards(pMove, pMove.getCardsSameWall, pAnimPriority)

  #Regola COPPIA:
  if pMove.same:
    #Messaggio:
    graph = graphicElement()
    graph.name = "SAME"
    graph.font = globalVars.get("fonts").get("fontResult")
    msg = globalVars.get("messages").SAME
    tWidth = functions.textWidth(graph.font, msg)
    tX = round((480 - tWidth) / 2)
    graph.posX = tX
    graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
    graph.text = msg
    anim = animation()
    anim.name = "SAME"
    anim.front = True
    anim.exclusive = True
    pAnimPriority -= 1
    anim.priority = pAnimPriority
    anim.data = None
    anim.code = functions.specialMessage
    graph.animation = anim
    formGraphics.set(graph)

    #Mi prendo le carte che mi spettano:
    pAnimPriority = getCards(pMove, pMove.getCardsSame, pAnimPriority)

  #Regola PLUS:
  if pMove.plus:
    #Messaggio:
    graph = graphicElement()
    graph.name = "PLUS"
    graph.font = globalVars.get("fonts").get("fontResult")
    msg = globalVars.get("messages").PLUS
    tWidth = functions.textWidth(graph.font, msg)
    tX = round((480 - tWidth) / 2)
    graph.posX = tX
    graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
    graph.text = msg
    anim = animation()
    anim.name = "PLUS"
    anim.front = True
    anim.exclusive = True
    pAnimPriority -= 1
    anim.priority = pAnimPriority
    anim.data = None
    anim.code = functions.specialMessage
    graph.animation = anim
    formGraphics.set(graph)

    #Mi prendo le carte che mi spettano:
    pAnimPriority = getCards(pMove, pMove.getCardsPlus, pAnimPriority)

  #Regola COMBO:
  if pMove.combo:
    #Messaggio:
    graph = graphicElement()
    graph.name = "COMBO"
    graph.font = globalVars.get("fonts").get("fontResult")
    msg = globalVars.get("messages").COMBO
    tWidth = functions.textWidth(graph.font, msg)
    tX = round((480 - tWidth) / 2)
    graph.posX = tX
    graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
    graph.text = msg
    anim = animation()
    anim.name = "COMBO"
    anim.front = True
    anim.exclusive = True
    pAnimPriority -= 1
    anim.priority = pAnimPriority
    anim.data = None
    anim.code = functions.specialMessage
    graph.animation = anim
    formGraphics.set(graph)

    #Mi prendo le carte che mi spettano:
    pAnimPriority = getCards(pMove, pMove.getCardsCombo, pAnimPriority)


  #Regola NORMALE:
  pAnimPriority = getCards(pMove, pMove.getCardsNormal, pAnimPriority)

  #Aggiorno il punteggio:
  graph = graphicElement()
  graph.name = "UPDATE_SCORE"
  anim = animation()
  anim.name = graph.name
  pAnimPriority -= 1
  anim.priority = pAnimPriority
  anim.exclusive = True
  def code(globalVars, formVars, target):
    target.name = "DEL_" + target.name
    updateScore()
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def playerMove():
  """Mossa del giocatore"""
  #Rilascio la carta da posizionare sul tavolo:
  tMove = releaseCardP1()
  makeMove(tMove)
  globalVars.set("", "message")

  #Controllo la fine della partita:
  if len(formVars.get("deck").freeCells()) == 0 and formVars.get("gameType").type == "SINGLE":
    graph = graphicElement()
    graph.name = "MATCHEND"
    anim = animation()
    anim.name = graph.name
    anim.priority = -100
    def code(globalVars, formVars, target):
      #if formVars.get("gameType").type == "SINGLE":
      matchEnd()
      target.name = "DEL_" + target.name
    anim.code = code
    graph.animation = anim
    formGraphics.set(graph)
  else:
    #Mossa avversario:
    if formVars.get("gameType").type == "SINGLE":
      graph = graphicElement()
      graph.name = "COMPUTERMOVE"
      anim = animation()
      anim.name = graph.name
      anim.priority = -100
      def code(globalVars, formVars, target):
        computerMove()
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)
    elif formVars.get("gameType").type == "MULTI":
      waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAIT)
      globalVars.get("net").makeMove(formVars.get("gameType").roomID, tMove.card.ID, tMove.cell, makeMoveCallback)


def makeMoveCallback(pResponse, pError):
  #Mossa del giocatore:
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      globalVars.get("net").makeMove(formVars.get("gameType").roomID, tMove.card.ID, tMove.cell, makeMoveCallback)
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) > 0:
    if response[0] != "OK":
      waitMessage(globalVars, formVars, globalVars.get("messages").GM_MOVEERROR, False, 5)
    else:
      if len(formVars.get("deck").freeCells()) == 0:
        graph = graphicElement()
        graph.name = "MATCHEND"
        anim = animation()
        anim.name = graph.name
        anim.priority = -100
        def code(globalVars, formVars, target):
          matchEndMulti()
          target.name = "DEL_" + target.name
        anim.code = code
        graph.animation = anim
        formGraphics.set(graph)
      else:
        #Aspetto la mossa dell'avversario:
        formVars.get("gameType").lastTurn += 1
        waitOpponentsMove()
  else:
    waitMessage(globalVars, formVars, globalVars.get("messages").GM_MOVEERROR, False, 5)


def computerMove():
  """Calcolo la mossa del computer"""
  freeCell = formVars.get("deck").freeCells()
  globalVars.set("", "message")

  last = None
  moves = []
  for i, tCard in enumerate(formVars.get("cardsP2").elements):
    if tCard.deckCell == None:
      if last != None:
        graph = graphicElement()
        graph.name = "MOVECARD_" + str(i)
        anim = animation()
        anim.name = graph.name
        anim.priority = 1000 - (i * 10)
        anim.exclusive = True
        anim.data = [last.data, anim.priority]
        def code(globalVars, formVars, target):
          graph = formGraphics.get(target.animation.data[0])
          anim = animation()
          anim.name = "MOVECARD"
          anim.data = [graph.posX + 20, graph.posY]
          anim.code = functions.moveFromTo
          anim.priority = target.animation.data[1]
          graph.animation = anim
          target.name = "DEL_" + target.name
        anim.code = code
        graph.animation = anim
        last.status = None
        graph.status = None
        formGraphics.set(graph)
        last.status = None

      graph = graphicElement()
      graph.name = "CARD_SOUND_" + str(i)
      anim = animation()
      anim.name = graph.name
      anim.priority = 1000 - (i * 10 + 1)
      anim.exclusive = True
      def code(globalVars, formVars, target):
        globalVars.set(formVars.get("cardSound"), "playingSound")
        globalVars.get("playingSound").start()
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)

      graph = graphicElement()
      graph.name = "MOVECARD_SEL_" + str(i)
      anim = animation()
      anim.name = graph.name
      anim.priority = 1000 - (i * 10 + 2)
      anim.exclusive = True
      anim.data = [tCard.data, anim.priority, tCard]
      def code(globalVars, formVars, target):
        graph = formGraphics.get(target.animation.data[0])
        anim = animation()
        anim.name = "MOVECARD"
        anim.data = [graph.posX - 20, graph.posY]
        anim.code = functions.moveFromTo
        anim.priority = target.animation.data[1]
        anim.exclusive = True
        graph.animation = anim
        if formVars.get("gameType").open:
          globalVars.set(target.animation.data[2].stringValue(), "message")
        target.name = "DEL_" + target.name
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)

      #Provo a metterla in tutte le celle:
      for cell in freeCell:
        tMove = move()
        #Calcolo il valore della mossa:
        tMove = evaluateMove(tCard, cell)
        moves.append(tMove)
      last = tCard

  i += 1
  last.status = None
  graph = graphicElement()
  graph.name = "MOVECARD_LAST_" + str(i)
  anim = animation()
  anim.name = graph.name
  anim.priority = 1000 - (i * 10 + 3)
  anim.exclusive = True
  anim.data = [last.data, anim.priority]
  def code(globalVars, formVars, target):
    graph = formGraphics.get(target.animation.data[0])
    anim = animation()
    anim.name = "MOVECARD"
    anim.data = [graph.posX + 20, graph.posY]
    anim.code = functions.moveFromTo
    anim.priority = target.animation.data[1]
    anim.exclusive = True
    graph.animation = anim
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Controllo le mosse calcolate:
  bestMoves = []
  bestValue = -100000
  for tMove in moves:
    if tMove.value > bestValue:
      bestMoves = [tMove]
      bestValue = tMove.value
    elif tMove.value == bestValue:
      bestMoves.append(tMove)

  #Scelgo la mossa:
  random = _random.Random()
  random.seed()
  bestMove = move()
  if len(bestMoves) >= 1:
    bestMove = bestMoves[int(round(random.random() * (len(bestMoves) - 1)))]

  #Faccio la mossa migliore:
  tCard = bestMove.card
  tCard.deckCell = bestMove.deckCell
  tCard.delta = bestMove.cardDelta
  formVars.get("deck").cells[bestMove.deckCell] = tCard

  globalVars.set(formVars.get("cardSound"), "playingSound")
  globalVars.get("playingSound").start()

  graph = graphicElement()
  graph.name = "MOVECARD_DECK_" + str(i)
  anim = animation()
  anim.name = graph.name
  anim.priority = 1000 - (i * 10 + 4)
  anim.exclusive = True
  anim.data = [tCard, anim.priority, bestMove.deckCell]
  def code(globalVars, formVars, target):
    #Ridisegno la carta:
    card = target.animation.data[0]
    card.status = "SELECTED"
    graph = formGraphics.get(card.data)
    graph.image = card.writeValues(psp2d.Image(card.buffImageP2))
    #Sposto la carta sul deck
    graph = formGraphics.get(target.animation.data[0].data)
    anim = animation()
    anim.name = "MOVECARD"
    toX = functions.fromCellToX(target.animation.data[2])
    toY = functions.fromCellToY(target.animation.data[2])
    anim.data = [toX, toY]
    anim.code = functions.moveFromTo
    anim.priority = target.animation.data[1]
    anim.exclusive = True
    anim.front = True
    graph.animation = anim
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  makeMove(bestMove, 1000 - (i * 10 + 5))

  #Torno in selezione per il player 1:
  graph = graphicElement()
  graph.name = "NEXTMOVE"
  anim = animation()
  anim.name = graph.name
  anim.priority = -100
  def code(globalVars, formVars, target):
    if len(formVars.get("deck").freeCells()) == 0:
      if formVars.get("gameType").type == "SINGLE":
        matchEnd()
      elif formVars.get("gameType").type == "MULTI":
        matchEndMulti()
    else:
      selectFirstCardP1()
      formVars.set("SELECTING", "status")
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def matchEnd():
  """End of the match"""
  tScore = score()

  graph = graphicElement()
  graph.name = "ENDMESSAGE"
  graph.priority = 100
  graph.font = globalVars.get("fonts").get("fontResult")

  sound = False
  postMessage = ""
  postFunction = None
  if tScore[0] == tScore[1]:
    #Controllo la regola sudden death:
    if formVars.get("gameType").suddenDeath:
      msg = globalVars.get("messages").SUDDENDEATH
      postMessage = "SUDDEN"
      postFunction = suddenDeath
    elif formVars.get("gameType").type == "MULTI":
      msg = globalVars.get("messages").DRAW
      postMessage = "TRADE"
      postFunction = tradeCards
      formVars.set("MULTI-ENDED", "status")
    else:
      formVars.set("ENDED", "status")
      msg = globalVars.get("messages").DRAW
  elif tScore[0] > tScore[1]:
    formVars.set("ENDED", "status")
    msg = globalVars.get("messages").WIN
    if formVars.get("gameType").type == "MULTI":
      postMessage = "TRADE"
      postFunction = tradeCards
      formVars.set("MULTI-ENDED", "status")
  elif tScore[1] > tScore[0]:
    formVars.set("ENDED", "status")
    msg = globalVars.get("messages").LOOSE
    if formVars.get("gameType").type == "MULTI":
      postMessage = "TRADE"
      postFunction = tradeCards
      formVars.set("MULTI-ENDED", "status")
  tWidth = functions.textWidth(graph.font, msg)
  tX = round((480 - tWidth) / 2)

  #Messaggio di fine partita:
  graph.posX = tX
  graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
  graph.text = msg
  anim = animation()
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.data = [tScore[0], tScore[1], sound, postMessage, None, postFunction]
  anim.code = functions.endMessage
  graph.animation = anim
  formGraphics.set(graph)


def matchEndMulti():
  """End of a multiplayer match"""
  #Comunico al server il termine della partita:
  waitMessage(globalVars, formVars, globalVars.get("messages").GM_VALIDATING)
  globalVars.get("net").matchEnd(formVars.get("gameType").roomID, formVars.get("deck").stringForServer(formVars.get("gameType").player1_ID, formVars.get("gameType").player2_ID))
  #Timer per aspettare la validazione del risultato:
  graph = graphicElement()
  graph.name = "MATCHVALIDATION"
  graph.data = globalVars.get("currentFPSTime")
  anim = animation()
  def code(globalVars, formVars, target):
    if not target.data \
    or globalVars.get("currentFPSTime") - target.data >= 2:
      if target.visible == True:
        globalVars.get("net").waitMatchValidation(formVars.get("gameType").roomID, matchEndMultiCallback)
        target.visible = False
      target.data = globalVars.get("currentFPSTime")
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def matchEndMultiCallback(pResponse, pError):
  """End of match, server response"""
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      formGraphics.get("MATCHVALIDATION").visible = True
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) > 0:
    if response[0] == "OK":
      hideWaitMessage(globalVars, formVars)
      formGraphics.get("MATCHVALIDATION").name = "DEL_" + formGraphics.get("MATCHVALIDATION").name
      matchEnd()
    elif response[0] == "NO":
      hideWaitMessage(globalVars, formVars)
      formGraphics.get("MATCHVALIDATION").name = "DEL_" + formGraphics.get("MATCHVALIDATION").name
      waitMessage(globalVars, formVars, globalVars.get("messages").GM_VALIDATIONERROR, False, 5)
  else:
    formGraphics.get("MATCHVALIDATION").visible = True


def getDecksAndRulesCallback(pResponse, pError):
  #Chiedo i mazzi dei giocatori e le regole:
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      globalVars.get("net").getDecksAndRules(formVars.get("gameType").roomID, getDecksAndRulesCallback)
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) >= 3:
    #Regole ed elementi:
    formVars.get("gameType").elemental = False
    formVars.get("gameType").plus = False
    formVars.get("gameType").same = False
    formVars.get("gameType").sameWall = False
    formVars.get("gameType").combo = False
    formVars.get("gameType").open = False
    formVars.get("gameType").suddenDeath = False
    ele = response[2].split("||")
    for i, e in enumerate(ele):
      #Elementi:
      if "//n//" in e:
        if formVars.get("gameType").elemental:
          subEle = e.split("//n//")
          el = globalVars.get("elements").index(subEle[0])
          elementImage = eval("globalVars.get(\"images\").get(\"element" + subEle[0].title() + "Image\")")
          pos = int(subEle[1])
          formVars.get("deck").elements[pos] = el
          formVars.get("bkgImage").blit(elementImage, 0, 0, -1, -1, functions.fromCellToX(pos) + 38, functions.fromCellToY(pos) + 38, True)
      #Regole:
      elif e == "SAME":
        formVars.get("gameType").same = True
      elif e == "SAMEWALL":
        formVars.get("gameType").sameWall = True
      elif e == "PLUS":
        formVars.get("gameType").plus = True
      elif e == "COMBO":
        formVars.get("gameType").combo = True
      elif e == "ELEM":
        formVars.get("gameType").elemental = True
      elif e == "OPEN":
        formVars.get("gameType").open = True
      elif e == "SUDDEN":
        formVars.get("gameType").suddenDeath = True
      elif e[:6] == "TRADE_":
        formVars.get("gameType").tradeRule = e
      else:
        globalVars.get("logFile").log("ERROR: Unknown rule " + e + " from server.")

    #Carte primo giocatore:
    ele = response[0].split("||")
    formVars.get("gameType").initialDeck_Player1 = []
    formVars.get("gameType").initialDeck_Player2 = []
    for i, e in enumerate(ele):
      formVars.get("gameType").initialDeck_Player1.append(int(e))
      tCard = globalVars.get("shopCards")[globalVars.get("shopCardsIndex").index(int(e))].copy()
      tCard.owner = "P1"
      tCard.data = str(i)
      tCard.deckCell = None
      tCard.buffImageP1 = psp2d.Image(tCard.imageP1)
      tCard.buffImageP2 = psp2d.Image(tCard.imageP2)
      formVars.get("cardsP1").elements.append(tCard)
      #Creo l'oggetto grafico:
      graph = graphicElement()
      graph.name = str(i)
      graph.image = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      graph.posX = 10
      graph.posY = -100
      graph.priority = i
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      anim.data = [graph.posX, 3 + 43 * i]
      anim.priority = 99 - i
      anim.code = functions.moveFromTo
      graph.animation = anim
      formGraphics.set(graph)

    #Carte secondo giocatore:
    ele = response[1].split("||")
    for i, e in enumerate(ele):
      formVars.get("gameType").initialDeck_Player2.append(int(e))
      tCard = globalVars.get("shopCards")[globalVars.get("shopCardsIndex").index(int(e))].copy()
      tCard.owner = "P2"
      tCard.data = str(100 + i)
      tCard.deckCell = None
      tCard.buffImageP1 = psp2d.Image(tCard.imageP1)
      tCard.buffImageP2 = psp2d.Image(tCard.imageP2)
      formVars.get("cardsP2").elements.append(tCard)
      #Creo l'oggetto grafico:
      graph = graphicElement()
      graph.name = str(100 + i)
      if formVars.get("gameType").open:
        graph.image = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
      else:
        graph.image = tCard.imageBack
      graph.posX = 396
      graph.posY = -100
      graph.priority = i
      anim = animation()
      anim.name = "MOVECARD"
      anim.data = [graph.posX, 3 + 43 * i]
      if i == 4:
        anim.exclusive = True
      anim.priority = 89 - i
      anim.code = functions.moveFromTo
      graph.animation = anim
      formGraphics.set(graph)

    #Messaggio CHALLENGE:
    formVars.set("SINGLE", "status")
    graph = graphicElement()
    graph.name = "CHALLENGE"
    graph.font = globalVars.get("fonts").get("fontResult")
    msg = globalVars.get("messages").CHALLENGE
    tWidth = functions.textWidth(graph.font, msg)
    tX = round((480 - tWidth) / 2)
    graph.posX = tX
    graph.posY = 272 / 2 - functions.textHeight(graph.font, "I") / 2
    graph.text = msg
    anim = animation()
    anim.name = graph.name
    anim.priority = -10
    anim.exclusive = True
    anim.front = True
    anim.code = functions.message
    graph.animation = anim
    formGraphics.set(graph)
    postInit()
  else:
    waitMessage(globalVars, formVars, globalVars.get("messages").GM_DECKSERROR, False, 5)
  #Mi metto in attesa (per capire chi deve iniziare):
  waitOpponentsMove()


def waitOpponentsMove():
  #Aspetto la risposta dell'altro giocatore (MULTI):
  if formVars.get("gameType").lastTurn == -1:
    waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAIT, False)
  else:
    waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAITOPPONENTS, False)
    #Timer per uscire dopo xx secondi:
    def showFunction():
      hideAllMenues()
    def hideFunction():
      formVars.set("WAITINGOPPONENT", "status")
    addTimeoutRestore(globalVars, formVars, globalVars.get("userData").TIMEOUT_MOVE, globalVars.get("messages").GM_WAITOPPONENTS, showFunction, hideFunction)

  formVars.set("WAITINGOPPONENT", "status")
  #Timer per richiedere la mossa:
  graph = graphicElement()
  graph.name = "OPPONENTSMOVE"
  anim = animation()
  def code(globalVars, formVars, target):
    if not target.data \
    or globalVars.get("currentFPSTime") - target.data >= 2:
      if target.visible == True:
        globalVars.get("net").waitOpponentsMove(formVars.get("gameType").roomID, formVars.get("gameType").lastTurn + 1, opponentsMoveCallback)
        target.visible = False
      target.data = globalVars.get("currentFPSTime")
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def opponentsMoveCallback(pResponse, pError):
  """Opponent's move"""
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      formGraphics.get("OPPONENTSMOVE").visible = True
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) > 0:
    if response[0] == "OK":
      delGraphic(formGraphics, "WAITTIMEOUT")
      if formVars.get("formMenu") and formVars.get("formMenu").name  == "CONFIRMABORT":
        formVars.get("formMenu").delGraphics()
        formVars.set(None, "formMenu")
        delGraphic(formGraphics, "CONFIRM")
      formGraphics.get("OPPONENTSMOVE").name = "DEL_" + formGraphics.get("OPPONENTSMOVE").name
      hideWaitMessage(globalVars, formVars)
      #Controllo se e' prima mossa:
      if len(response[1]) == 0:
        formVars.set("P1", "firstTurn")
        postInit()
      else:
        formVars.get("gameType").lastTurn += 1
        #Cerco la carta:
        tCard = None
        for card in formVars.get("cardsP2").elements:
          if int(response[1]) == card.ID and card.deckCell == None:
            tCard = card
            break
        if not tCard:
          waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAITOPPONENTSERROR, False, 10)
          return
        tCard.deckCell = int(response[2])

        #Calcolo la mossa:
        tMove = evaluateMove(tCard, tCard.deckCell)
        tCard.delta = tMove.cardDelta

        #Sposto la carta:
        globalVars.set(formVars.get("cardSound"), "playingSound")
        globalVars.get("playingSound").start()

        graph = graphicElement()
        graph.name = "MOVECARD_DECK_1"
        anim = animation()
        anim.name = graph.name
        anim.priority = 1000
        anim.exclusive = True
        anim.data = [tCard, anim.priority, tMove.deckCell]
        def code(globalVars, formVars, target):
          #Ridisegno la carta:
          card = target.animation.data[0]
          card.status = "SELECTED"
          graph = formGraphics.get(card.data)
          graph.image = card.writeValues(psp2d.Image(card.buffImageP2))
          #Sposto la carta sul deck
          graph.data = graph.priority
          graph.priority = 99
          anim = animation()
          anim.name = "MOVECARD"
          toX = functions.fromCellToX(target.animation.data[2])
          toY = functions.fromCellToY(target.animation.data[2])
          anim.data = [toX, toY]
          anim.code = functions.moveFromTo
          anim.priority = target.animation.data[1]
          anim.exclusive = True
          graph.animation = anim
          target.name = "DEL_" + target.name
        anim.code = code
        graph.animation = anim
        formGraphics.set(graph)

        #Riporto priority al suo valore:
        graph = graphicElement()
        graph.name = "MOVECARD_DECK_2"
        anim = animation()
        anim.name = graph.name
        anim.priority = 1000
        anim.exclusive = True
        anim.data = [tCard]
        def code(globalVars, formVars, target):
          #Ridisegno la carta:
          card = target.animation.data[0]
          graph = formGraphics.get(card.data)
          graph.priority = graph.data
          graph.data = None
          target.name = "DEL_" + target.name
        anim.code = code
        graph.animation = anim
        formGraphics.set(graph)

        #Faccio la mossa:
        formVars.get("deck").cells[tMove.deckCell] = tCard
        makeMove(tMove, 1000)

        #Torno in selezione per il player 1:
        if len(formVars.get("deck").freeCells()) == 0:
          matchEndMulti()
        else:
          selectFirstCardP1()
          if formVars.get("status") in ["MENU", "VIEWRULES"] or formVars.get("formKeyboard"):
            formVars.get("data")[0] = "SELECTING"
          else:
            formVars.set("SELECTING", "status")
    else:
      formGraphics.get("OPPONENTSMOVE").visible = True
  else:
    formGraphics.get("OPPONENTSMOVE").name = "DEL_" + formGraphics.get("OPPONENTSMOVE").name
    waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAITOPPONENTSERROR, False, 10)


def tradeCards():
  """Cards trading:"""
  formVars.set("TRADING", "status")
  #Sposto le carte:
  tScore = score()
  formVars.get("gameType").score = tScore
  if tScore[0] > tScore[1]:
    formVars.set(formVars.get("gameType").initialDeck_Player2, "tradingCards")
    formVars.get("gameType").winner = "P1"
    formVars.get("gameType").winnerCards = formVars.get("cardsP1").elements[:]
    formVars.get("gameType").loserCards = formVars.get("cardsP2").elements[:]
  elif tScore[0] < tScore[1]:
    formVars.set(formVars.get("gameType").initialDeck_Player1, "tradingCards")
    formVars.get("gameType").winner = "P2"
    formVars.get("gameType").winnerCards = formVars.get("cardsP2").elements[:]
    formVars.get("gameType").loserCards = formVars.get("cardsP1").elements[:]
  elif tScore[0] == tScore[1]:
    formVars.set(formVars.get("gameType").initialDeck_Player1, "tradingCards")
    formVars.get("gameType").winner = "PX"
    formVars.get("gameType").winnerCards = formVars.get("cardsP1").elements[:]
    formVars.get("gameType").loserCards = formVars.get("cardsP2").elements[:]

  #Controllo la regola di trading:
  if formVars.get("gameType").winner == "PX":
    formVars.set(0, "toBeTraded")
    toLaunch = tradeDraw
  elif formVars.get("gameType").tradeRule == "TRADE_ONE":
    formVars.set(1, "toBeTraded")
    toLaunch = tradeOneDiff
  elif formVars.get("gameType").tradeRule == "TRADE_DIFF":
    if formVars.get("gameType").winner == "P1":
      tToBeTraded = tScore[0] - 5
    else:
      tToBeTraded = tScore[1] - 5
    formVars.set(tToBeTraded, "toBeTraded")
    toLaunch = tradeOneDiff
  elif formVars.get("gameType").tradeRule == "TRADE_ALL":
    formVars.set(5, "toBeTraded")
    toLaunch = tradeAll
  elif formVars.get("gameType").tradeRule == "TRADE_DIRECT":
    if formVars.get("gameType").winner == "P1":
      formVars.set(tScore[0], "toBeTraded")
    elif formVars.get("gameType").winner == "P2":
      formVars.set(tScore[1], "toBeTraded")
    toLaunch = tradeDirect
  else:
    formVars.get("gameType").winner = "PX"
    toLaunch = noTrade

  #Cancello il risultato:
  delGraphic(formGraphics, "SCORE_P1")
  delGraphic(formGraphics, "SCORE_P2")
  formVars.remove("scoreImages")
  toLaunch()


def updateTradingInfo():
  """Update info during trading cards"""
  sel = formGraphics.get("TRADE_SELECTOR")
  graph = formGraphics.get("TRADING_BG")
  graph.image = None
  graph.image = psp2d.Image(graph.bufferImage)
  posX = 10
  posY = 10#120
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenuSel"), "I")

  #Informazioni regola:
  if formVars.get("gameType").tradeRule == "TRADE_ONE":
    globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_TRADE_ONE_DSC)
  elif formVars.get("gameType").tradeRule == "TRADE_DIFF":
    globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_TRADE_DIFF_DSC.replace("xx", str(formVars.get("toBeTraded"))))

  #Informazioni carta selezionata:
  selected = formVars.get("tradingCards")[sel.data - 1]
  for tCard in formVars.get("cardsP2").elements:
    if tCard.ID == selected:
      globalVars.set(tCard.stringValue(), "message")
      #Quante ne ho gia'?
      if formVars.get("gameType").type == "MULTI":
        if tCard.ID in globalVars.get("playerCardsIndex"):
          pCard = globalVars.get("playerCards")[globalVars.get("playerCardsIndex").index(tCard.ID)]
          globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 1 * tHeight, globalVars.get("messages").GM_YOUHAVE + " " + str(pCard.count) + " " + globalVars.get("messages").GM_THISCARD)
        else:
          globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 1 * tHeight, globalVars.get("messages").GM_YOUDONTHAVE)
      break
  #Se ho finito di scegliere:
  if len(formVars.get("choosenCards")) == formVars.get("toBeTraded"):
    graph.image.blit(globalVars.get("images").get("buttonStart"), 0, 0, globalVars.get("images").get("buttonStart").width, globalVars.get("images").get("buttonStart").height, posX, posY + 2 * tHeight, True)
    globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX + globalVars.get("images").get("buttonStart").width, posY + 2 * tHeight, globalVars.get("messages").GM_CONFIRMTRADE)


def noTrade():
  """No trade"""
  initialDeck_Player1 = formVars.get("gameType").initialDeck_Player1[:]
  #Rotazione carte:
  for i, tCard in enumerate(formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards):
    graph = formGraphics.get(tCard.data)
    if graph:
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      anim.code = functions.changeOwner

      if tCard.ID in initialDeck_Player1:
        initialDeck_Player1.remove(tCard.ID)
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      else:
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP2))

    #L'ultima e' esclusiva:
    if i == 9:
      anim.priority = 50
      anim.exclusive = True
    graph.animation = anim

  #Pausa:
  graph = graphicElement()
  graph.name = "WAIT"
  anim = animation()
  anim.name = "WAIT"
  anim.priority = -50
  anim.exclusive = True
  anim.lockKeyboard = True
  anim.code = functions.message
  graph.animation = anim
  formGraphics.set(graph)

  #Spostamento carte:
  graph = graphicElement()
  graph.name = "MOVE_CARDS"
  anim = animation()
  anim.name = "MOVE_CARDS"
  anim.priority = -100
  anim.exclusive = True
  def code(globalVars, formVars, target):
    #Animo le carte:
    initialDeck_Player1 = formVars.get("gameType").initialDeck_Player1[:]
    for i, tCard in enumerate(formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards):
      #Faccio uscire la carta:
      graph = formGraphics.get(tCard.data)
      graph.priority = 10
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      if tCard.ID in initialDeck_Player1:
        initialDeck_Player1.remove(tCard.ID)
        anim.data = [-74, 70, 99]
      else:
        anim.data = [480, 70, 99]
      anim.priority = 90
      anim.code = functions.moveFromToAndDelete
      graph.animation = anim
      #L'ultima e' esclusiva:
      if i == 9:
        anim.priority = -100
        anim.exclusive = True
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Faccio il confirm trade:
  if formVars.get("gameType").winner == "P1":
    waitMessage(globalVars, formVars, globalVars.get("messages").GM_VALIDATINGTRADE)
    globalVars.get("net").confirmTrade(formVars.get("gameType").roomID, "", confirmNoTradeCallback)
  else:
    formVars.get("gameType").winner = "PX"
    #Mi metto in attesa di trading:
    waitOpponentsTraiding()


def confirmNoTradeCallback(pResponse, pError):
  """Confirm trade for no trade match"""
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      globalVars.get("net").confirmTrade(formVars.get("gameType").roomID, "", confirmNoTradeCallback)
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) > 0:
    if response[0] == "OK":
      hideWaitMessage(globalVars, formVars)
      formVars.get("gameType").winner = "PX"
      matchResult(True, int(response[1]), int(response[2]))
    elif response[0] == "NO":
      waitMessage(globalVars, formVars, globalVars.get("messages").GM_TRADINGERROR, False, 5)


def tradeDraw():
  """Draw in multiplayer"""
  #Rotazione carte:
  initialDeck_Player1 = formVars.get("gameType").initialDeck_Player1[:]
  for i, tCard in enumerate(formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards):
    graph = formGraphics.get(tCard.data)
    if graph:
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      anim.code = functions.changeOwner

      if tCard.ID in initialDeck_Player1:
        initialDeck_Player1.remove(tCard.ID)
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      else:
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP2))

    #L'ultima e' esclusiva:
    if i == 9:
      anim.priority = 50
      anim.exclusive = True
    graph.animation = anim

  #Pausa:
  graph = graphicElement()
  graph.name = "WAIT"
  anim = animation()
  anim.name = "WAIT"
  anim.priority = -50
  anim.exclusive = True
  anim.lockKeyboard = True
  anim.code = functions.message
  graph.animation = anim
  formGraphics.set(graph)

  #Spostamento carte:
  graph = graphicElement()
  graph.name = "MOVE_CARDS"
  anim = animation()
  anim.name = "MOVE_CARDS"
  anim.priority = -100
  anim.exclusive = True
  def code(globalVars, formVars, target):
    #Animo le carte:
    initialDeck_Player1 = formVars.get("gameType").initialDeck_Player1[:]
    for i, tCard in enumerate(formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards):
      #Faccio uscire la carta:
      graph = formGraphics.get(tCard.data)
      graph.priority = 10
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      if tCard.ID in initialDeck_Player1:
        initialDeck_Player1.remove(tCard.ID)
        anim.data = [-74, 70, 99]
      else:
        anim.data = [480, 70, 99]
      anim.priority = 90
      anim.code = functions.moveFromToAndDelete
      graph.animation = anim
      #L'ultima e' esclusiva:
      if i == 9:
        anim.priority = -100
        anim.exclusive = True
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Lancio il matchResult:
  graph = graphicElement()
  graph.name = "LAUNCH_MATCHRESULT"
  anim = animation()
  anim.name = graph.name
  anim.priority = -110
  def code(globalVars, formVars, target):
    matchResult(False, 0, 0)
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def tradeOneDiff():
  """Trade-one and trade-diff"""
  #Rotazione carte:
  for i, tCard in enumerate(formVars.get("gameType").loserCards):
    graph = formGraphics.get(tCard.data)
    if graph:
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      anim.code = functions.changeOwner

      if formVars.get("gameType").winner == "P1":
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      else:
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
    #L'ultima e' esclusiva:
    if i == 4:
      anim.priority = 50
      anim.exclusive = True
    graph.animation = anim

  for i, tCard in enumerate(formVars.get("gameType").winnerCards):
    graph = formGraphics.get(tCard.data)
    if graph:
      #Giro la carta sul retro:
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      anim.code = functions.changeOwner
      anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, None, None]
      graph.animation = anim

  #Pausa:
  graph = graphicElement()
  graph.name = "WAIT"
  anim = animation()
  anim.name = "WAIT"
  anim.priority = -50
  anim.exclusive = True
  anim.lockKeyboard = True
  anim.code = functions.message
  graph.animation = anim
  formGraphics.set(graph)

  #Riquadro:
  graph = graphicElement()
  graph.name = "TRADING_BG"
  graph.data = [400, 200, globalVars.get("themeOptions").MENU_OUTERCOLOR, globalVars.get("themeOptions").MENU_INNERCOLOR, globalVars.get("themeOptions").MENU_CORNERCOLOR, globalVars.get("themeOptions").MENU_BORDERWIDTH, globalVars.get("themeOptions").MENU_ALPHA]
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_trading.png')
  posX = 10
  posY = 10
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenuSel"), "I")
  globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_TRADE_WAIT)
  if formVars.get("gameType").winner == "P1":
    graph.bufferImage = psp2d.Image(graph.image)
  graph.posX = 45
  graph.posY = -200
  anim = animation()
  anim.name = graph.name
  anim.priority = -60
  anim.data = [graph.posX, 150, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Spostamento carte:
  graph = graphicElement()
  graph.name = "MOVE_CARDS"
  anim = animation()
  anim.name = "MOVE_CARDS"
  anim.priority = -100
  anim.exclusive = True
  def code(globalVars, formVars, target):
    #Animo le carte:
    for i, tCard in enumerate(formVars.get("gameType").loserCards):
      #Posiziono la carta
      graph = formGraphics.get(tCard.data)
      graph.priority = 10
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      anim.data = [55 + 74 * i, 70, 99]
      anim.priority = 90
      anim.code = functions.moveFromTo
      graph.animation = anim
      if i == 4:
        anim.priority = -100
        anim.exclusive = True

    for i, tCard in enumerate(formVars.get("gameType").winnerCards):
      #Faccio uscire la carta:
      graph = formGraphics.get(tCard.data)
      graph.priority = 0
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      anim.data = [-74, 70, 99]
      anim.priority = 80
      anim.code = functions.moveFromToAndDelete
      graph.animation = anim
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Selector:
  if formVars.get("gameType").winner == "P1":
    graph = graphicElement()
    graph.name = "TRADE_SELECTOR"
    graph.visible = False
    graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/selector_top.png')
    graph.data = 1
    graph.posX = 55 + 37 - graph.image.width / 2
    graph.posY = 70 - graph.image.height - 15
    graph.visible = False
    graph.priority = 10
    anim = animation()
    anim.name = graph.name
    anim.lockKeyboard = False
    anim.data = [+0.5,0]
    anim.code = functions.selectorTopAnim
    graph.animation = anim
    formGraphics.set(graph)

    graph = graphicElement()
    graph.name = "SHOW_TRADE_SELECTOR"
    anim = animation()
    anim.name = graph.name
    anim.priority = -200
    def code(globalVars, formVars, target):
      formGraphics.get("TRADE_SELECTOR").visible = True
      target.name = "DEL_" + target.name
    anim.code = code
    graph.animation = anim
    formGraphics.set(graph)
    updateTradingInfo()
  else:
    #Mi metto in attesa del trading:
    waitOpponentsTraiding()


def tradeDirect():
  """Trade direct"""
  #Riquadro:
  graph = graphicElement()
  graph.name = "TRADING_BG"
  graph.data = [400, 200, globalVars.get("themeOptions").MENU_OUTERCOLOR, globalVars.get("themeOptions").MENU_INNERCOLOR, globalVars.get("themeOptions").MENU_CORNERCOLOR, globalVars.get("themeOptions").MENU_BORDERWIDTH, globalVars.get("themeOptions").MENU_ALPHA]
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_trading.png')
  #graph.bufferImage = psp2d.Image(graph.image)
  graph.posX = 45
  graph.posY = -200
  anim = animation()
  anim.name = graph.name
  anim.priority = -60
  anim.data = [graph.posX, 25, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Spostamento carte:
  graph = graphicElement()
  graph.name = "MOVE_CARDS"
  anim = animation()
  anim.name = "MOVE_CARDS"
  anim.priority = -100
  anim.exclusive = True
  def code(globalVars, formVars, target):
    #Animo le carte:
    tradingCards = [(tCard.owner, tCard) for tCard in formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards]
    tradingCards.sort()
    toY = 70
    toX = 0
    for i, tCard in enumerate(tradingCards):
      if i == 5:
        toY = 146
        toX = 0
      tCard = tCard[1]

      #Posiziono la carta
      graph = formGraphics.get(tCard.data)
      graph.priority = 10
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      anim.data = [55 + 74 * toX, toY, 99]
      anim.priority = 90
      anim.code = functions.moveFromTo
      graph.animation = anim
      if i == 9:
        anim.priority = -100
        anim.exclusive = True
      toX += 1
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  #Mi metto in attesa del trading:
  waitOpponentsTraiding()


def tradeAll():
  """Trade all"""
  #Rotazione carte:
  for i, tCard in enumerate(formVars.get("gameType").loserCards):
    graph = formGraphics.get(tCard.data)
    if graph:
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      anim.code = functions.changeOwner

      if formVars.get("gameType").winner == "P1":
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      else:
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
    graph.animation = anim
  for i, tCard in enumerate(formVars.get("gameType").winnerCards):
    graph = formGraphics.get(tCard.data)
    if graph:
      anim = animation()
      anim.name = "CHANGEOWNER"
      anim.lockKeyboard = True
      anim.priority = 100
      anim.code = functions.changeOwner

      if formVars.get("gameType").winner == "P2":
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP1)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
      else:
        anim.data = [tCard.writeValues(psp2d.Image(tCard.buffImageP2)), tCard.imageBack, tCard.imageBack, None]
        tCard.delta = 0
        anim.data[3] = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
    #L'ultima e' esclusiva:
    if i == 4:
      anim.priority = 50
      anim.exclusive = True
    graph.animation = anim

  #Pausa:
  graph = graphicElement()
  graph.name = "WAIT"
  anim = animation()
  anim.name = "WAIT"
  anim.priority = -50
  anim.exclusive = True
  anim.lockKeyboard = True
  anim.code = functions.message
  graph.animation = anim
  formGraphics.set(graph)

  #Riquadro:
  graph = graphicElement()
  graph.name = "TRADING_BG"
  graph.data = [400, 200, globalVars.get("themeOptions").MENU_OUTERCOLOR, globalVars.get("themeOptions").MENU_INNERCOLOR, globalVars.get("themeOptions").MENU_CORNERCOLOR, globalVars.get("themeOptions").MENU_BORDERWIDTH, globalVars.get("themeOptions").MENU_ALPHA]
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_trading.png')
  #graph.bufferImage = psp2d.Image(graph.image)
  graph.posX = 45
  graph.posY = -200
  anim = animation()
  anim.name = graph.name
  anim.priority = -60
  anim.data = [graph.posX, 25, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Spostamento carte:
  graph = graphicElement()
  graph.name = "MOVE_CARDS"
  anim = animation()
  anim.name = "MOVE_CARDS"
  anim.priority = -100
  anim.exclusive = True
  def code(globalVars, formVars, target):
    #Animo le carte:
    toY = 70
    toX = 0
    for i, tCard in enumerate(formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards):
      if i == 5:
        toY = 146
        toX = 0

      #Posiziono la carta
      graph = formGraphics.get(tCard.data)
      graph.priority = 10
      anim = animation()
      anim.name = "MOVECARD_" + str(tCard.ID)
      anim.data = [55 + 74 * toX, toY, 99]
      anim.priority = 90
      anim.code = functions.moveFromTo
      graph.animation = anim
      if i == 9:
        anim.priority = -100
        anim.exclusive = True
      toX += 1
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)

  if formVars.get("gameType").winner == "P1":
    #Faccio il trading di tutte le carte dell'avversario:
    formVars.set([0,1,2,3,4], "choosenCards")
    confirmTrade()
  else:
    #Mi metto in attesa del trading:
    waitOpponentsTraiding()


def selectNextTradingCard():
  """Select the next trading card"""
  #Controllo la mia posizione:
  sel = formGraphics.get("TRADE_SELECTOR")
  if sel.data == 5:
    sel.data = 1
  else:
    sel.data += 1

  updateTradingInfo()
  #Sposto il selector:
  anim = animation()
  anim.name = sel.name
  anim.exclusive = True
  anim.data = [55 + 74 * (sel.data - 1) + 37 - sel.image.width / 2, 70 - sel.image.height - 15]
  anim.code = functions.moveFromTo
  sel.animation = anim

  #Riattivo l'animazione:
  graph = graphicElement()
  graph.name = "ANIMATE_TRADE_SELECTOR"
  anim = animation()
  anim.name = graph.name
  anim.priority = -200
  def code(globalVars, formVars, target):
    graph = formGraphics.get("TRADE_SELECTOR")
    anim = animation()
    anim.name = graph.name
    anim.lockKeyboard = False
    anim.data = [+0.5,0]
    anim.code = functions.selectorTopAnim
    graph.animation = anim
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def selectPreviousTradingCard():
  """Select the next trading card"""
  #Controllo la mia posizione:
  sel = formGraphics.get("TRADE_SELECTOR")
  if sel.data == 1:
    sel.data = 5
  else:
    sel.data -= 1

  updateTradingInfo()
  #Sposto il selector:
  anim = animation()
  anim.name = sel.name
  anim.exclusive = True
  anim.data = [55 + 74 * (sel.data - 1) + 37 - sel.image.width / 2, 70 - sel.image.height - 15]
  anim.code = functions.moveFromTo
  sel.animation = anim

  #Riattivo l'animazione:
  graph = graphicElement()
  graph.name = "ANIMATE_TRADE_SELECTOR"
  anim = animation()
  anim.name = graph.name
  anim.priority = -200
  def code(globalVars, formVars, target):
    graph = formGraphics.get("TRADE_SELECTOR")
    anim = animation()
    anim.name = graph.name
    anim.lockKeyboard = False
    anim.data = [+0.5,0]
    anim.code = functions.selectorTopAnim
    graph.animation = anim
    target.name = "DEL_" + target.name
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def selectTradingCard():
  """Select a card for trading"""
  sel = formGraphics.get("TRADE_SELECTOR")
  card = formVars.get("cardsP2").elements[sel.data - 1]
  graph = formGraphics.get(card.data)
  anim = animation()
  anim.name = str(graph.name)
  anim.exclusive = True
  anim.data = [graph.posX, graph.posY - 15]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formVars.get("choosenCards").append(sel.data - 1)
  updateTradingInfo()


def deselectTradingCard():
  """Deselect a card for trading"""
  sel = formGraphics.get("TRADE_SELECTOR")
  card = formVars.get("cardsP2").elements[sel.data - 1]
  graph = formGraphics.get(card.data)
  anim = animation()
  anim.name = str(graph.name)
  anim.exclusive = True
  anim.data = [graph.posX, graph.posY + 15]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formVars.get("choosenCards").remove(sel.data - 1)
  updateTradingInfo()


def confirmTrade():
  """Confirm trading"""
  #Nascondo il selector:
  sel = formGraphics.get("TRADE_SELECTOR")
  if sel:
    sel.visible = False
  waitMessage(globalVars, formVars, globalVars.get("messages").GM_VALIDATINGTRADE)
  cardsID = ""
  for IDs in formVars.get("choosenCards"):
    card = formVars.get("cardsP2").elements[IDs]
    cardsID += str(card.ID) + "//%//"
  cardsID = cardsID[:-5]
  globalVars.get("net").confirmTrade(formVars.get("gameType").roomID, cardsID, confirmTradeCallback)


def confirmTradeCallback(pResponse, pError):
  """Confirm trading callback"""
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      globalVars.get("net").confirmTrade(formVars.get("gameType").roomID, cardsID, confirmTradeCallback)
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) > 0:
    if response[0] == "OK":
      hideWaitMessage(globalVars, formVars)
      globalVars.get("player").guil += int(response[1])
      #Animo le carte:
      for i, ID in enumerate(formVars.get("choosenCards")):
        card = formVars.get("cardsP2").elements[ID]
        #Aggiungo la carta:
        if card.ID in globalVars.get("playerCardsIndex"):
          tCard = globalVars.get("playerCards")[globalVars.get("playerCardsIndex").index(card.ID)]
          tCard.count += 1
        else:
          tCard = globalVars.get("shopCards")[globalVars.get("shopCardsIndex").index(card.ID)].copy()
          tCard.count = 1
          globalVars.get("playerCards").append(tCard)
          globalVars.get("playerCardsIndex").append(tCard.ID)

        graph = formGraphics.get(card.data)
        anim = animation()
        anim.name = "MOVECARD_" + str(ID)
        if i == len(formVars.get("choosenCards")) - 1:
          anim.exclusive = True
        anim.data = [graph.posX, -74, 99]
        anim.priority = 80
        anim.code = functions.moveFromTo
        graph.animation = anim

      #Faccio uscire le carte ed il riquadro:
      for i, tCard in enumerate(formVars.get("gameType").loserCards + formVars.get("gameType").winnerCards):
        #Posiziono la carta
        graph = formGraphics.get(tCard.data)
        if not graph.animation:
          anim = animation()
          anim.name = "MOVECARD_" + str(tCard.ID)
          anim.data = [-74, graph.posY, 99]
          anim.priority = 70
          anim.code = functions.moveFromToAndDelete
          graph.animation = anim

      graph = formGraphics.get("TRADING_BG")
      anim = animation()
      anim.name = "MOVE_TRADING_BG"
      anim.data = [-graph.image.width, graph.posY, 99]
      anim.code = functions.moveFromTo
      graph.animation = anim
      anim.priority = 60
      anim.exclusive = True

      graph = graphicElement()
      graph.name = "LAUNCH_MATCHRESULT"
      graph.data = [int(response[1]), int(response[2])]
      anim = animation()
      def code(globalVars, formVars, target):
        matchResult(True, target.data[0], target.data[1])
        target.name = "DEL_" + target.name
      anim.name = graph.name
      anim.priority = 55
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)

    elif response[0] == "NO":
      waitMessage(globalVars, formVars, globalVars.get("messages").GM_TRADINGERROR, False, 5)


def waitOpponentsTraiding():
  #Aspetto la risposta dell'altro giocatore (MULTI):
  formVars.set("WAITTRADING", "status")
  waitMessage(globalVars, formVars, globalVars.get("messages").GM_WAITOPPONENTS, False)

  #Timer per uscire dopo xx secondi:
  def showFunction():
    hideAllMenues()
  def hideFunction():
    formVars.set("WAITTRADING", "status")
  addTimeoutRestore(globalVars, formVars, globalVars.get("userData").TIMEOUT_TRADE, globalVars.get("messages").GM_WAITOPPONENTS, showFunction, hideFunction)

  #Timer per richiedere il trading:
  graph = graphicElement()
  graph.name = "OPPONENTSTRADE"
  anim = animation()
  def code(globalVars, formVars, target):
    if not target.data \
    or globalVars.get("currentFPSTime") - target.data >= 2:
      if target.visible == True:
        globalVars.get("net").waitOpponetsTrading(formVars.get("gameType").roomID, opponentsTradingCallback)
        target.visible = False
      target.data = globalVars.get("currentFPSTime")
  anim.name = graph.name
  anim.lockKeyboard = False
  anim.code = code
  graph.animation = anim
  formGraphics.set(graph)


def opponentsTradingCallback(pResponse, pError):
  """Wait for opponents trading"""
  if pError:
    if "TIMEOUT" in pError[1][1].upper():
      formGraphics.get("OPPONENTSTRADE").visible = True
    else:
      waitMessage(globalVars, formVars, pError[1][1], False, 10)
    return

  response = functions.analyzeServerString(pResponse)
  if len(response) > 0:
    if response[0] == "OK":
      delGraphic(formGraphics, "WAITTIMEOUT")
      if formVars.get("formMenu") and formVars.get("formMenu").name  == "CONFIRMABORT":
        formVars.get("formMenu").delGraphics()
        formVars.set(None, "formMenu")
        delGraphic(formGraphics, "CONFIRM")
      tGraph = formGraphics.get("OPPONENTSTRADE")
      if tGraph:
        tGraph.name = "DEL_" + tGraph.name
      hideWaitMessage(globalVars, formVars)

      globalVars.get("player").guil += int(response[2])
      #Animo le carte:
      if formVars.get("gameType").winner != "PX":
        choosenCards = response[1].split("||")
        formVars.set([], "choosenCards")
        for i, ID in enumerate(choosenCards):
          ID = ID.strip()
          if ID:
            for idxCard, card in enumerate(formVars.get("cardsP1").elements):
              if int(ID) == card.ID:
                formVars.get("choosenCards").append(card.ID)
                graph = formGraphics.get(card.data)
                #Rimuovo dal deck:
                for j, deckCard in enumerate(globalVars.get("player").deck[:]):
                  if deckCard and deckCard.ID == card.ID:
                    globalVars.get("player").deck.pop(j)
                    globalVars.get("player").deck.append(None)
                    break
                anim = animation()
                anim.name = "MOVECARD_" + str(ID)
                if i == len(choosenCards) - 1:
                  anim.exclusive = True
                anim.data = [graph.posX, -74, 99]
                anim.priority = 80
                anim.code = functions.moveFromTo
                graph.animation = anim
                break

        #Faccio uscire le carte ed il riquadro:
        for i, tCard in enumerate(formVars.get("gameType").loserCards):
          #Posiziono la carta
          if tCard.ID not in formVars.get("choosenCards"):
            graph = formGraphics.get(tCard.data)
            anim = animation()
            anim.name = "MOVECARD_" + str(tCard.ID)
            anim.data = [-74, graph.posY, 99]
            anim.priority = 70
            anim.code = functions.moveFromToAndDelete
            graph.animation = anim

        graph = formGraphics.get("TRADING_BG")
        anim = animation()
        anim.name = "MOVE_TRADING_BG"
        anim.data = [-graph.image.width, graph.posY, 99]
        anim.code = functions.moveFromToAndDelete
        graph.animation = anim
        anim.priority = 60
        anim.exclusive = True

      graph = graphicElement()
      graph.name = "LAUNCH_MATCHRESULT"
      graph.data = abs(int(response[2]))
      anim = animation()
      def code(globalVars, formVars, target):
        matchResult(False, target.data)
        target.name = "DEL_" + target.name
      anim.name = graph.name
      anim.priority = 55
      anim.code = code
      graph.animation = anim
      formGraphics.set(graph)
  else:
    formGraphics.get("OPPONENTSTRADE").visible = True


def matchResult(pWin=True, pGuil=0, pEXP=0):
  #Riquadro esito match:
  graph = graphicElement()
  graph.name = "MATCH_RESULT"
  graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_result.png')
  graph.posX = 140
  graph.posY = -graph.image.height
  anim = animation()
  anim.name = graph.name
  anim.priority = 50
  if formVars.get("gameType").winner != "PX":
    anim.data = [graph.posX, 10, 99]
  else:
    anim.data = [graph.posX, 100, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  posX = 10
  posY = 5
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenuSel"), "I")

  if pWin:
    globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_YOUWONGUIL.replace("xxx", str(pGuil)))
    globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX, posY + 1 * tHeight, globalVars.get("messages").GM_YOUWONEXP.replace("xxx", str(pEXP)))
    globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 3 * tHeight, globalVars.get("messages").GM_PRESSANYTOEXIT)
  else:
    globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX, posY + 0 * tHeight, globalVars.get("messages").GM_YOULOSTGUIL.replace("xxx", str(pGuil)))
    globalVars.get("fonts").get("fontMenu").drawText(graph.image, posX, posY + 1 * tHeight, globalVars.get("messages").GM_YOUNOEXP)
    globalVars.get("fonts").get("fontMenuSel").drawText(graph.image, posX, posY + 3 * tHeight, globalVars.get("messages").GM_PRESSANYTOEXIT)

  #Carte vinte/perse:
  if formVars.get("gameType").winner != "PX":
    graph = graphicElement()
    graph.name = "MATCH_RESULT_CARDSWON"
    graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_bg_endcards.png')
    graph.posX = -graph.image.width
    graph.posY = 82
    anim = animation()
    anim.name = graph.name
    anim.priority = 50
    anim.data = [50, graph.posY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formGraphics.set(graph)

    graph = graphicElement()
    graph.name = "MATCH_RESULT_CARDSLOST"
    graph.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_bg_endcards.png')
    graph.posX = 480
    graph.posY = 162
    anim = animation()
    anim.name = graph.name
    anim.priority = 49
    anim.exclusive = True
    anim.data = [50, graph.posY, 99]
    anim.code = functions.moveFromTo
    graph.animation = anim
    formGraphics.set(graph)

    initCards = formVars.get("cardsP1").elements[:]
    endCards = initCards[:]
    for idCard in formVars.get("choosenCards"):
      for tCard in initCards:
        if tCard.ID == idCard:
          if formVars.get("gameType").winner == "P2":
            endCards.remove(tCard)
          elif formVars.get("gameType").winner == "P1":
            endCards.append(tCard)
    #Carte vinte:
    wonCards = 0
    for tCard in endCards:
      if tCard not in initCards:
        graph = formGraphics.get(tCard.data)
        graph.posX = -graph.image.width
        graph.posY = 95
        graph.image = tCard.writeValues(psp2d.Image(tCard.buffImageP1))
        anim = animation()
        anim.name = graph.name
        anim.priority = 45
        anim.data = [52 + wonCards * 74, 84, 99]
        anim.code = functions.moveFromTo
        graph.animation = anim
        wonCards += 1
      else:
        initCards.remove(tCard)
    #Carte perse:
    lostCards = 0
    for tCard in initCards:
      if tCard not in endCards:
        graph = formGraphics.get(tCard.data)
        graph.posX = -graph.image.width
        graph.posY = 185
        graph.image = tCard.writeValues(psp2d.Image(tCard.buffImageP2))
        anim = animation()
        anim.name = graph.name
        anim.priority = 45
        anim.data = [52 + lostCards * 74, 164, 99]
        anim.code = functions.moveFromTo
        graph.animation = anim
        lostCards += 1
      else:
        endCards.remove(tCard)
  formVars.set("WAITEXIT", "status")


def showMatchRules():
  """Show match rules"""
  #Nascondo il menu:
  formVars.get("formMenu").delGraphics()
  delGraphic(formGraphics, "TITLE")
  delGraphic(formGraphics, "TITLET")

  gInfo = graphicElement()
  gInfo.name = "RULESINFO"
  gInfo.priority = 200
  gInfo.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_rules_info.png')
  gInfo.posX = (480 - gInfo.image.width) / 2
  gInfo.posY = 272
  gInfo.data = [formVars.get("status"), globalVars.get("message")]
  anim = animation()
  anim.name = gInfo.name
  anim.data = [gInfo.posX, 32]#(272 - gInfo.image.height) / 2]
  globalVars.set(None, "message")
  anim.code = functions.moveFromTo
  gInfo.animation = anim
  formGraphics.set(gInfo)

  gTitle = graphicElement()
  gTitle.name = "RULESINFOTITLE"
  gTitle.priority = 200
  gTitle.image = psp2d.Image('themes/' + globalVars.get("userData").THEME + '/formsbg/gm_rules_title.png')
  gTitle.posX = -gTitle.image.width
  gTitle.posY = 10#(272 - gInfo.image.height) / 2 - gTitle.image.height - 2
  anim = animation()
  anim.name = gTitle.name
  anim.data = [(480 - gTitle.image.width) / 2, gTitle.posY]
  anim.code = functions.moveFromTo
  gTitle.animation = anim
  formGraphics.set(gTitle)

  graph = graphicElement()
  graph.name = "RULESINFOTITLET"
  graph.priority = 201
  graph.font = globalVars.get("fonts").get("fontMenuSel")
  graph.text = globalVars.get("messages").RULESINFO
  graph.posX = -functions.textWidth(graph.font, graph.text)
  graph.posY = 13#(272 - gInfo.image.height) / 2 - gTitle.image.height - 2 + 3
  anim = animation()
  anim.name = graph.name
  anim.priority = 100
  anim.data = [(480 - functions.textWidth(graph.font, graph.text)) / 2, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Regole:
  rules = []
  gType = formVars.get("gameType")
  if gType.elemental:
    rules.append(globalVars.get("messages").GM_ELEM)
  if gType.plus:
    rules.append(globalVars.get("messages").GM_PLUS)
  if gType.same:
    rules.append(globalVars.get("messages").GM_SAME)
  if gType.sameWall:
    rules.append(globalVars.get("messages").GM_SAMEWALL)
  if gType.combo:
    rules.append(globalVars.get("messages").GM_COMBO)
  if gType.open:
    rules.append(globalVars.get("messages").GM_OPEN)
  if gType.suddenDeath:
    rules.append(globalVars.get("messages").GM_SUDDEN)

  #Trade rule:
  if gType.tradeRule:
    rules.append(eval("globalVars.get(\"messages\").GM_" + gType.tradeRule))

  #Scrivo le regole:
  posX = 5
  posY = 5
  tHeight = functions.textHeight(globalVars.get("fonts").get("fontMenu"), "I")
  for i, riga in enumerate(rules):
    if len(riga):
      if i < 4:
        globalVars.get("fonts").get("fontMenu").drawText(gInfo.image, posX, posY + i * tHeight, riga)
      else:
        globalVars.get("fonts").get("fontMenu").drawText(gInfo.image, gInfo.image.width - posX - functions.textWidth(globalVars.get("fonts").get("fontMenu"), riga), posY + (i - 4) * tHeight, riga)

  #Info player 1:
  graph = drawPlayerInfo(globalVars, formVars.get("Player_1_info"), "gm_bg_blplayerinfo.png")
  graph.name = "PLAYER1_INFO"
  graph.priority = 200
  graph.posY = 105
  graph.posX = -graph.image.width
  anim = animation()
  anim.name = graph.name
  anim.priority = 100
  anim.data = [10, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  #Info player 2:
  graph = drawPlayerInfo(globalVars, formVars.get("Player_2_info"), "gm_bg_blplayerinfo.png")
  graph.name = "PLAYER2_INFO"
  graph.priority = 200
  graph.posY = 105
  graph.posX = 480
  anim = animation()
  anim.name = graph.name
  anim.priority = 100
  anim.data = [480 - graph.image.width - 10, graph.posY, 99]
  anim.code = functions.moveFromTo
  graph.animation = anim
  formGraphics.set(graph)

  formVars.set(None, "formMenu")
  formVars.set("VIEWRULES", "status")


def hideMatchRules():
  """Hide match rules"""
  formVars.set(formGraphics.get("RULESINFO").data[0], "status")
  globalVars.set(formGraphics.get("RULESINFO").data[1], "message")

  delGraphic(formGraphics, "RULESINFO")
  delGraphic(formGraphics, "RULESINFOTITLE")
  delGraphic(formGraphics, "RULESINFOTITLET")
  delGraphic(formGraphics, "PLAYER1_INFO")
  delGraphic(formGraphics, "PLAYER2_INFO")
  if formVars.get("gameType").type == "SINGLE":
    formVars.set(inGameMenu(), "formMenu")
  elif formVars.get("gameType").type == "MULTI":
    formVars.set(inGameMenuMulti(), "formMenu")
  formVars.get("formMenu").createGraphicElements("BOTTOM")


def button_down(pbuttons):
  #Gestione tasti form TripleTriad:
  if formVars.get("status") == "WAITEXIT":
    #Torno al gioco:
    graph = graphicElement()
    graph.name = "BACK_TO_GAME"
    anim = animation()
    anim.name = graph.name
    anim.priority = -200
    def code(globalVars, formVars, target):
      #Scarico il font:
      globalVars.get("fonts").remove("fontResult")
      #Esco dalla stanza:
      globalVars.get("net").leaveRoom(formVars.get("gameType").roomID)
      #Cambio lo status:
      globalVars.get("net").changeStatus("N")
      #Ritorno al gioco:
      target.name = "DEL_" + target.name
      globalVars.set(["RESTORE"], "formParams")
      globalVars.set("mdlGame", "formName")
      globalVars.set("YES", "breakLoop")
    anim.code = code
    graph.animation = anim
    formGraphics.set(graph)
    return True

  #Nascondo le regole:
  if formVars.get("status") == "VIEWRULES":
    hideMatchRules()
    return True

  if buttons.start in pbuttons:
    #Confermo il trading:
    if formVars.get("status") == "TRADING" and len(formVars.get("choosenCards")) == formVars.get("toBeTraded"):
      #Controllo se ho scelto tutte le carte:
      if len(formVars.get("choosenCards")) == formVars.get("toBeTraded"):
        globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
        globalVars.get("playingSound").start()
        confirmTrade()
      else:
        globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
        globalVars.get("playingSound").start()
    #Visualizzo il menu:
    elif formVars.get("status") != "MENU" and formVars.get("gameType").type == "SINGLE":
      formVars.set([formVars.get("status"), globalVars.get("message")], "data")
      formVars.set("MENU", "status")
      #Nascondo il selector:
      graph = formGraphics.get("CARD_SELECTOR")
      if graph:
        graph.visible = False
      formVars.set(inGameMenu(), "formMenu")
      formVars.get("formMenu").createGraphicElements("BOTTOM")
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
    elif formVars.get("status") != "MENU" and formVars.get("gameType").type == "MULTI":
      formVars.set([formVars.get("status"), globalVars.get("message")], "data")
      formVars.set("MENU", "status")
      #Nascondo il selector:
      graph = formGraphics.get("CARD_SELECTOR")
      if graph:
        graph.visible = False
      graph = formGraphics.get("TRADE_SELECTOR")
      if graph:
        graph.visible = False
      formVars.set(inGameMenuMulti(), "formMenu")
      formVars.get("formMenu").createGraphicElements("BOTTOM")
      formVars.set("MENU", "status")
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
    return True
  #Selezione carta:
  if formVars.get("status") == "SELECTING":
    if buttons.down in pbuttons:
      #Seleziono la carta successiva:
      selectNextCardP1()
      return True
    elif buttons.up in pbuttons:
      #Seleziono la carta precedente:
      selectPreviousCardP1()
      return True
    elif buttons.cross in pbuttons:
      #Seleziono la carta:
      selectCardP1()
      return True
  #Posizionamento carta:
  elif formVars.get("status") == "POSITIONING":
    if buttons.down in pbuttons:
      MoveCardP1Down()
      return True
    elif buttons.up in pbuttons:
      MoveCardP1Up()
      return True
    elif buttons.left in pbuttons:
      MoveCardP1Left()
      return True
    elif buttons.right in pbuttons:
      MoveCardP1Right()
      return True
    elif buttons.circle in pbuttons:
      #Annullo il posizionamento:
      abortCardP1()
      return True
    elif buttons.cross in pbuttons:
      playerMove()
      return True
  #Trading delle carte:
  elif formVars.get("status") == "TRADING":
    if buttons.right in pbuttons:
      #Seleziono la carta successiva:
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      selectNextTradingCard()
      return True
    elif buttons.left in pbuttons:
      #Seleziono la carta successiva:
      globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
      globalVars.get("playingSound").start()
      selectPreviousTradingCard()
      return True
    elif buttons.cross in pbuttons:
      sel = formGraphics.get("TRADE_SELECTOR")
      if sel.data - 1 in formVars.get("choosenCards"):
        #Deseleziono la carta per il trading:
        globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
        globalVars.get("playingSound").start()
        deselectTradingCard()
      else:
        #Controllo se posso selezionarla:
        if len(formVars.get("choosenCards")) >= formVars.get("toBeTraded"):
          globalVars.set(globalVars.get("sounds").get("menuCancelSound"), "playingSound")
          globalVars.get("playingSound").start()
        else:
          #Seleziono la carta per il trading:
          globalVars.set(globalVars.get("sounds").get("menuSelectSound"), "playingSound")
          globalVars.get("playingSound").start()
          selectTradingCard()
      return True
  #Partita finita:
  elif formVars.get("status") == "ENDED":
    #Visualizzo il menu:
    formVars.set([formVars.get("status")], "data")
    formVars.set(inGameMenu(), "formMenu")
    formVars.set("MENU", "status")
    formVars.get("formMenu").createGraphicElements("BOTTOM")
    return True
